unit DemoMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls, ComCtrls,
  xorDX8;

type
  TForm1 = class(TForm)
    PanelOut: TPanel;
    Panel2: TPanel;
    Label1: TLabel;
    CheckSinus: TCheckBox;
    CheckParable: TCheckBox;
    CheckCylinder: TCheckBox;
    CheckSphere: TCheckBox;
    CheckRect: TCheckBox;
    Label2: TLabel;
    Label3: TLabel;
    CheckCandle: TCheckBox;
    CheckSpot1: TCheckBox;
    CheckSpot2: TCheckBox;
    CheckSun: TCheckBox;
    ButtonLightsOn: TButton;
    ButtonLightsOff: TButton;
    CheckFog: TCheckBox;
    CheckTransparency: TCheckBox;
    RadioCam2: TRadioButton;
    RadioCam1: TRadioButton;
    Label5: TLabel;
    CheckSceneRotation: TCheckBox;
    CheckGlowingCylinder: TCheckBox;
    CheckMouseInvertY: TCheckBox;
    CheckMouseInvertX: TCheckBox;
    RadioCamTransform: TRadioGroup;
    Memo1: TMemo;
    CheckAutoMove: TCheckBox;
    ButtonSetStandard: TButton;
    CheckMetalicSinus: TCheckBox;
    ButtonShowHideMouse: TButton;
    CheckPointAdapter: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SettingsClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    Scene: TxorDXScene;
    Camera1: TxorDXCamera;
    Camera2: TxorDXFreeCam;
    Candle: TxorDXPointLight;
    Cylinder: TxorDXPolygon;
    Fog: TxorDXFog;
    Font: TxorDXFont;
    Keyboard: TxorDXKeyboard;
    Mouse: TxorDXMouse;
    Parable: TxorDXIndexedPolygon;
    PointAdapter: TxorDXPointAdapter;
    Rect: TxorDXRect;
    Shadow: TxorDXShadow;
    Sinus: TxorDXMap;
    Sphere: TxorDXGridMap;
    Spot1: TxorDXSpotLight;
    Spot2: TxorDXSpotLight;
    Sun: TxorDXDirectionalLight;
    Texture: TxorDXTexture;
    Texture2: TxorDXTexture;
    Transparency: TxorDXTransparency;
    procedure Draw(Sender: TObject);
    procedure StopFlicker(var Msg: TWMEraseBkgnd); message WM_ERASEBKGND;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses Math, directxgraphics;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var
  MaxVertices: Cardinal;
begin
  try
    xorDXThrowAllOtherExceptions := True;
    xorDXThrowCameraTransformationExceptions := False;
    xorDXThrowIndexSizeException := True;
    xorDXThrowCapsException := True;
    InitRotationPerSecond;
    //creations
    if MessageDlg('Use in windowed mode? (no fullscreen)', mtConfirmation, [mbYes, mbNo], 0) = mrNo
    then Scene := TxorDXScene.CreateFullScreen(Handle)
    else Scene := TxorDXScene.CreateWindowed(Handle);
    Scene.OutputHandle := PanelOut.Handle;
    Camera1 := TxorDXCamera.Create(Scene);
    Camera1.SetStandard;
    Camera1.Ratio := PanelOut.Width / PanelOut.Height;
    Camera2 := TxorDXFreeCam.Create(Scene);
    Camera2.SetStandard;
    Camera2.Ratio := Camera1.Ratio;
    Candle := TxorDXPointLight.Create(Scene);
    Candle.SetDiffuseColor(0, 200, 0, 0);
    Candle.Attenuation0 := 0.5;
    Candle.Attenuation1 := 0.3;
    Candle.Attenuation2 := 0.6;
    Candle.Position := GetVector(1, 0.6, 1.5);
    Candle.FixTransformation;
    Cylinder := TxorDXPolygon.Create(Scene, 100);
    Cylinder.SetStandard;
    Cylinder.RotateX(-0.6);
    Cylinder.FixTransformation;
    Cylinder.MaterialEmissiveColor := $007F7F7F;                                //self lightning
    Fog := TxorDXFog.Create(Scene);
    Fog.Deactivate;
    if Fog.RadialEnable then Fog.Radial := True;
    Font := TxorDXFont.Create(Scene, 'Times New Roman', 45, [fsBold, fsItalic]);
    Keyboard := TxorDXKeyboard.Create(Scene);
    if not Keyboard.Attached then ShowMessage('No keyboard attached.');
    Mouse := TxorDXMouse.Create(Scene);
    Mouse.Speed := 0.002;
    if not Mouse.Attached then ShowMessage('No mouse attached.');
    Parable := TxorDXIndexedPolygon.Create(Scene, 100, GetEntryCount(xorDXPrimTriangleList, 81));
    Parable.SetStandard;
    Rect := TxorDXRect.CreateParam(Scene, 15, 30, 2, 4, xorDXVertexSizeTexture2, xorDXVertexFlagTexture2);
    Rect.SetStandard;
    Sinus := TxorDXMap.Create(Scene, 50, 60);
    Sinus.SetStandard;
    Sinus.Move(3, -1, -2);
    Sinus.MaterialSpecularColor := clWhite;                                     //metallic surface
    Sphere := TxorDXGridMap.Create(Scene, 18, 18);
    Sphere.SetStandard;
    Spot1 := TxorDXSpotLight.Create(Scene);
    Spot1.Theta := 0.1;
    Spot1.Phi := 0.4;
    Spot1.DiffuseColor := clYellow;
    Spot1.Position := GetVector(3, 3.6, 6);
    Spot1.Direction := GetVector(-1.06, -1.6, -1.3);
    Spot1.Activate;
    Spot2 := TxorDXSpotLight.Create(Scene);
    Spot2.Theta := 0.4;
    Spot2.Phi := 0.7;
    Spot2.Direction := GetVector(-0.5, -0.8, -0.2);
    Spot2.FixTransformation;
    Sun := TxorDXDirectionalLight.Create(Scene);
    Sun.Direction := GetVector(-1, -1, -1);
    Sun.Activate;
    Texture := TxorDXTexture.Create(Scene);
    Texture.LoadFromFile('..\dali.bmp', False, clWhite, 0);
    Texture.Scale(1, 2, 1);
    Texture.SetTextureAdressU(xorDXTexAddrMIRROR);
    Texture.SetTextureAdressV(xorDXTexAddrMIRROR);
    Texture2 := TxorDXTexture.Create(Scene);
    Texture2.LoadFromFile('..\graustufe.bmp', False, clWhite, 0);
    Texture2.Stage := 1;
    Transparency := TxorDXTransparency.Create(Scene);
    Transparency.Deactivate;
    //get max vertexcount of all shadowed objects
    MaxVertices := Max(Max(Max(Max(Rect.VertexCount, Sphere.VertexCount), Sinus.VertexCount), Parable.VertexCount), Cylinder.VertexCount);
    PointAdapter := TxorDXPointAdapter.Create(Scene, MaxVertices);
    Shadow := TxorDXShadow.Create(Scene, MaxVertices);
    Shadow.Plane := GetPlaneXZ(-3);
    Shadow.LightPosition := GetVector(0, 10, 0);
    //create buffers after object creations (you can set vertex+indexbuffersize before to save memory for later creations of other objects)
    Scene.CreateBuffers;
    Scene.OnDraw := Draw;
    Scene.StartLooping;
  except on E: Exception do
    begin
    ShowMessage(E.Message);
    FormDestroy(Self);
    Application.Terminate;
    end;
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  if Assigned(Scene) then Scene.StopLooping;
  if Assigned(PointAdapter) then FreeAndNil(PointAdapter);
  if Assigned(Shadow) then FreeAndNil(Shadow);
  if Assigned(Transparency) then FreeAndNil(Transparency);
  if Assigned(Texture2) then FreeAndNil(Texture2);
  if Assigned(Texture) then FreeAndNil(Texture);
  if Assigned(Sun) then FreeAndNil(Sun);
  if Assigned(Spot2) then FreeAndNil(Spot2);
  if Assigned(Spot1) then FreeAndNil(Spot1);
  if Assigned(Sphere) then FreeAndNil(Sphere);
  if Assigned(Sinus) then FreeAndNil(Sinus);
  if Assigned(Rect) then FreeAndNil(Rect);
  if Assigned(Parable) then FreeAndNil(Parable);
  if Assigned(Mouse) then FreeAndNil(Mouse);
  if Assigned(Keyboard) then FreeAndNil(Keyboard);
  if Assigned(Font) then FreeAndNil(Font);
  if Assigned(Fog) then FreeAndNil(Fog);
  if Assigned(Cylinder) then FreeAndNil(Cylinder);
  if Assigned(Candle) then FreeAndNil(Candle);
  if Assigned(Camera2) then FreeAndNil(Camera2);
  if Assigned(Camera1) then FreeAndNil(Camera1);
  if Assigned(Scene) then FreeAndNil(Scene);
end;

procedure TForm1.Draw(Sender: TObject);
var
  Cam: TxorDXCustomCamera;  R: TRect;  i: Integer;
begin
  with Candle do
    begin
    UndoTransformation;
    Move(0.5 * Cos(RotationsPerSecond(1)), 0, 0.5 * sin(RotationsPerSecond(1)));
    if CheckCandle.Checked then Activate else Deactivate;
    end;

  with Spot2 do
    begin
    UndoTransformation;
    RotateX(0.4 * cos(RotationsPerSecond(0.4)));
    RotateY(RotationsPerSecond(0.4));
    Position := GetVector(5, 2.5, -0.5);
    if CheckSpot2.Checked then Activate else Deactivate;
    end;

  if RadioCam1.Checked then Cam := Camera1 else Cam := Camera2;
  with Cam do
    begin
    if Keyboard.Get then with Keyboard do if KeyPressedAny then
      begin
      CheckAutoMove.Checked := False;
      if KeyPressed(VK_ESCAPE) then Application.Terminate;
      if KeyPressed(Ord('A')) then MoveHoriz(-0.1, RadioCamTransform.ItemIndex);
      if KeyPressed(Ord('D')) then MoveHoriz(0.1, RadioCamTransform.ItemIndex);
      if KeyPressed(Ord('W')) then MoveForward(0.1, RadioCamTransform.ItemIndex);
      if KeyPressed(Ord('S')) then MoveForward(-0.1, RadioCamTransform.ItemIndex);
      if KeyPressed(Ord('T')) then MoveVert(0.1, RadioCamTransform.ItemIndex);
      if KeyPressed(Ord('B')) then MoveVert(-0.1, RadioCamTransform.ItemIndex);
      end;
    if CheckAutoMove.Checked then
      begin
      if RadioCam1.Checked then
        MoveTo(8 * cos(RotationsPerSecond(0.07)), 5 * sin(RotationsPerSecond(0.06)), 2.5 * sin(RotationsPerSecond(0.03)), False)
      else RotateAround(2,1,0, 1,-1,1, 0.01);
      Mouse.Get;
      end
    else
      if Mouse.Get then
        begin
        RotateHoriz(Mouse.DX, RadioCamTransform.ItemIndex);
        RotateVert(-Mouse.DY, RadioCamTransform.ItemIndex);
        if Mouse.ClickedRight then RotateSlope(0.02, RadioCamTransform.ItemIndex);
        end;
    Activate;
    end;

  with Scene do
    begin
    Clear;
    if CheckSceneRotation.Checked then
      begin
      UndoTransformation;
      RotateY(RotationsPerSecond(0.3));
      end;
    end;

  with Sphere do
    begin
    UndoTransformation;
    Insert;
    MoveCenter(True);
    RotateX(RotationsPerSecond(0.1));
    Move(-2, 2, -1);
    Draw;
    Move(0, 0, -1.4);
    Draw;
    end;

  with Parable do
    begin
    UndoTransformation;
    Insert;
    RotateZ(RotationsPerSecond(0.1));
    Move(0, 3, 3);
    Draw;
    end;

  with PointAdapter do
    begin
    UndoTransformation;
    Insert;
    RotateZ(RotationsPerSecond(0.1));
    Move(0, 4, 5); 
    Draw;
    end;

  with Rect do
    begin
    UndoTransformation;
    Insert;
    RotateX(Pi05);
    Draw;
    RotateZ(-Pi05);
    Move(0, RectWidth, 0);
    //texture with ColorStageBlending
    Texture2.ColorStageBlendTextureAndCurrent(bop2X);
//    Texture2.ColorStageBlendTextureOnly;
    Texture2.Activate;
    Texture.ColorStageBlendTextureAndDiffuseColor(bop1X);
    Texture.RotateZ(0.01);
    Texture.Activate;
    Draw;
    Texture2.Deactivate;
    Texture2.ColorStageBlendDisable;
    Texture.Deactivate;
    Texture.ColorStageBlendDisable;
    end;

  with Cylinder do
    begin
    Insert;
    SetStandard;                                                                //cylinder with normal x-y-z-vertexvalues
    for i := 0 to VertexCount - 1 do
      with PxorDXVertexClassic(Vertex[i])^ do
        Position.x := Position.x + 0.3 * cos(Pi * i / VertexCount) * sin(RotationsPerSecond(1)); //stratching body of cylinder
    UndoTransformation;
    RotateY(RotationsPerSecond(0.2));
    Draw;
    end;

  with Sinus do
    begin
    Insert;
    SetStandard; //waving
    Draw;
    end;

  with Shadow do
    begin
    Insert(Cylinder);  Visible := Cylinder.Visible;  Draw;
    Insert(Parable);  Visible := Parable.Visible;  Draw;
    Insert(Sinus);  Visible := Sinus.Visible;  Draw;
    Insert(Sphere);  Visible := Sphere.Visible;  Draw;
    end;

  with Font do
    begin
    R := Classes.Rect(0, 0, Scene.ScreenWidth, Scene.ScreenHeight);
    R.Left := (Scene.DrawCalls * 3) mod R.Right;
    TextOut(R, xorDXFontBottom or xorDXFontNoClip, clGreen xor + (R.Left * 256 div R.Right), 255, 'Use ESC to quit xorDX8-example!');
    end;
end;

procedure TForm1.SettingsClick(Sender: TObject);
begin
  if Sender = CheckCylinder then Cylinder.Visible := CheckCylinder.Checked;
  if Sender = CheckParable then Parable.Visible := CheckParable.Checked;
  if Sender = CheckSinus then Sinus.Visible := CheckSinus.Checked;
  if Sender = CheckSphere then Sphere.Visible := CheckSphere.Checked;
  if Sender = CheckRect then Rect.Visible := CheckRect.Checked;
  if Sender = CheckPointAdapter then PointAdapter.Visible := CheckPointAdapter.Checked;
  if Sender = CheckFog then if CheckFog.Checked then Fog.Activate else Fog.Deactivate;
  if Sender = CheckTransparency then if CheckTransparency.Checked then Transparency.Activate else Transparency.Deactivate;
  if Sender = CheckSpot1 then if CheckSpot1.Checked then Spot1.Activate else Spot1.Deactivate;
  if Sender = CheckSun then if CheckSun.Checked then Sun.Activate else Sun.Deactivate;
  if Sender = RadioCam1 then RadioCam2.Checked := not RadioCam1.Checked;
  if Sender = RadioCam2 then RadioCam1.Checked := not RadioCam2.Checked;
  if Sender = CheckGlowingCylinder then Cylinder.MaterialEmissiveColor := ($007F7F7F) * Ord(CheckGlowingCylinder.Checked);
  if Sender = CheckMetalicSinus then Sinus.MaterialSpecularColor := clWhite * Ord(CheckMetalicSinus.Checked);
  if Sender = ButtonLightsOn then Scene.LightsOn := True;
  if Sender = ButtonLightsOff then Scene.LightsOn := False;
  if Sender = CheckMouseInvertX then Mouse.InvertX := CheckMouseInvertX.Checked;
  if Sender = CheckMouseInvertY then Mouse.InvertY := CheckMouseInvertY.Checked;
  if Sender = ButtonSetStandard then begin Camera1.SetStandard; Camera2.SetStandard end;
  if Sender = ButtonShowHideMouse then Mouse.Visible := not Mouse.Visible;
end;

procedure TForm1.StopFlicker(var Msg: TWMEraseBkgnd);
begin
  Msg.Result := LRESULT(False);
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  if Assigned(Camera1) and Assigned(Camera2) then
    begin
    Camera1.Ratio := PanelOut.Width / PanelOut.Height;
    Camera2.Ratio := Camera1.Ratio;
    if RadioCam1.Checked then Camera1.Activate else Camera2.Activate;
    end;
end;

end.
