unit DrawModeMain;

interface

uses
  SysUtils, Forms, Graphics,
  xorDX8;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    Scene: TxorDXScene;
    Camera: TxorDXCamera;
    Polygon: TxorDXPolygon;
    Keyboard: TxorDXKeyboard;
    procedure Draw(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  xorDXThrowAllOtherExceptions := True;
  xorDXThrowCameraTransformationExceptions := False;
  xorDXThrowIndexSizeException := True;
  xorDXThrowCapsException := True;
  //creation
  Scene := TxorDXScene.Create(Handle);
  Camera := TxorDXCamera.Create(Scene);
  Polygon := TxorDXPolygon.Create(Scene, 6);
  Keyboard := TxorDXKeyboard.Create(Scene);
  Scene.CreateBuffers;

  //settings & start
  with Polygon do
    begin
    PxorDXVertexClassic(Vertex[0])^ := GetVertexClassic(0,0,0,    0,0,0, $0000A0A0, 0);
    PxorDXVertexClassic(Vertex[1])^ := GetVertexClassic(0,1,0,    0,0,0, $008040F0, 0);
    PxorDXVertexClassic(Vertex[2])^ := GetVertexClassic(1,0,0.4,  0,0,0, $00A00000, 0);
    PxorDXVertexClassic(Vertex[3])^ := GetVertexClassic(1,1,0.4,  0,0,0, $0000A000, 0);
    PxorDXVertexClassic(Vertex[4])^ := GetVertexClassic(2,0,0,    0,0,0, $000000A0, 0);
    PxorDXVertexClassic(Vertex[5])^ := GetVertexClassic(2,1,0,    0,0,0, $00000000, 0);
    end;
  Camera.SetStandard;
  Camera.Position := GetVector(0, 0, -3.3);
  Camera.Ratio := Width / Height;
  Camera.Activate;
  Scene.OnDraw := Draw;
  Scene.StartLooping;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  //free in inverted order of creation
  FreeAndNil(Keyboard);
  FreeAndNil(Polygon);
  FreeAndNil(Camera);
  FreeAndNil(Scene);
end;

//repeated drawing routine
procedure TForm1.Draw(Sender: TObject);
const
  DrawModeStr: Array[1..6] of String[28] = ('PointList (%d points)', 'LineList (%d lines)', 'LineStrip (%d lines)', 'TriangleList (%d triangles)', 'TriangleStrip (%d triangles)', 'TriangleFan (%d triangles)');
begin
  Scene.Clear;
  with Keyboard do
    if Get and KeyPressedAny then
      begin
      if KeyPressed(Ord('1')) then Polygon.DrawMode := xorDXDrawPointList;
      if KeyPressed(Ord('2')) then Polygon.DrawMode := xorDXDrawLineList;
      if KeyPressed(Ord('3')) then Polygon.DrawMode := xorDXDrawLineStrip;
      if KeyPressed(Ord('4')) then Polygon.DrawMode := xorDXDrawTriangleList;
      if KeyPressed(Ord('5')) then Polygon.DrawMode := xorDXDrawTriangleStrip;
      if KeyPressed(Ord('6')) then Polygon.DrawMode := xorDXDrawTriangleFan;
      end;

  with Polygon do
    begin
    Caption := 'xorDX - Example   DrawMode = xorDXDraw' + Format(DrawModeStr[DrawMode], [DrawPrimCount]) + '    (Use 1..6 to change!)';
    Insert;
    RotateY(0.0035);
    Draw;
    end;
end;

end.
