unit KeyAndMouseMain;

interface

uses
  SysUtils, Forms, Dialogs, Windows,
  xorDX8;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private-Deklarationen }
    CamTrans: TxorDXCameraTransformation; //kind of camera transformation
  public
    { Public-Deklarationen }
    Scene: TxorDXScene;
    Camera: TxorDXCamera;
    Rect: TxorDXRect;
    Mouse: TxorDXMouse;
    Keyboard: TxorDXKeyboard;
    procedure Draw(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  xorDXThrowAllOtherExceptions := True;
  xorDXThrowCameraTransformationExceptions := False;
  xorDXThrowIndexSizeException := True;
  xorDXThrowCapsException := True;
  //creation
  Scene    := TxorDXScene.Create(Handle);
  Camera   := TxorDXCamera.Create(Scene);
  Rect     := TxorDXRect.Create(Scene, 30, 15, 4, 2); //set standard coordinates and colors
  Mouse    := TxorDXMouse.Create(Scene);
  Keyboard := TxorDXKeyboard.Create(Scene);
  Scene.CreateBuffers;
  //start
  CamTrans := xorDXCamHuman;
  Camera.SetStandard;
  Camera.Activate;
  Scene.OnDraw := Draw; //set draw routine
  Scene.StartLooping;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  //free in inverted order of creation
  FreeAndNil(Keyboard);
  FreeAndNil(Mouse);
  FreeAndNil(Rect);
  FreeAndNil(Camera);
  FreeAndNil(Scene);
end;

//repeated drawing routine
procedure TForm1.Draw(Sender: TObject);
const
  CamTransStr: Array[0..3] of String[5] = ('Human','ID','Scene','Up');
begin
  Scene.Clear;

  if Mouse.Get then //read mouse buffer
    with Camera do
      begin
      RotateHoriz(Mouse.DX/600, CamTrans);
      RotateVert(-Mouse.DY/600, CamTrans);
      if Mouse.ClickedLeft then UndoTransformation;
      end;

  with Keyboard, Camera do
    if Get and KeyPressedAny then //read keyboard buffer
      begin
      //use windows "virtual keycodes"
      if KeyPressed(VK_ESCAPE) then Application.Terminate;
      if KeyPressed(VK_LEFT) then MoveHoriz(-0.03, CamTrans);
      if KeyPressed(VK_RIGHT) then MoveHoriz(0.03, CamTrans);
      if KeyPressed(VK_UP) then MoveForward(0.03, CamTrans);
      if KeyPressed(VK_DOWN) then MoveForward(-0.03, CamTrans);
      if KeyPressed(VK_PRIOR) then MoveVert(0.03, CamTrans);
      if KeyPressed(VK_NEXT) then MoveVert(-0.03, CamTrans);
      if KeyPressed(Ord('1')) then CamTrans := xorDXCamHuman;
      if KeyPressed(Ord('2')) then CamTrans := xorDXCamID;
      if KeyPressed(Ord('3')) then CamTrans := xorDXCamScene;
      if KeyPressed(Ord('4')) then CamTrans := xorDXCamUp;
      end;

  Camera.Activate; //set camera changings
  Caption := 'xorDX - Example    Camera-Transformation = xorDXCam'
             + CamTransStr[CamTrans]
             + '   (press 1..4 to change)          Use CURSOR, PAGE and ESC keys and mouse!';

  Rect.Insert;
  Rect.Draw;
end;

end.
