unit PolygonMain;

interface

uses
  SysUtils, Forms, Graphics,
  xorDX8;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    Scene: TxorDXScene;
    Camera: TxorDXCamera;
    Polygon: TxorDXPolygon;
    procedure Draw(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

{ TForm1 }
        
procedure TForm1.FormCreate(Sender: TObject);
begin
  xorDXThrowAllOtherExceptions := True;
  xorDXThrowCameraTransformationExceptions := False;
  xorDXThrowIndexSizeException := True;
  xorDXThrowCapsException := True;
  //creation
  Scene := TxorDXScene.Create(Handle);
  Camera := TxorDXCamera.Create(Scene);
  Polygon := TxorDXPolygon.Create(Scene, 6);
  Scene.CreateBuffers;

  //settings & start
  with Polygon do
    begin
    DrawMode := xorDXPrimTriangleList; //each 3 vertices form a triangle
    //triangle 1
    PxorDXVertexClassic(Vertex[0])^ := GetVertexClassic(-0.5,0,-0.3,  0,0,0, clBlack, 0);
    PxorDXVertexClassic(Vertex[1])^ := GetVertexClassic(1,1,0.4,      0,0,0, clBlue, 0);
    PxorDXVertexClassic(Vertex[2])^ := GetVertexClassic(1,-1,0.4,     0,0,0, clRed, 0);
    //triangle 2
    PxorDXVertexClassic(Vertex[3])^ := GetVertexClassic(0.5,0,-0.3,   0,0,0, clBlack, 0);
    PxorDXVertexClassic(Vertex[4])^ := GetVertexClassic(-1,1,0.4,     0,0,0, clYellow, 0);
    PxorDXVertexClassic(Vertex[5])^ := GetVertexClassic(-1,-1,0.4,    0,0,0, clGreen, 0);
    MoveCenter(True); //move to 0
    FixTransformation;
    end;
  Camera.SetStandard;
  Camera.Position := GetVector(0, 0, -3.3);
  Camera.Ratio := Width / Height;
  Camera.Activate;
  Scene.OnDraw := Draw;
  Scene.StartLooping;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  //free in inverted order of creation
  FreeAndNil(Polygon);
  FreeAndNil(Camera);
  FreeAndNil(Scene);
end;

//repeated drawing routine
procedure TForm1.Draw(Sender: TObject);
begin
  Scene.Clear;
  with Polygon do
    begin //this polygon is drawn twice
    Insert;                   //get vertex coordinates
    UndoTransformation;       //undo transformation until last fix transformation
    RotateX(0.0075);          //rotate around x-axis
    FixTransformation;        //fix this transformation --> start next time with that rotation
    Move(-2, 0, 0);           //move left
    Draw;                     //render
    UndoTransformation;       //move to 0
    RotateZ(Pi05) ;           //rotate additional around y-axis
    Move(2,0,0);              //move right
    Draw;                     //render
    end;
end;

end.
