unit xorDX8;

{******************************************************************************}
{*                 ______________________________________                     *}
{*            ---  xorDX8 : DirectX 8.1 Delphi Interface  ---                 *}
{*                                       *}
{*                  Tools and Classes for using DirectX                       *}
{*                                                                            *}
{******************************************************************************)
{*                                                                            *}
{*  Developer: XOR (Sven Hanke)                                               *}
{*  Mail:      mustermann@spakkenknorke.de                                    *}
{*  Home:      www.informatik.hu-berlin.de/~hanke                             *}
{*  Version:   30-July-2004                                                   *}
{*  Uses:      1. DirectX 8.1b                                                *}
{*                www.microsoft.com/downloads/results.aspx?productID=9C954C37-1ED1-4846-8A7D-85FC422D1388&freetext=8.1b&DisplayLang=en *}
{*             2. DirectX 8 Delphi adaptation by Tim Baumgarten               *}
{*                Home: http://www.ampaze.de/dx/eight/                        *}
{*                Mail: ampaze at gmx dot net                                 *}
{*                                                                            *}
{******************************************************************************)
{*  __________                                                                *}
{*  CARD WARE:  Free use, but you have to send a post card to me.             *}
{*                                                                   *}
{*  (C) by XOR (Sven Hanke). All rights reserved.                             *}
{*  Its allowed to hand down the unmodified unit.                             *}
{*                                                                            *}
{*  Last address: (Please ask for current address per mail!)                  *}
{*  Sven Hanke                                                                *}
{*  Aronsstr. 77                                                              *}
{*  12057 Berlin                                                              *}
{*  Germany                                                                   *}
{*                                                                            *}
{*  Write me, how you find out this unit, like my page, criticism, greetings. *}
{*  Or send me your project per mail.                                         *}
{*                                                                            *}
{******************************************************************************}

{$TYPEDADDRESS OFF} //@ returns an untyped pointer
{$BOOLEVAL OFF} //short boolean interpretations

interface

uses
  Windows, Dialogs, Classes, Forms, Graphics, SysUtils,
  DirectXGraphics, D3DX81mo, DirectInput8;

//==============================================================================
{ Exception behaviour and error messages }

const
  xorDXThrowAllOtherExceptions: Boolean = False; // Exceptions on insert, drawing or settings will be thrown
  xorDXThrowCameraTransformationExceptions: Boolean = False; // Exception on camera transformations, especially if you look upward
  xorDXThrowIndexSizeException: Boolean = True; // if you use a 16-bit-index but more than 65536 (0..65535) vertices
  xorDXThrowCapsException: Boolean = False; // if you use values out of range of your graphic adapter

resourcestring
  errAssignedBuffer =     'Operation not possible after buffer creation.';
  errBadArgument =        'No valid argument.';
  errBadCameraAngle =     'Angle between Camera Direction and Up-Vector would be to small.';
  errBadDrawMode =        'Operation dont work together with that draw mode.';
  errBadVertexFormat =    'Operation dont work together with that vertex format.';
  errBufferCreation =     'Cant create buffer.';
  errBufferLock =         'Cant lock/unlock buffer.';
  errCamLightScale =      'Scaling of cameras and lights is not possible.';
  errCamDeactivate =      'Camera is not active, cant be deactivated.';
  errCaps =               'Couldnt read device-caps.';
  errCaps32Bit =          '32Bit-Index is not supported. (device caps: max vertex index)';
  errCapsFogRange =       'Range fog is not supported. (device caps)';
  errCapsLights =         'Count of lights is not supported. (device caps)';
  errCapsNotRead =        'Caps not initialized, couldnt read caps. (set xorDXThrowCapsException true before scene creation)';
  errCapsPrimCount =      'Count of draw primitives is not supported. (device caps: max primitive count)';
  errCapsStage =          'Count of stages or simultaneous textures is not supported.';
  errCapsVertexCount =    'Count of vertices not supported. (device-caps: max vertex index)';
  errCapsVertexSize =     'Size of vertex not supported (device caps: max stream stride)';
  errClearDevice =        'Cant clear device.';
  errDeviceCreate =       'Cant create main device.';
  errDeviceLostReset =    'Cant do OnLostDevice/OnResetDevice.';
  errDisplayFormat =      'Display/Backbuffer-Format is not supported.';
  errDisplaySettings =    'Cant read display settings.';
  errDrawing =            'Cant draw.';
  errFontCreate =         'Cant create font device.';
  errFontShow =           'Cant show font.';
  errFormatBuffer =       'Buffer format cant be changed after creation of indexed objects.';
  errIndexSize =          'More than 65536 vertices require a 32-Bit-Index.';
  errIndices =            'Cant set indices.';
  errInput =              'Device is not attached.';
  errInputAcquire =       'Cant acquire input device.';
  errInputCaps =          'Cant read input device capabilities.';
  errInputCooperation =   'Cant set input cooperation level.';
  errInputAccess =        'Operation is a fullscreen-operation.';
  errInputCreate =        'Cant create input device.';
  errInputDataFormat =    'Cant set input data format.';
  errInputDeviceState =   'Cant read device state.';
  errInputKeyboard =      'Cant create keyboard device.';
  errInputMouse =         'Cant create mouse device.';
  errInputUnacquire =     'Cant unacquire input device.';
  errInterfaceCreate =    'Cant create interface.';
  errLightMatSet =        'Cant set light/material properties.';
  errNoClassicStructure = 'Inserted Polygon has no classic vertex format.';
  errNoOnDraw =           'No OnDraw-event found.';
  errNoPolygon =          'No polygon inserted before shadow/pointadapter.';
  errNotLastInserted =    'No insert method before draw method.';
  errRenderState =        'Cant read/set render state.';
  errScene =              'Cant begin/end/present scene.';
  errStreamSource =       'Cant set stream source.';
  errTextureActivate =    'No texture loaded, use LoadFromFile before.';
  errTextureActive =      'Operation is not possible because texture is active in some stage, use Deactivate with this stage before.';
  errTextureConflict =    'Any other texture ist active on this stage.';
  errTextureDeactivate =  'Texture is not active on this stage, couldnt be deactivated.';
  errTextureLoad =        'Cant load texture.';
  errTextureOverload =    'Other texture is loaded and active in some stage, use Deactivate and Unload before loading new texture.)';
  errTextureSet =         'Cant set texture to the stage.';
  errTextureState =       'Cant read/set texture state.';
  errTextureTransform =   'TextureTransformation is 2-dimensional, operation or parameter not possible.';
  errTransform =          'Cant get/set transformation.';
  errVertexShader =       'Cant set vertex shader.';

type
  TxorDXErrorMessage = function(Value: HResult): String;
  TxorDXDrawException = procedure(Msg: String) of object;

const
  GetErrorMsgD3D: TxorDXErrorMessage = DXGErrorString;
  GetErrorMsgInput: TxorDXErrorMessage = DIErrorString;

//==============================================================================
{ Imported types and constants }

//Anti-aliasing value
type
  TxorDXMultiSampleType = TD3DMultiSampleType;

const
  xorDXMultiSample_None = D3DMULTISAMPLE_NONE;
  xorDXMultiSample_2_Samples = D3DMULTISAMPLE_2_SAMPLES;
  xorDXMultiSample_3_Samples = D3DMULTISAMPLE_3_SAMPLES;
  xorDXMultiSample_4_Samples = D3DMULTISAMPLE_4_SAMPLES;
  xorDXMultiSample_5_Samples = D3DMULTISAMPLE_5_SAMPLES;
  xorDXMultiSample_6_Samples = D3DMULTISAMPLE_6_SAMPLES;
  xorDXMultiSample_7_Samples = D3DMULTISAMPLE_7_SAMPLES;
  xorDXMultiSample_8_Samples = D3DMULTISAMPLE_8_SAMPLES;
  xorDXMultiSample_9_Samples = D3DMULTISAMPLE_9_SAMPLES;
  xorDXMultiSample_10_Samples = D3DMULTISAMPLE_10_SAMPLES;
  xorDXMultiSample_11_Samples = D3DMULTISAMPLE_11_SAMPLES;
  xorDXMultiSample_12_Samples = D3DMULTISAMPLE_12_SAMPLES;
  xorDXMultiSample_13_Samples = D3DMULTISAMPLE_13_SAMPLES;
  xorDXMultiSample_14_Samples = D3DMULTISAMPLE_14_SAMPLES;
  xorDXMultiSample_15_Samples = D3DMULTISAMPLE_15_SAMPLES;
  xorDXMultiSample_16_Samples = D3DMULTISAMPLE_16_SAMPLES;

//Backbuffer interval
type
  TxorDXPresentationInterval = LongWord;

const
  xorDXPresentIntervalDefault = D3DPRESENT_INTERVAL_DEFAULT;
  xorDXPresentIntervalOne = D3DPRESENT_INTERVAL_ONE;
  xorDXPresentIntervalTwo = D3DPRESENT_INTERVAL_TWO;
  xorDXPresentIntervalThree = D3DPRESENT_INTERVAL_THREE;
  xorDXPresentIntervalFour = D3DPRESENT_INTERVAL_FOUR;
  xorDXPresentIntervalImmediate = D3DPRESENT_INTERVAL_IMMEDIATE;

//Backbuffer swap effect
type
  TxorDXSwapEffect = TD3DSwapEffect;

const
  xorDXSwapDiscard = D3DSWAPEFFECT_DISCARD;
  xorDXSwapFlip = D3DSWAPEFFECT_FLIP;
  xorDXSwapCopy = D3DSWAPEFFECT_COPY;
  xorDXSwapCopyVSync = D3DSWAPEFFECT_COPY_VSYNC;

//Blending
type
  TxorDXBlend = TD3DBlend;

const                   
  xorDXBlendBOTHINVSRCALPHA = D3DBLEND_BOTHINVSRCALPHA;
  xorDXBlendBOTHSRCALPHA = D3DBLEND_BOTHSRCALPHA;
  xorDXBlendDESTALPHA = D3DBLEND_DESTALPHA;
  xorDXBlendDESTCOLOR = D3DBLEND_DESTCOLOR;
  xorDXBlendFORCE_DWORD = D3DBLEND_FORCE_DWORD;
  xorDXBlendINVDESTALPHA = D3DBLEND_INVDESTALPHA;
  xorDXBlendINVDESTCOLOR = D3DBLEND_INVDESTCOLOR;
  xorDXBlendINVSRCALPHA = D3DBLEND_INVSRCALPHA;
  xorDXBlendINVSRCCOLOR = D3DBLEND_INVSRCCOLOR;
  xorDXBlendONE = D3DBLEND_ONE;
  xorDXBlendSRCALPHA = D3DBLEND_SRCALPHA;
  xorDXBlendSRCALPHASAT = D3DBLEND_SRCALPHASAT;
  xorDXBlendSRCCOLOR = D3DBLEND_SRCCOLOR;
  xorDXBlendZERO = D3DBLEND_ZERO;

//Color
type
  TxorDXColor = TD3DColor;
  TxorDXColorValue = TD3DColorValue;

function GetAlp(Col: TxorDXColor): LongWord;
function GetAlpVal(Col: TxorDXColor): Single;
function GetCol(Col: TColor): TxorDXColor;
function GetColA(Col: TColor; Alpha: LongWord): TxorDXColor;
function GetColS(Col: TColor; Alpha: Single): TxorDXColor;
function GetColRGB(r, g, b: LongWord): TxorDXColor;
function GetColRGBA(r, g, b, Alpha: LongWord): TxorDXColor;
function GetColRGBS(r, g, b: LongWord; Alpha: Single): TxorDXColor;
function GetColValA(r, g, b, Alpha: LongWord): TxorDXColorValue;
function GetColValSS(r, g, b, Alpha: Single): TxorDXColorValue;
function GetColValS(r, g, b: LongWord; Alpha: Single): TxorDXColorValue;
function GetTCol(Col: TxorDXColor): TColor; overload;
function GetTCol(Col: TxorDXColorValue): TColor; overload;
procedure SetAlp(var Col: TxorDXColor; Alpha: LongWord);
procedure SetAlpS(var Col: TxorDXColor; Alpha: Single);
procedure SetCol(var Col: TxorDXColor; TCol: TColor); overload;
procedure SetCol(var Col: TxorDXColorValue; TCol: TColor); overload;

//Cull mode = (drawing only clockwise directed triangles)
type
  TxorDXCull = TD3DCull;

const
  xorDXCullNone = D3DCULL_NONE;
  xorDXCullCW = D3DCULL_CW;
  xorDXCullCCW = D3DCULL_CCW;
  xorDXCullForceDWord = D3DCULL_FORCE_DWORD;

//Draw mode
type
  TxorDXPrimitiveType = TD3DPrimitiveType;                                      //kind of combining vertices
  TxorDXDrawMode = TxorDXPrimitiveType; //alternative name

const
  xorDXPrimPointList = D3DPT_POINTLIST;
  xorDXPrimLineList = D3DPT_LINELIST;
  xorDXPrimLineStrip = D3DPT_LINESTRIP;
  xorDXPrimTriangleList = D3DPT_TRIANGLELIST;
  xorDXPrimTriangleStrip = D3DPT_TRIANGLESTRIP;
  xorDXPrimTriangleFan = D3DPT_TRIANGLEFAN;
  //same values with alternativ name
  xorDXDrawPointList = xorDXPrimPointList;
  xorDXDrawLineList = xorDXPrimLineList;
  xorDXDrawLineStrip = xorDXPrimLineStrip;
  xorDXDrawTriangleList = xorDXPrimTriangleList;
  xorDXDrawTriangleStrip = xorDXPrimTriangleStrip;
  xorDXDrawTriangleFan = xorDXPrimTriangleFan;

function GetEntryCount(Mode: TxorDXPrimitiveType; PrimCount: Cardinal): Cardinal; //returns the minimal number of entries used for PrimCount primitives in this mode
function GetPrimCount(Mode: TxorDXPrimitiveType; EntryCount: Cardinal): Cardinal; //returns the number of primitives using EntryCount entries in this mode

//Filter
type
  TxorDXFilter = LongWord;

const
  xorDXFilterNONE = D3DX_FILTER_NONE;
  xorDXFilterPOINT = D3DX_FILTER_POINT;
  xorDXFilterLINEAR = D3DX_FILTER_LINEAR;
  xorDXFilterTRIANGLE = D3DX_FILTER_TRIANGLE;
  xorDXFilterBOX = D3DX_FILTER_BOX;
  xorDXFilterMIRRORU = D3DX_FILTER_MIRROR_U;
  xorDXFilterMIRRORV = D3DX_FILTER_MIRROR_V;
  xorDXFilterMIRRORW = D3DX_FILTER_MIRROR_W;
  xorDXFilterMIRROR = D3DX_FILTER_MIRROR;
  xorDXFilterDITHER = D3DX_FILTER_DITHER;

//Fog mode
type
  TxorDXFogMode = TD3DFogMode;                                                  //fog interpolation between adjacent vertices

const
  xorDXFogNONE = D3DFOG_NONE;                //= 0
  xorDXFogEXP = D3DFOG_EXP;                  //= 1
  xorDXFogEXP2 = D3DFOG_EXP2;                //= 2
  xorDXFogLINEAR = D3DFOG_LINEAR;            //= 3

//Font textout-formats: similiar to GDI's DrawText parameters (s. windows help)
type
  TxorDXFontFormat = LongWord;

const
  xorDXFontBottom = DT_BOTTOM;
  xorDXFontCalcRect = DT_CALCRECT;
  xorDXFontCenter = DT_CENTER;
  xorDXFontEditControl = DT_EDITCONTROL;
  xorDXFontEndEllipsis = DT_END_ELLIPSIS;
  xorDXFontPathEllipsis = DT_PATH_ELLIPSIS;
  xorDXFontExpandTabs = DT_EXPANDTABS;
  xorDXFontExternalLeading = DT_EXTERNALLEADING;
  xorDXFontLeft = DT_LEFT;
  xorDXFontModifyString = DT_MODIFYSTRING;
  xorDXFontNoClip = DT_NOCLIP;
  xorDXFontNoPrefix = DT_NOPREFIX;
  xorDXFontRight = DT_RIGHT;
  xorDXFontRtlReading = DT_RTLREADING;
  xorDXFontSingleLine = DT_SINGLELINE;
  xorDXFontTabStop = DT_TABSTOP;
  xorDXFontTop = DT_TOP;
  xorDXFontVCenter = DT_VCENTER;
  xorDXFontWordBreak = DT_WORDBREAK;

//Key codes:   windows-virtual-key-codes (vk_...) to directinput-key-codes
const
  VK2DIK : array[Byte] of Byte = (
  0, 0, 0, 0, 0, 0, 0, 0, DIK_BACK, DIK_TAB, 0, 0, 0, DIK_RETURN, 0, 0,
  DIK_RSHIFT, DIK_RCONTROL, DIK_RMenu, DIK_PAUSE, DIK_CAPITAL, 0, 0, 0, 0, 0, 0, DIK_ESCAPE, 0, 0, 0, 0,
  DIK_SPACE, DIK_PRIOR, DIK_NEXT, DIK_END, DIK_HOME, DIK_LEFT, DIK_UP, DIK_RIGHT, DIK_DOWN, 0, DIK_MULTIPLY, 0, DIK_COMMA , DIK_INSERT, DIK_DELETE, DIK_SLASH,
  DIK_0, DIK_1, DIK_2, DIK_3, DIK_4, DIK_5, DIK_6, DIK_7, DIK_8, DIK_9, DIK_COLON, DIK_SEMICOLON, 0, DIK_EQUALS, 0, 0,
  DIK_AT, DIK_A, DIK_B, DIK_C, DIK_D, DIK_E, DIK_F, DIK_G, DIK_H, DIK_I, DIK_J, DIK_K, DIK_L, DIK_M, DIK_N, DIK_O,
  DIK_P, DIK_Q, DIK_R, DIK_S, DIK_T, DIK_U, DIK_V, DIK_W, DIK_X, DIK_Y, DIK_Z, DIK_LBRACKET, DIK_BACKSLASH, DIK_RBRACKET, 0, DIK_UNDERLINE,
  DIK_NUMPAD0, DIK_NUMPAD1, DIK_NUMPAD2, DIK_NUMPAD3, DIK_NUMPAD4, DIK_NUMPAD5, DIK_NUMPAD6, DIK_NUMPAD7, DIK_NUMPAD8, DIK_NUMPAD9, 0, DIK_ADD, 0, DIK_SUBTRACT, DIK_DECIMAL, DIK_DIVIDE,
  DIK_F1, DIK_F2, DIK_F3, DIK_F4, DIK_F5, DIK_F6, DIK_F7, DIK_F8, DIK_F9, DIK_F10, DIK_F11, DIK_F12, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  DIK_NUMLOCK, DIK_SCROLL, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  DIK_LSHIFT, DIK_RSHIFT, DIK_LCONTROL, DIK_RCONTROL, DIK_LMenu, DIK_RMenu, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);

//Material source
type
  TxorDXMaterialSource = TD3DMaterialColorSource;

const
  xorDXMatMaterial = D3DMCS_MATERIAL;
  xorDXMatColor1 = D3DMCS_COLOR1;
  xorDXMatColor2 = D3DMCS_COLOR2;
  xorDXMatForceDWord = D3DMCS_FORCE_DWORD;

//Render states
type
  TxorDXRenderState = TD3DRenderStateType;

const                                      
  xorDXRendALPHABLENDENABLE = D3DRS_ALPHABLENDENABLE;
  xorDXRendALPHAFUNC = D3DRS_ALPHAFUNC;
  xorDXRendALPHAREF = D3DRS_ALPHAREF;
  xorDXRendALPHATESTENABLE = D3DRS_ALPHATESTENABLE;
  xorDXRendAMBIENT = D3DRS_AMBIENT;
  xorDXRendAMBIENTMATERIALSOURCE = D3DRS_AMBIENTMATERIALSOURCE;
  xorDXRendBLENDOP = D3DRS_BLENDOP;
  xorDXRendCLIPPING = D3DRS_CLIPPING;
  xorDXRendCLIPPLANEENABLE = D3DRS_CLIPPLANEENABLE;
  xorDXRendCOLORVERTEX = D3DRS_COLORVERTEX;
  xorDXRendCOLORWRITEENABLE = D3DRS_COLORWRITEENABLE;
  xorDXRendCULLMODE = D3DRS_CULLMODE;
  xorDXRendDEBUGMONITORTOKEN = D3DRS_DEBUGMONITORTOKEN;
  xorDXRendDESTBLEND = D3DRS_DESTBLEND;
  xorDXRendDIFFUSEMATERIALSOURCE = D3DRS_DIFFUSEMATERIALSOURCE;
  xorDXRendDITHERENABLE = D3DRS_DITHERENABLE;
  xorDXRendEDGEANTIALIAS = D3DRS_EDGEANTIALIAS;
  xorDXRendEMISSIVEMATERIALSOURCE = D3DRS_EMISSIVEMATERIALSOURCE;
  xorDXRendFILLMODE = D3DRS_FILLMODE;
  xorDXRendFOGCOLOR = D3DRS_FOGCOLOR;
  xorDXRendFOGDENSITY = D3DRS_FOGDENSITY;
  xorDXRendFOGENABLE = D3DRS_FOGENABLE;
  xorDXRendFOGEND = D3DRS_FOGEND;
  xorDXRendFOGSTART = D3DRS_FOGSTART;
  xorDXRendFOGTABLEMODE = D3DRS_FOGTABLEMODE;
  xorDXRendFOGVERTEXMODE = D3DRS_FOGVERTEXMODE;
  xorDXRendFORCE_DWORD = D3DRS_FORCE_DWORD;
  xorDXRendINDEXEDVERTEXBLENDENABLE = D3DRS_INDEXEDVERTEXBLENDENABLE;
  xorDXRendLASTPIXEL = D3DRS_LASTPIXEL;
  xorDXRendLIGHTING = D3DRS_LIGHTING;
  xorDXRendLINEPATTERN = D3DRS_LINEPATTERN;
  xorDXRendLOCALVIEWER = D3DRS_LOCALVIEWER;
  xorDXRendMULTISAMPLEANTIALIAS = D3DRS_MULTISAMPLEANTIALIAS;
  xorDXRendMULTISAMPLEMASK = D3DRS_MULTISAMPLEMASK;
  xorDXRendNORMALIZENORMALS = D3DRS_NORMALIZENORMALS;
  xorDXRendNORMALORDER = D3DRS_NORMALORDER;
  xorDXRendPATCHEDGESTYLE = D3DRS_PATCHEDGESTYLE;
  xorDXRendPATCHSEGMENTS = D3DRS_PATCHSEGMENTS;
  xorDXRendPOINTSCALE_A = D3DRS_POINTSCALE_A;
  xorDXRendPOINTSCALE_B = D3DRS_POINTSCALE_B;
  xorDXRendPOINTSCALE_C = D3DRS_POINTSCALE_C;
  xorDXRendPOINTSCALEENABLE = D3DRS_POINTSCALEENABLE;
  xorDXRendPOINTSIZE = D3DRS_POINTSIZE;
  xorDXRendPOINTSIZE_MAX = D3DRS_POINTSIZE_MAX;
  xorDXRendPOINTSIZE_MIN = D3DRS_POINTSIZE_MIN;
  xorDXRendPOINTSPRITEENABLE = D3DRS_POINTSPRITEENABLE;
  xorDXRendPOSITIONORDER = D3DRS_POSITIONORDER;
  xorDXRendRANGEFOGENABLE = D3DRS_RANGEFOGENABLE;
  xorDXRendSHADEMODE = D3DRS_SHADEMODE;
  xorDXRendSOFTWAREVERTEXPROCESSING = D3DRS_SOFTWAREVERTEXPROCESSING;
  xorDXRendSPECULARENABLE = D3DRS_SPECULARENABLE;
  xorDXRendSPECULARMATERIALSOURCE = D3DRS_SPECULARMATERIALSOURCE;
  xorDXRendSRCBLEND = D3DRS_SRCBLEND;
  xorDXRendSTENCILENABLE = D3DRS_STENCILENABLE;
  xorDXRendSTENCILFAIL = D3DRS_STENCILFAIL;
  xorDXRendSTENCILFUNC = D3DRS_STENCILFUNC;
  xorDXRendSTENCILMASK = D3DRS_STENCILMASK;
  xorDXRendSTENCILPASS = D3DRS_STENCILPASS;
  xorDXRendSTENCILREF = D3DRS_STENCILREF;
  xorDXRendSTENCILWRITEMASK = D3DRS_STENCILWRITEMASK;
  xorDXRendSTENCILZFAIL = D3DRS_STENCILZFAIL;
  xorDXRendTEXTUREFACTOR = D3DRS_TEXTUREFACTOR;
  xorDXRendTWEENFACTOR = D3DRS_TWEENFACTOR;
  xorDXRendVERTEXBLEND = D3DRS_VERTEXBLEND;
  xorDXRendWRAP0 = D3DRS_WRAP0;
  xorDXRendWRAP1 = D3DRS_WRAP1;
  xorDXRendWRAP2 = D3DRS_WRAP2;
  xorDXRendWRAP3 = D3DRS_WRAP3;
  xorDXRendWRAP4 = D3DRS_WRAP4;
  xorDXRendWRAP5 = D3DRS_WRAP5;
  xorDXRendWRAP6 = D3DRS_WRAP6;
  xorDXRendWRAP7 = D3DRS_WRAP7;
  xorDXRendZBIAS = D3DRS_ZBIAS;
  xorDXRendZENABLE = D3DRS_ZENABLE;
  xorDXRendZFUNC = D3DRS_ZFUNC;
  xorDXRendZVISIBLE = D3DRS_ZVISIBLE;
  xorDXRendZWRITEENABLE = D3DRS_ZWRITEENABLE;

//Texture address
type
  TxorDXTexAddress = TD3DTextureAddress;

const
  xorDXTexAddrWRAP = D3DTADDRESS_WRAP;
  xorDXTexAddrMIRROR = D3DTADDRESS_MIRROR;
  xorDXTexAddrCLAMP = D3DTADDRESS_CLAMP;
  xorDXTexAddrBORDER = D3DTADDRESS_BORDER;
  xorDXTexAddrMIRRORONCE = D3DTADDRESS_MIRRORONCE;
  xorDXTexAddrFORCE_DWORD = D3DTADDRESS_FORCE_DWORD;

//Texture argument
type
  TxorDXTexArg = LongWord;

const
  xorDXTexArgSELECTMASK = D3DTA_SELECTMASK;
  xorDXTexArgDIFFUSE = D3DTA_DIFFUSE;
  xorDXTexArgCURRENT = D3DTA_CURRENT;
  xorDXTexArgTEXTURE = D3DTA_TEXTURE;
  xorDXTexArgTFACTOR = D3DTA_TFACTOR;
  xorDXTexArgSPECULAR = D3DTA_SPECULAR;
  xorDXTexArgTEMP = D3DTA_TEMP;
  xorDXTexArgCOMPLEMENT = D3DTA_COMPLEMENT;
  xorDXTexArgALPHAREPLICATE = D3DTA_ALPHAREPLICATE;

//Texture coordinates
type
  TxorDXTexCoord = LongWord;

const
  xorDXTexCoordIndex0 = 0; //Index of texture coordinates in vertex format
  xorDXTexCoordIndex1 = 1;
  xorDXTexCoordIndex2 = 2;
  xorDXTexCoordIndex3 = 3;
  xorDXTexCoordIndex4 = 4;
  xorDXTexCoordIndex5 = 5;
  xorDXTexCoordIndex6 = 6;
  xorDXTexCoordIndex7 = 7;
  xorDXTexCoordPASSTHRU = D3DTSS_TCI_PASSTHRU;
  xorDXTexCoordCAMERASPACENORMAL = D3DTSS_TCI_CAMERASPACENORMAL;
  xorDXTexCoordCAMERASPACEPOSITION = D3DTSS_TCI_CAMERASPACEPOSITION;
  xorDXTexCoordCAMERASPACEREFLECTIONVECTOR = D3DTSS_TCI_CAMERASPACEREFLECTIONVECTOR;

//Texture operation
type
  TxorDXTexOp = TD3DTextureOp;

const                                   
  xorDXTexOpADD = D3DTOP_ADD;
  xorDXTexOpADDSIGNED = D3DTOP_ADDSIGNED;
  xorDXTexOpADDSIGNED2X = D3DTOP_ADDSIGNED2X;
  xorDXTexOpADDSMOOTH = D3DTOP_ADDSMOOTH;
  xorDXTexOpBLENDCURRENTALPHA = D3DTOP_BLENDCURRENTALPHA;
  xorDXTexOpBLENDDIFFUSEALPHA = D3DTOP_BLENDDIFFUSEALPHA;
  xorDXTexOpBLENDFACTORALPHA = D3DTOP_BLENDFACTORALPHA;
  xorDXTexOpBLENDTEXTUREALPHA = D3DTOP_BLENDTEXTUREALPHA;
  xorDXTexOpBLENDTEXTUREALPHAPM = D3DTOP_BLENDTEXTUREALPHAPM;
  xorDXTexOpBUMPENVMAP = D3DTOP_BUMPENVMAP;
  xorDXTexOpBUMPENVMAPLUMINANCE = D3DTOP_BUMPENVMAPLUMINANCE;
  xorDXTexOpDISABLE = D3DTOP_DISABLE;
  xorDXTexOpDOTPRODUCT3 = D3DTOP_DOTPRODUCT3;
  xorDXTexOpFORCEDWORD = D3DTOP_FORCE_DWORD;
  xorDXTexOpLERP = D3DTOP_LERP;
  xorDXTexOpMODULATE = D3DTOP_MODULATE;
  xorDXTexOpMODULATE2X = D3DTOP_MODULATE2X;
  xorDXTexOpMODULATE4X = D3DTOP_MODULATE4X;
  xorDXTexOpMODULATEALPHA_ADDCOLOR = D3DTOP_MODULATEALPHA_ADDCOLOR;
  xorDXTexOpMODULATECOLOR_ADDALPHA = D3DTOP_MODULATECOLOR_ADDALPHA;
  xorDXTexOpMODULATEINVALPHA_ADDCOLOR = D3DTOP_MODULATEINVALPHA_ADDCOLOR;
  xorDXTexOpMODULATEINVCOLOR_ADDALPHA = D3DTOP_MODULATEINVCOLOR_ADDALPHA;
  xorDXTexOpMULTIPLYADD = D3DTOP_MULTIPLYADD;
  xorDXTexOpPREMODULATE = D3DTOP_PREMODULATE;
  xorDXTexOpSELECTARG1 = D3DTOP_SELECTARG1;
  xorDXTexOpSELECTARG2 = D3DTOP_SELECTARG2;
  xorDXTexOpSUBTRACT = D3DTOP_SUBTRACT;


//Texture states
type
  TxorDXTexState = TD3DTextureStageStateType;

const                                         
  xorDXTexStADDRESSU = D3DTSS_ADDRESSU;
  xorDXTexStADDRESSV = D3DTSS_ADDRESSV;
  xorDXTexStADDRESSW = D3DTSS_ADDRESSW;
  xorDXTexStALPHAARG0 = D3DTSS_ALPHAARG0;
  xorDXTexStALPHAARG1 = D3DTSS_ALPHAARG1;
  xorDXTexStALPHAARG2 = D3DTSS_ALPHAARG2;
  xorDXTexStALPHAOP = D3DTSS_ALPHAOP;
  xorDXTexStBORDERCOLOR = D3DTSS_BORDERCOLOR;
  xorDXTexStBUMPENVLOFFSET = D3DTSS_BUMPENVLOFFSET;
  xorDXTexStBUMPENVLSCALE = D3DTSS_BUMPENVLSCALE;
  xorDXTexStBUMPENVMAT00 = D3DTSS_BUMPENVMAT00;
  xorDXTexStBUMPENVMAT01 = D3DTSS_BUMPENVMAT01;
  xorDXTexStBUMPENVMAT10 = D3DTSS_BUMPENVMAT10;
  xorDXTexStBUMPENVMAT11 = D3DTSS_BUMPENVMAT11;
  xorDXTexStCOLORARG0 = D3DTSS_COLORARG0;
  xorDXTexStCOLORARG1 = D3DTSS_COLORARG1;
  xorDXTexStCOLORARG2 = D3DTSS_COLORARG2;
  xorDXTexStCOLOROP = D3DTSS_COLOROP;
  xorDXTexStFORCEDWORD = D3DTSS_FORCE_DWORD;
  xorDXTexStMAGFILTER = D3DTSS_MAGFILTER;
  xorDXTexStMAXANISOTROPY = D3DTSS_MAXANISOTROPY;
  xorDXTexStMAXMIPLEVEL = D3DTSS_MAXMIPLEVEL;
  xorDXTexStMINFILTER = D3DTSS_MINFILTER;
  xorDXTexStMIPFILTER = D3DTSS_MIPFILTER;
  xorDXTexStMIPMAPLODBIAS = D3DTSS_MIPMAPLODBIAS;
  xorDXTexStRESULTARG = D3DTSS_RESULTARG;
  xorDXTexStTEXCOORDINDEX = D3DTSS_TEXCOORDINDEX;
  xorDXTexStTEXTURETRANSFORMFLAGS = D3DTSS_TEXTURETRANSFORMFLAGS;

//Texture usage
type
  TxorDXUsage = LongWord;

const
  xorDXUsageRENDERTARGET = D3DUSAGE_RENDERTARGET;
  xorDXUsageDEPTHSTENCIL = D3DUSAGE_DEPTHSTENCIL;
  xorDXUsageWRITEONLY = D3DUSAGE_WRITEONLY;
  xorDXUsageSOFTWAREPROCESSING = D3DUSAGE_SOFTWAREPROCESSING;
  xorDXUsageDONOTCLIP = D3DUSAGE_DONOTCLIP;
  xorDXUsagePOINTS = D3DUSAGE_POINTS;
  xorDXUsageRTPATCHES = D3DUSAGE_RTPATCHES;
  xorDXUsageNPATCHES = D3DUSAGE_NPATCHES;
  xorDXUsageDYNAMIC = D3DUSAGE_DYNAMIC;

//Stuff
type
  TxorDXCaps = TD3DCaps8;                                                       //properties of graphic adapter
  TxorDXFormat = TD3DFormat;
  PxorDXImageInfo = PD3DXImageInfo;
  TxorDXInputCaps = TDIDevCaps;
  PxorDXLightProperties = ^TxorDXLightProperties;
  TxorDXLightProperties = TD3DLight8;
  PxorDXMaterial = ^TxorDXMaterial;
  TxorDXMaterial = TD3DMaterial8;
  PxorDXPaletteEntry = PPaletteEntry;
  TxorDXPool = TD3DPool;
  TxorDXPresentationParamters = TD3DPresent_Parameters;

//==============================================================================
{ Math tools }

const
  Pi2: Single = 0; // set in initialization section
  Pi05: Single = 0; // set in initialization section
  MaxDWord = 4294967295;

type
  TxorDXVector = TD3DXVector3; //(packed) Record x, y, z: Single; end;
  TxorDXCoord = TD3DXVector2; //(packed) Record x, y: Single; end;
  TxorDXMatrix = TD3DXMatrix;  //M._ij   1 <= i,j <= 4
  TxorDXPlane = TD3DXPlane;

function GetAngle(V1, V2: TxorDXVector): Single;
function GetAngleProj(V1, V2: TxorDXVector): Single;
function GetCoord(x, y: Single): TxorDXCoord;
function GetPlaneFromPoints(V1, V2, V3: TxorDXVector): TxorDXPlane;
function GetPlaneFromPointVector(P, V: TxorDXVector): TxorDXPlane;
function GetPlaneXY(Z: Single): TxorDXPlane;
function GetPlaneXZ(Y: Single): TxorDXPlane;
function GetPlaneYZ(X: Single): TxorDXPlane;
function GetRadiansMod2Pi(Rad: Single): Single;
function GetVector(x, y, z: Single): TxorDXVector;

{ proj a on b := b * <a,b> / |b| }

//==============================================================================
{ Vertex }

//Flexible vertex types
type
  PxorDXVertexPlain = ^TxorDXVertexPlain;
  TxorDXVertexPlain = packed record
                      Position: TxorDXVector;
                      end;
  PxorDXVertexColored = ^TxorDXVertexColored;
  TxorDXVertexColored =  packed record
                         Position: TxorDXVector;
                         Color: TxorDXColor;
                         end;
  PxorDXVertexClassic = ^TxorDXVertexClassic;
  TxorDXVertexClassic =  packed record
                         Position: TxorDXVector;
                         Normal: TxorDXVector;
                         Color: TxorDXColor;
                         end;
  PxorDXVertexPoint = ^TxorDXVertexPoint;
  TxorDXVertexPoint =  packed record
                       Position: TxorDXVector;
                       Normal: TxorDXVector;
                       Size: Single;
                       Color: TxorDXColor;
                       end;
  PxorDXVertexTexture = ^TxorDXVertexTexture;
  TxorDXVertexTexture =  packed record
                         Position: TxorDXVector;
                         Normal: TxorDXVector;
                         Color: TxorDXColor;
                         TexCoordinates: TxorDXCoord;
                         end;

  PxorDXVertexTexture2 = ^TxorDXVertexTexture2;
  TxorDXVertexTexture2 =  packed record
                          Position: TxorDXVector;
                          Normal: TxorDXVector;
                          Color: TxorDXColor;
                          TexCoordinates: TxorDXCoord;
                          TexCoordinates2: TxorDXCoord;
                          end;

//Flexible vertex format flags and size
type
  TxorDXVertexFlag = LongWord;

const
  xorDXVertexFlagPlain = D3DFVF_XYZ;
  xorDXVertexFlagColored = D3DFVF_XYZ or D3DFVF_DIFFUSE;
  xorDXVertexFlagClassic = D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_DIFFUSE;
  xorDXVertexFlagPoint = D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_PSIZE or D3DFVF_DIFFUSE;
  xorDXVertexFlagTexture = D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_DIFFUSE or D3DFVF_TEX1;
  xorDXVertexFlagTexture2 = D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_DIFFUSE or D3DFVF_TEX2;

type
  TxorDXVertexSize = Cardinal;

const
  xorDXVertexSizePlain = SizeOf(TxorDXVertexPlain);
  xorDXVertexSizeColored = SizeOf(TxorDXVertexColored);
  xorDXVertexSizeClassic = SizeOf(TxorDXVertexClassic);
  xorDXVertexSizePoint = SizeOf(TxorDXVertexPoint);
  xorDXVertexSizeTexture = SizeOf(TxorDXVertexTexture);
  xorDXVertexSizeTexture2 = SizeOf(TxorDXVertexTexture2);

function GetVertexPlain(x, y, z: Single): TxorDXVertexPlain;
function GetVertexColored(x, y, z: Single; r, g, b, Alpha: LongWord): TxorDXVertexColored; overload; //red, green, blue, alpha
function GetVertexColored(x, y, z: Single; r, g, b: LongWord; Alpha: Single): TxorDXVertexColored; overload;
function GetVertexColored(x, y, z: Single; Col: TColor; Alpha: LongWord): TxorDXVertexColored; overload;
function GetVertexColored(x, y, z: Single; Col: TColor; Alpha: Single): TxorDXVertexColored; overload;
function GetVertexClassic(x, y, z, n1, n2, n3: Single; r, g, b, Alpha: LongWord): TxorDXVertexClassic; overload;
function GetVertexClassic(x, y, z, n1, n2, n3: Single; r, g, b: LongWord; Alpha: Single): TxorDXVertexClassic; overload;
function GetVertexClassic(x, y, z, n1, n2, n3: Single; Col: TColor; Alpha: LongWord): TxorDXVertexClassic; overload;
function GetVertexClassic(x, y, z, n1, n2, n3: Single; Col: TColor; Alpha: Single): TxorDXVertexClassic; overload;
function GetVertexPoint(x, y, z, n1, n2, n3, PSize: Single; r, g, b, Alpha: LongWord): TxorDXVertexPoint; overload;
function GetVertexPoint(x, y, z, n1, n2, n3, PSize: Single; r, g, b: LongWord; Alpha: Single): TxorDXVertexPoint; overload;
function GetVertexPoint(x, y, z, n1, n2, n3, PSize: Single; Col: TColor; Alpha: LongWord): TxorDXVertexPoint; overload;
function GetVertexPoint(x, y, z, n1, n2, n3, PSize: Single; Col: TColor; Alpha: Single): TxorDXVertexPoint; overload;
function GetVertexTexture(x, y, z, n1, n2, n3, tx, ty: Single; r, g, b, Alpha: LongWord): TxorDXVertexTexture; overload;
function GetVertexTexture(x, y, z, n1, n2, n3, tx, ty: Single; r, g, b: LongWord; Alpha: Single): TxorDXVertexTexture; overload;
function GetVertexTexture(x, y, z, n1, n2, n3, tx, ty: Single; Col: TColor; Alpha: LongWord): TxorDXVertexTexture; overload;
function GetVertexTexture(x, y, z, n1, n2, n3, tx, ty: Single; Col: TColor; Alpha: Single): TxorDXVertexTexture; overload;
function GetVertexTexture2(x, y, z, n1, n2, n3, tx, ty, tx2, ty2: Single; r, g, b, Alpha: LongWord): TxorDXVertexTexture2; overload;
function GetVertexTexture2(x, y, z, n1, n2, n3, tx, ty, tx2, ty2: Single; r, g, b: LongWord; Alpha: Single): TxorDXVertexTexture2; overload;
function GetVertexTexture2(x, y, z, n1, n2, n3, tx, ty, tx2, ty2: Single; Col: TColor; Alpha: LongWord): TxorDXVertexTexture2; overload;
function GetVertexTexture2(x, y, z, n1, n2, n3, tx, ty, tx2, ty2: Single; Col: TColor; Alpha: Single): TxorDXVertexTexture2; overload;

function VertexFormatHasClassicStructure(VertexFlag: TxorDXVertexFlag): Boolean;
function VertexFormatHasTextureStructure(VertexFlag: TxorDXVertexFlag): LongWord;

//Vertex holder
type
  TxorDXVertexArray = Array of Byte;

//==============================================================================
{ Index }

//Index-16-Bit-types
type
  TxorDXIndex = Word;
  TxorDXIndexArray = Array of TxorDXIndex;
  PxorDXTriangle = ^TxorDXTriangle;
  TxorDXTriangle = packed Record V1, V2, V3: TxorDXIndex; end;
  PxorDXLine = ^TxorDXLine;
  TxorDXLine = packed Record V1, V2: TxorDXIndex; end;

function GetLine(Idx1, Idx2: TxorDXIndex): TxorDXLine;
function GetTriangle(Idx1, Idx2, Idx3: TxorDXIndex): TxorDXTriangle;

//Index-32-Bit-types
type
  TxorDXIndex32 = LongWord;
  TxorDXIndexArray32 = Array of TxorDXIndex32;
  PxorDXTriangle32 = ^TxorDXTriangle32;
  TxorDXTriangle32 = packed Record V1, V2, V3: TxorDXIndex32; end;
  PxorDXLine32 = ^TxorDXLine32;
  TxorDXLine32 = packed Record V1, V2: TxorDXIndex32; end;

function GetLine32(Idx1, Idx2: TxorDXIndex32): TxorDXLine32;
function GetTriangle32(Idx1, Idx2, Idx3: TxorDXIndex32): TxorDXTriangle32;

//==============================================================================
{ Misc }

//All stuff
const
  Enter = #13#10;

type
  PCardinal = ^Cardinal;
  PLongWord = ^LongWord;
  PSingle = ^Single;

//Blend light effects
type
  TxorDXBlendLightOperation = (bop1X, bop2X, bop4X);                            //normal modulate=1X, lighter=2X, lightest=4X

function GetBlendLightOp(BOp: TxorDXBlendLightOperation): TxorDXTexOp;

//Camera Tools
type
  TxorDXCameraTransformation = Byte;

const
  xorDXCamHuman = 0;
  xorDXCamID = 1;
  xorDXCamScene = 2;
  xorDXCamUp = 3;

//Font
const
  xorDXStandardFont: TLogFont = (lfHeight: 24; lfWidth: 0; lfEscapement: 0; lfOrientation: 0;
                                 lfWeight: FW_NORMAL; lfItalic: 0; lfUnderline: 0;
                                 lfStrikeOut: 0; lfCharSet: DEFAULT_CHARSET;
                                 lfOutPrecision: OUT_TT_PRECIS; lfClipPrecision: CLIP_DEFAULT_PRECIS;
                                 lfQuality: PROOF_QUALITY; lfPitchAndFamily: DEFAULT_PITCH;
                                 lfFaceName: 'Arial');

//Render options
type
  TxorDXRenderOption = (vpPure, vpHard, vpMix, vpSoft);

//Screen options
type
  TxorDXResolution = (rs800x600x16, rs800x600x32, rs1024x768x16, rs1024x768x32);

procedure GetResolution(Res: TxorDXResolution; var Width, Height, Depth: Cardinal);

//Stages
type
  TxorDXStage = 0..7;

//Time
procedure InitRotationPerSecond;
function RotationsPerSecond(R: Single): Single;

//==============================================================================
{ directX-interface types }

type

  //forward declarations
  TxorDXCustomCamera = class;
  TxorDXCustomInsertObject = class;
  TxorDXScene = class;
  TxorDXTexture = class;

  { TxorDXCustomObject }

  TxorDXCustomObject = class(TObject)
  protected
    FScene: TxorDXScene;
    function GetAlpha(Index: Integer): Single;
    function GetTColor(Index: Integer): TColor;
    procedure SetAlpha(const Index: Integer; const Value: Single);
    procedure SetTColor(const Index: Integer; const Value: TColor);
  public
    constructor Create(Scene: TxorDXScene);
    function ErrorTest(HR: HResult; Msg: String; InternalMsg: TxorDXErrorMessage; ThrowException: Boolean): Boolean; overload;
    function ErrorTest(HR: HResult; ResourceMsg: PResStringRec; InternalMsg: TxorDXErrorMessage; ThrowException: Boolean): Boolean; overload;
  end;

  { TxorDXRendering }

  TxorDXRendering = class(TxorDXCustomObject)
  public
    procedure GetRenderState(State: TxorDXRenderState; var Color: TColor; var Alpha: Single); overload; virtual;
    procedure GetRenderState(State: TxorDXRenderState; var Value: Boolean); overload; virtual;
    procedure GetRenderState(State: TxorDXRenderState; var Value: LongWord); overload; virtual;
    procedure GetRenderState(State: TxorDXRenderState; var Value: Single); overload; virtual;
    procedure Render(State: TxorDXRenderState; Color: TColor; Alpha: Single); overload; virtual;
    procedure Render(State: TxorDXRenderState; Value: Boolean); overload; virtual;
    procedure Render(State: TxorDXRenderState; Value: LongWord); overload; virtual;
    procedure Render(State: TxorDXRenderState; Value: Single); overload; virtual;
  end;

  { TxorDXCustomInput }

  TxorDXCustomInput = class(TxorDXCustomObject)
  protected
    FCaps: TxorDXInputCaps;
    FDevice: IDirectInputDevice8;
    function GetAttached: Boolean;
  public
    constructor Create(Scene: TxorDXScene);
    destructor Destroy; override;
    function Get: Boolean; virtual; abstract;
    function GetCaps: Boolean;
    property Attached: Boolean read GetAttached;
    property _Device: IDirectInputDevice8 read FDevice;
    property Caps: TxorDXInputCaps read FCaps;
  end;

  { TxorDXKeyboard }

  TxorDXKeyboard = class(TxorDXCustomInput)
  protected
    FKeyboardState: TDIKeyboardState;
  public
    constructor Create(Scene: TxorDXScene);
    function Get: Boolean; override;
    function KeyPressed(Key: Byte): Boolean;                                    //using Win Virtual Key Codes
    function KeyPressedAny: Boolean;
    function KeyPressedDIK(Key: Byte): Boolean;                                 //using DirectInputKeyCodes
  end;

  { TxorDXMouse }

  TxorDXMouse = class(TxorDXCustomInput)
  protected
    FClickedLeft, FClickedRight, FClickedButton3, FClickedButton4: Boolean;
    FDX, FDY, FDZ: Single;
    FInvertX, FInvertY, FInvertZ: Boolean;
    FMouseState: TDIMouseState;
    FSpeed: Single;
    FVisible: Boolean;
    FX, FY, FZ: Single;                                //DONE 4: evtl. cursor als textur laden
    procedure SetVisible(Value: Boolean);
  public
    constructor Create(Scene: TxorDXScene);
    function Get: Boolean; override;
    property DX: Single read FDX;
    property DY: Single read FDY;
    property DZ: Single read FDZ;
    property ClickedLeft: Boolean read FClickedLeft;
    property ClickedRight: Boolean read FClickedRight;
    property ClickedButton3: Boolean read FClickedButton3;
    property ClickedButton4: Boolean read FClickedButton4;
    property InvertX: Boolean read FInvertX write FInvertX;
    property InvertY: Boolean read FInvertY write FInvertY;
    property InvertZ: Boolean read FInvertZ write FInvertZ;
    property Speed: Single read FSpeed write FSpeed;
    property Visible: Boolean read FVisible write SetVisible;
    property X: Single read FX;
    property Y: Single read FY;
    property Z: Single read FZ;
  end;

  { TxorDXCustomState }

  TxorDXCustomState = class(TxorDXCustomObject)
  protected
    FIsActive: Boolean;
    procedure SetActive(const Value: Boolean);
  public
    procedure Activate; virtual;
    procedure Deactivate; virtual;
    property IsActive: Boolean read FIsActive write SetActive;
  end;

  { TxorDXFog }

  TxorDXFog = class(TxorDXCustomState)
  protected
    FAutoColor: Boolean;
    FColor: TxorDXColor;
    FDensity: Single;
    FEnd: Single;
    FRadial: Boolean;
    FStart: Single;                                              
    FTableMode: TxorDXFogMode;
    FVertexMode: TxorDXFogMode;
    function GetRadialEnable: Boolean;
    procedure SetRadial(const Value: Boolean);
  public
    constructor Create(Scene: TxorDXScene);
    procedure Activate; override;
    procedure Deactivate; override;
    procedure SetColor(r, g, b, Alpha: LongWord);
    property Alpha: Single index 0 read GetAlpha write SetAlpha;
    property AutoColor: Boolean read FAutoColor write FAutoColor;
    property Color: TColor index 0 read GetTColor write SetTColor;
    property Density: Single read FDensity write FDensity;
    property FogEnd: Single read FEnd write FEnd;
    property FogStart: Single read FStart write FStart;
    property Radial: Boolean read FRadial write SetRadial;
    property RadialEnable: Boolean read GetRadialEnable;
    property TableMode: TxorDXFogMode read FTableMode write FTableMode;
    property VertexMode: TxorDXFogMode read FVertexMode write FVertexMode;
  end;

  { TxorDXAlphaBlending }

  TxorDXAlphaBlending = class(TxorDXCustomState)
  protected
    FStage: TxorDXStage;
    function GetMaxStageNo: TxorDXStage;
    function GetMaxSimultaneousTextures: Word;
    procedure SetStage(const Value: TxorDXStage);
  public
    constructor Create(Scene: TxorDXScene);
    procedure Activate; override;
    procedure AlphaStageBlend(Arg1, Arg2: TxorDXTexArg; Op: TxorDXTexOp);       //Alpha Stage blending
    procedure AlphaStageBlendCurrentOnly;
    procedure AlphaStageBlendDefault;
    procedure AlphaStageBlendDiffuseOnly;
    procedure AlphaStageBlendDisabled;
    procedure AlphaStageBlendModCurrentAndTexture;
    procedure AlphaStageBlendModDiffuseAndTexture;
    procedure AlphaStageBlendTextureOnly;
    procedure Blend(SrcBlend, DestBlend: TxorDXBlend);                          //Src Dest Blending
    procedure BlendDefault;
    procedure BlendDestAlpha;
    procedure BlendDestAlphaInv;
    procedure BlendDestColor;
    procedure BlendDestColorInv;
    procedure BlendDestOnly;
    procedure BlendDisabled;
    procedure BlendSrcAlpha;
    procedure BlendSrcAlphaInv;
    procedure BlendSrcColor;
    procedure BlendSrcColorInv;
    procedure BlendSrcOnly;
    procedure Deactivate; override;
    property MaxSimultaneousTextures: Word read GetMaxSimultaneousTextures;
    property MaxStageNo: TxorDXStage read GetMaxStageNo;
    property Stage: TxorDXStage read FStage write SetStage;
  end;

  { TxorDXTransparency }

  TxorDXTransparency = class(TxorDXAlphaBlending)
  public
    procedure Activate; override;
    procedure Deactivate; override;
  end;

  { TxorDXCustomDevice }

  TxorDXCustomDevice = class(TxorDXCustomObject)
  private
    PrevDevice: TxorDXCustomDevice;
    procedure OnLostDevice; virtual;
    procedure OnResetDevice; virtual;
  public
    constructor Create(Scene: TxorDXScene);
  end;

  { TxorDXFont }

  TxorDXFont = class(TxorDXCustomDevice)
  private
    procedure OnLostDevice; override;
    procedure OnResetDevice; override;
  protected
    FDevice: ID3DXFont;
  public
    constructor Create(Scene: TxorDXScene); overload;
    constructor Create(Scene: TxorDXScene; FontParams: TLogFont); overload;
    constructor Create(Scene: TxorDXScene; Name: ShortString; Height: Word; Style: TFontStyles); overload;
    destructor Destroy; override;
    procedure TextOut(Rect: TRect; Format: TxorDXFontFormat; Color: TColor; Alpha: Single; const Text: AnsiString); overload;
    procedure TextOut(Rect: TRect; Format: TxorDXFontFormat; Color: TColor; Alpha: LongWord; const Text: AnsiString); overload;
    property _Device: ID3DXFont read FDevice;
  end;

  { TxorDXCustomTransform }

  TxorDXCustomTransform = class(TxorDXCustomObject)
  private
    FTransformationSave: TxorDXMatrix;
  protected
    FTransformed: Boolean;
    FTransformation: TxorDXMatrix;
    function GetIsTransformed: Boolean; virtual;
    function GetTransformationMatrix: TxorDXMatrix; virtual;
    procedure SetTransformationMatrix(const Value: TxorDXMatrix); virtual;
  public
    constructor Create(Scene: TxorDXScene);
    procedure ClearTransformation; virtual;
    procedure FixTransformation; virtual;
    procedure Move(x, y, z: Single); virtual;
    procedure RotateAxis(x, y, z, Angle: Single); virtual;
    procedure RotateX(Angle: Single); virtual;
    procedure RotateY(Angle: Single); virtual;
    procedure RotateZ(Angle: Single); virtual;
    procedure Scale(x, y, z: Single); virtual;
    procedure UndoTransformation; virtual;
    property IsTransformed: Boolean read GetIsTransformed;
    property TransformationMatrix: TxorDXMatrix read GetTransformationMatrix write SetTransformationMatrix;
  end;

  { TxorDXScene }

  TxorDXScene = class(TxorDXCustomTransform)
  private
    FActiveLights: Cardinal;
    FActiveInputs: Cardinal;
    FBufferCreated: Boolean;
    FFormatFixed: Boolean;
    FIndexBufferSize: Cardinal;
    FIsDrawing: Boolean;
    FUse32BitIndex: Boolean;
    FUsedLights: Cardinal;
    FVertexBufferFlags: TxorDXVertexFlag;
    FVertexBufferSize: TxorDXVertexSize;
  protected
    FAutoLightOnOff: Boolean;
    FBackgroundColor: TxorDXColor;
    FCaps: TxorDXCaps;                           //DONE 4: chmod und reset
    FCapsRead: Boolean;
    FDevice: IDirect3DDevice8;                                                  //Device
    FDrawCalls: Integer;
    FHandleWindow: HWND;
    FHandleOutput: HWND;
    FIndexBuffer: IDirect3DIndexBuffer8;
    FInput: IDirectInput8;
    FInterface: IDirect3D8;
    FLastError: HResult;
    FLastErrorMsg: String;
    FLastActivatedCamera: TxorDXCustomCamera;
    FLastActivatedTextures: Array[TxorDXStage] of TxorDXTexture;
    FLastInsertedObject: TxorDXCustomInsertObject;                              //Last inserted object
    FLastInsertedDevice: TxorDXCustomDevice;                                    //Last inserted device (for onlostdevice, onresetdevice events)
    FOnDraw: TNotifyEvent;                                                      //Scene-Draw-Event
    FOnDrawException: TxorDXDrawException;
    FPresentationParameters: TxorDXPresentationParamters;
    FVertexBuffer: IDirect3DVertexBuffer8;
    procedure DrawScene(Sender: TObject; var Done: Boolean); virtual;
    function GetBoolValue(Index: Integer): Boolean;
    function GetCardinalValue(Index: Integer): Cardinal;
    function GetCullMode: TxorDXCull;
    function GetMaterialSource(Index: Integer): TxorDXMaterialSource;
    procedure LostDevice;                                                       //call this before reset to avoid lost device problem by inserted devices
    procedure ResetDevice;                                                      //call this after reset to avoid lost device problem by inserted devices
    procedure Set32BitIndex(Value: Boolean);
    procedure SetBoolValue(const Index: Integer; const Value: Boolean);
    procedure SetCullMode(Value: TxorDXCull);
    procedure SetIndexBufferSize(Value: Cardinal);
    procedure SetMaterialSource(const Index: Integer; const Value: TxorDXMaterialSource);
    procedure SetTransformation; virtual;
    procedure SetVertexBufferSize(Value: TxorDXVertexSize);
    procedure SetVertexBufferFlags(Value: TxorDXVertexFlag);
  public
    constructor Create(FormHandle: HWND);
    constructor CreateFullScreen(FormHandle: HWND); overload;
    constructor CreateFullScreen(FormHandle: HWND; Resolution: TxorDXResolution); overload;
    constructor CreateParams(FormHandle: HWND; FullScreen: Boolean; BufferSwapEffect: TxorDXSwapEffect; PresentationInterval: TxorDXPresentationInterval; AntiAliasValue: TxorDXMultiSampleType; ScreenWidth, ScreenHeight: Cardinal; ColorDepth32Bit: Boolean; Behaviour: TxorDXRenderOption);
    constructor CreateWindowed(FormHandle: HWND);
    destructor Destroy; override;
    procedure Clear; virtual;                                                   // Clear the scene.
    procedure ClearError;
    procedure CreateBuffers; virtual;
    procedure Draw; virtual;                                                    // Draw the scene, occures onDraw-event.
    function Failed: Boolean;
    procedure SetAmbientLight(r, g, b, Alpha: LongWord); overload;              // Sets the ambient light color.
    procedure SetAmbientLight(r, g, b: LongWord; Alpha: Single); overload;
    procedure SetBackgroundColor(r, g, b, Alpha: LongWord); overload;           // Sets the background color.
    procedure SetBackgroundColor(r, g, b: LongWord; Alpha: Single); overload;
    procedure StartLooping; virtual;                                            // Loop the draw routine.
    procedure StopLooping; virtual;                                             // Stop looping the draw routine.
    function Succeeded: Boolean;
    property _Device: IDirect3DDevice8 read FDevice;
    property _Handle: HWND read FHandleWindow;
    property _Input: IDirectInput8 read FInput;
    property _Interface: IDirect3D8 read FInterface;
    property _Vertexbuffer: IDirect3DVertexBuffer8 read FVertexBuffer;
    property _IndexBuffer: IDirect3DIndexBuffer8 read FIndexBuffer;
    property AmbientLightAlpha: Single index 10 read GetAlpha write SetAlpha;
    property AmbientLightColor: TColor index 10 read GetTColor write SetTColor;
    property AutoLightOnOff: Boolean read FAutoLightOnOff write FAutoLightOnOff;
    property BackgroundAlpha: Single index 1 read GetAlpha write SetAlpha;
    property BackgroundColor: TColor index 1 read GetTColor write SetTColor;
    property Caps: TxorDXCaps read FCaps;
    property CullMode: TxorDXCull read GetCullMode write SetCullMode;
    property DitherEnable: Boolean index 1 read GetBoolValue write SetBoolValue;
    property DrawCalls: Integer read FDrawCalls write FDrawCalls;
    property ErrorCode: HResult read FLastError;
    property ErrorMsg: String read FLastErrorMsg;
    property FullScreen: Boolean index 4 read GetBoolValue;
    property IndexBufferSize: Cardinal read FIndexBufferSize write SetIndexBufferSize;
    property LightsOn: Boolean index 0 read GetBoolValue write SetBoolValue;
    property MaterialSourceAmbient: TxorDXMaterialSource index 0 read GetMaterialSource write SetMaterialSource;
    property MaterialSourceDiffuse: TxorDXMaterialSource index 1 read GetMaterialSource write SetMaterialSource;
    property MaterialSourceEmissive: TxorDXMaterialSource index 2 read GetMaterialSource write SetMaterialSource;
    property MaterialSourceSpecular: TxorDXMaterialSource index 3 read GetMaterialSource write SetMaterialSource;
    property OnDraw: TNotifyEvent read FOnDraw write FOnDraw;                   // Event called by draw method.
    property OnDrawException: TxorDXDrawException read FOnDrawException write FOnDrawException;
    property OutputHandle: HWND read FHandleOutput write FHandleOutput;
    property PresentationParameters: TxorDXPresentationParamters read FPresentationParameters;
    property SpecularEnable: Boolean index 2 read GetBoolValue write SetBoolValue;
    property ScreenHeight: Cardinal index 0 read GetCardinalValue;
    property ScreenWidth: Cardinal index 1 read GetCardinalValue;
    property Use32BitIndex: Boolean read FUse32BitIndex write Set32BitIndex;
    property VertexBufferFlags: TxorDXVertexFlag read FVertexBufferFlags write SetVertexBufferFlags;
    property VertexBufferSize: TxorDXVertexSize read FVertexBufferSize write SetVertexBufferSize;
    property ZEnable: Boolean index 3 read GetBoolValue write SetBoolValue;
  end;

  { TxorDXCustomActiveTransform }

  TxorDXCustomActiveTransform = class(TxorDXCustomTransform)
  protected
    function GetActive: Boolean; virtual; abstract;
    procedure SetActive(const Value: Boolean);
  public
    procedure Activate; virtual; abstract;
    procedure Deactivate; virtual; abstract;
    property IsActive: Boolean read GetActive write SetActive;
  end;

  { TxorDXTexture }

  TxorDXTexture = class(TxorDXCustomActiveTransform)
  protected
    FMipLevels: Word;
    FStage: TxorDXStage;
    FTexture: IDirect3DTexture8;
    FTransparencyColor: TxorDXColor;
    function GetActive: Boolean; override;
    function GetAnyStageActivity: Boolean;
    function GetMaxSimultaneousTextures: Word;
    function GetMaxStageNo: TxorDXStage;
    function GetTransparencyAlpha: Single;
    function GetTransparencyColor: TColor;
    function GetTransparent: Boolean;
    procedure SetStage(const Value: TxorDXStage);
  public
    constructor Create(Scene: TxorDXScene);
    procedure Activate; override;
    procedure ColorStageBlend(Arg1, Arg2: TxorDXTexArg; Op: TxorDXTexOp);       //Color Stage Blending
    procedure ColorStageBlendAdd;
    procedure ColorStageBlendAsBumpMap;
    procedure ColorStageBlendAsBumpMapLuminance;
    procedure ColorStageBlendAsDarkMap;
    procedure ColorStageBlendAsDetailMap;
    procedure ColorStageBlendAsGlowMap;
    procedure ColorStageBlendAsLightMap(BlendLightOperation: TxorDXBlendLightOperation);
    procedure ColorStageBlendCurrentAlpha;
    procedure ColorStageBlendDiffuseAlpha;
    procedure ColorStageBlendDiffuseColorOnly;
    procedure ColorStageBlendDisable;
    procedure ColorStageBlendFactor(PerCent: Single);
    procedure ColorStageBlendModulate;
    procedure ColorStageBlendTextureAlpha;
    procedure ColorStageBlendTextureAlphaPM;
    procedure ColorStageBlendTextureAndCurrent(BlendLightOperation: TxorDXBlendLightOperation);
    procedure ColorStageBlendTextureAndDiffuseColor(BlendLightOperation: TxorDXBlendLightOperation);
    procedure ColorStageBlendTextureOnly;
    procedure Deactivate; override;
    procedure LoadEmptyEditTexture; overload; virtual;
    procedure LoadEmptyEditTexture(Width, Height: Cardinal); overload; virtual;
    procedure LoadEmptyEditTexture(Width, Height, MipLevels: Cardinal); overload; virtual;
    procedure LoadEmptyRenderTexture; overload; virtual;
    procedure LoadEmptyRenderTexture(Width, Height: Cardinal); overload; virtual;
    procedure LoadEmptyRenderTexture(Width, Height, MipLevels: Cardinal); overload; virtual;
    procedure LoadEmptyTexture(Width, Height, MipLevels: Cardinal; Usage: TxorDXUsage; Format: TxorDXFormat; Pool: TxorDXPool); overload; virtual;
    procedure LoadFromFile(FileName: AnsiString); overload; virtual;
    procedure LoadFromFile(FileName: AnsiString; Transparent: Boolean; TransparencyColor: TColor; TransparencyAlpha: Single); overload; virtual;
    procedure LoadFromFile(FileName: AnsiString; MipLevels: Cardinal; Transparent: Boolean; TransparencyColor: TColor; TransparencyAlpha: Single); overload; virtual;
    procedure LoadFromFile(FileName: AnsiString; Width, Height: Cardinal; Transparent: Boolean; TransparencyColor: TColor; TransparencyAlpha: Single); overload; virtual;
    procedure LoadFromFile(FileName: AnsiString; Width, Height, MipLevels: Cardinal; Filter, MipFilter: TxorDXFilter; Transparent: Boolean; TransparencyColor: TColor; TransparencyAlpha: Single); overload; virtual;
    procedure LoadFromFile(FileName: AnsiString; Width, Height, MipLevels: Cardinal; Usage: TxorDXUsage; Format: TxorDXFormat; Pool: TxorDXPool; Filter, MipFilter: TxorDXFilter; Transparent: Boolean; TransparencyColor: TColor; TransparencyAlpha: Single; SrcInfo: PxorDXImageInfo; Palette: PxorDXPaletteEntry); overload; virtual;
    procedure Move(x, y, z: Single); override;
    procedure RotateAxis(x, y, z, Angle: Single); override;
    procedure RotateX(Angle: Single); override;
    procedure RotateY(Angle: Single); override;
    procedure Scale(x, y, z: Single); override;
    procedure SetTextureAdressU(Adr: TxorDXTexAddress);
    procedure SetTextureAdressV(Adr: TxorDXTexAddress);
    procedure SetTextureBorderColor(C: TxorDXColor);
    procedure SetTextureCoordinates(Coord: TxorDXTexCoord);
    procedure SetTextureState(State: TxorDXTexState; Value: LongWord); overload;
    procedure SetTextureState(State: TxorDXTexState; Value: Single); overload;
    procedure Unload; virtual;
    property IsTransparent: Boolean read GetTransparent;
    property TransparencyAlpha: Single read GetTransparencyAlpha;
    property TransparencyColor: TColor read GetTransparencyColor;
    property MaxSimultaneousTextures: Word read GetMaxSimultaneousTextures;
    property MaxStageNo: TxorDXStage read GetMaxStageNo;
    property MipLevels: Word read FMipLevels;
    property Stage: TxorDXStage read FStage write SetStage;
    end;

  { TxorDXBumpMap }

  TxorDXBumpMap = class(TxorDXTexture)
  protected
    FOffset: Single;
    FScale: Single;
    FWithLuminance: Boolean;
  public
    constructor Create(Scene: TxorDXScene);
    procedure Activate; override;
    procedure Deactivate; override;
    property OffsetEnv: Single read FOffset write FOffset;
    property ScaleEnv: Single read FScale write FScale;
    property WithLuminance: Boolean read FWithLuminance write FWithLuminance;
  end;
  //DONE 4: bumpmaps testen (s.a. caps), und was ist mit der env map bei bumpmaps s. selfdxd



  { TxorDXCustomCamera }

  TxorDXCustomCamera = class(TxorDXCustomActiveTransform)
  private
    FLookAtSave: TxorDXVector;
    FPositionSave: TxorDXVector;
    FUpSave: TxorDXVector;
  protected
    FAngleOfView: Single;
    FIgnoreDirection: Boolean;
    FLookAt: TxorDXVector;
    FLowerBoundDistance: Single;
    FMinAngle: Real;
    FPosition: TxorDXVector;
    FRatio: Single;
    FUp: TxorDXVector;
    FUpperBoundDistance: Single;
    function GetActive: Boolean; override;
    function GetDirection: TxorDXVector;
    procedure SetDirection(Value: TxorDXVector); virtual;
    procedure SetLookAt(Value: TxorDXVector); virtual;
    procedure SetMinAngle(Value: Real);
    procedure SetPosition(Value: TxorDXVector); virtual;
    procedure SetUp(Value: TxorDXVector); virtual;
  public
    constructor Create(Scene: TxorDXScene);
    procedure ClearTransformation; override;
    procedure Activate; override;
    procedure Deactivate; override;
    procedure FixTransformation; override;
    procedure Move(x, y, z: Single); override;
    procedure MoveForward(D: Single; Transform: TxorDXCameraTransformation); virtual;
    procedure MoveHoriz(D: Single; Transform: TxorDXCameraTransformation); virtual;
    procedure MoveTo(x, y, z: Single; FixDirection: Boolean); virtual;
    procedure MoveVert(D: Single; Transform: TxorDXCameraTransformation); virtual;
    procedure RotateAround(x, y, z, axis_x, axis_y, axis_z, Angle: Single); virtual;
    procedure RotateAxis(x, y, z, Angle: Single); override;
    procedure RotateHoriz(Angle: Single; Transform: TxorDXCameraTransformation); virtual;
    procedure RotateSlope(Angle: Single; Transform: TxorDXCameraTransformation); virtual;
    procedure RotateVert(Angle: Single; Transform: TxorDXCameraTransformation); virtual; abstract;
    procedure RotateX(Angle: Single); override;
    procedure RotateY(Angle: Single); override;
    procedure RotateZ(Angle: Single); override;
    procedure Scale(x, y, z: Single); override;
    procedure SetStandard; virtual; abstract;
    procedure UndoTransformation; override;
    property AngleOfView: Single read FAngleOfView write FAngleOfView;
    property Direction: TxorDXVector read GetDirection write SetDirection;
    property IgnoreDirection: Boolean read FIgnoreDirection write FIgnoreDirection;
    property LookAt: TxorDXVector read FLookAt write SetLookAt;
    property LowerBoundDistance: Single read FLowerBoundDistance write FLowerBoundDistance;
    property MinAngle: Real read FMinAngle write SetMinAngle;
    property Position: TxorDXVector read FPosition write SetPosition;
    property Ratio: Single read FRatio write FRatio;
    property Up: TxorDXVector read FUp write SetUp;
    property UpperBoundDistance: Single read FUpperBoundDistance write FUpperBoundDistance;
  end;

  { TxorDXCamera }

  TxorDXCamera = class(TxorDXCustomCamera)
  public
    procedure RotateVert(Angle: Single; Transform: TxorDXCameraTransformation); override;
    procedure SetStandard; override;
  end;

  { TxorDXFreeCam }

  TxorDXFreeCam = class(TxorDXCustomCamera)
  public
    procedure RotateVert(Angle: Single; Transform: TxorDXCameraTransformation); override;
    procedure SetStandard; override;
  end;

  { TxorDXCustomLight }

  TxorDXCustomLight = class(TxorDXCustomActiveTransform)
  private
    FDirectionSave: TxorDXVector;
    FPositionSave: TxorDXVector;
  protected
    FActive: Boolean;
    FNumber: LongWord;
    FProperties: TxorDXLightProperties;
    function GetActive: Boolean; override;
    function GetDirection: TxorDXVector;
    function GetPosition: TxorDXVector;
    function GetProperties: PxorDXLightProperties;
    function GetValue(Index: Integer): Single;
    procedure SetDirection(Value: TxorDXVector);
    procedure SetPosition(Value: TxorDXVector);
    procedure SetValue(Index: Integer; Value: Single);
  public
    constructor Create(Scene: TxorDXScene);
    procedure Activate; override;
    procedure ClearTransformation; override;
    procedure Deactivate; override;
    procedure FixTransformation; override;
    procedure Move(x, y, z: Single); override;
    procedure RotateAxis(x, y, z, Angle: Single); override;
    procedure RotateX(Angle: Single); override;
    procedure RotateY(Angle: Single); override;
    procedure RotateZ(Angle: Single); override;
    procedure Scale(x, y, z: Single); override;
    procedure SetAmbientColor(r, g, b, Alpha: Single); overload;                //diffuse color will lower to this color
    procedure SetAmbientColor(r, g, b, Alpha: LongWord); overload;
    procedure SetAmbientColor(r, g, b: LongWord; Alpha: Single); overload;
    procedure SetDiffuseColor(r, g, b, Alpha: Single); overload;                //color of light
    procedure SetDiffuseColor(r, g, b, Alpha: LongWord); overload;
    procedure SetDiffuseColor(r, g, b: LongWord; Alpha: Single); overload;
    procedure SetSpecularColor(r, g, b, Alpha: Single); overload;               //object specular color
    procedure SetSpecularColor(r, g, b, Alpha: LongWord); overload;
    procedure SetSpecularColor(r, g, b: LongWord; Alpha: Single); overload;
    procedure UndoTransformation; override;
    property AmbientColor: TColor index 2 read GetTColor write SetTColor;
    property AmbientAlpha: Single index 2 read GetAlpha write SetAlpha;
    property Active: Boolean read FActive;
    property DiffuseColor: TColor index 3 read GetTColor write SetTColor;
    property DiffuseAlpha: Single index 3 read GetAlpha write SetAlpha;
    property Number: LongWord read FNumber;
    property Properties: PxorDXLightProperties read GetProperties;
    property SpecularColor: TColor index 4 read GetTColor write SetTColor;
    property SpecularAlpha: Single index 4 read GetAlpha write SetAlpha;
  end;

  { TxorDXPointLight }

  TxorDXPointLight = class(TxorDXCustomLight)
  public
    constructor Create(Scene: TxorDXScene);
    property Attenuation0: Single index 0 read GetValue write SetValue;
    property Attenuation1: Single index 1 read GetValue write SetValue;
    property Attenuation2: Single index 2 read GetValue write SetValue;
    property Position: TxorDXVector read GetPosition write SetPosition;
    property Range: Single index 3 read GetValue write SetValue;
  end;

  { TxorDXSpotLight }

  TxorDXSpotLight = class(TxorDXCustomLight)
  public
    constructor Create(Scene: TxorDXScene);
    property Direction: TxorDXVector read GetDirection write SetDirection;
    property FallOf: Single index 6 read GetValue write SetValue;
    property Attenuation0: Single index 0 read GetValue write SetValue;
    property Attenuation1: Single index 1 read GetValue write SetValue;
    property Attenuation2: Single index 2 read GetValue write SetValue;
    property Phi: Single index 5 read GetValue write SetValue;
    property Position: TxorDXVector read GetPosition write SetPosition;
    property Range: Single index 3 read GetValue write SetValue;
    property Theta: Single index 4 read GetValue write SetValue;
  end;

  { TxorDXDirectionalLight }

  TxorDXDirectionalLight = class(TxorDXCustomLight)
  public
    constructor Create(Scene: TxorDXScene);
    property Direction: TxorDXVector read GetDirection write SetDirection;
  end;

  { TxorDXCustomInsertObject }

  TxorDXCustomInsertObject = class(TxorDXCustomTransform)
  protected
    FMaterial: TxorDXMaterial;
    FVisible: Boolean;
    procedure InsertByShadow; virtual;                                          //Reintroduce all information for draw without insert
    procedure SetPower(const Value: Single);
    function GetMaterialPointer: PxorDXMaterial;
    function GetPower: Single;
  public
    constructor Create(Scene: TxorDXScene);
    procedure Insert; overload; virtual;
    procedure Draw; virtual;
    procedure SetMaterialAmbientColor(r, g, b, Alpha: Single); overload;        //relflecting color of ambient light color
    procedure SetMaterialAmbientColor(r, g, b, Alpha: LongWord); overload;
    procedure SetMaterialAmbientColor(r, g, b: LongWord; Alpha: Single); overload;
    procedure SetMaterialDiffuseColor(r, g, b, Alpha: Single); overload;        //relflecting color of direct light diffuse color
    procedure SetMaterialDiffuseColor(r, g, b, Alpha: LongWord); overload;
    procedure SetMaterialDiffuseColor(r, g, b: LongWord; Alpha: Single); overload;
    procedure SetMaterialEmissiveColor(r, g, b, Alpha: Single); overload;       //object light color
    procedure SetMaterialEmissiveColor(r, g, b, Alpha: LongWord); overload;
    procedure SetMaterialEmissiveColor(r, g, b: LongWord; Alpha: Single); overload;
    procedure SetMaterialSpecularColor(r, g, b, Alpha: Single); overload;       //relflecting color of direct light specular color
    procedure SetMaterialSpecularColor(r, g, b, Alpha: LongWord); overload;
    procedure SetMaterialSpecularColor(r, g, b: LongWord; Alpha: Single); overload;
    procedure SetStandard; virtual; abstract;
    property Material: PxorDXMaterial read GetMaterialPointer;
    property MaterialAmbientAlpha: Single index 5 read GetAlpha write SetAlpha;
    property MaterialAmbientColor: TColor index 5 read GetTColor write SetTColor;
    property MaterialDiffuseAlpha: Single index 6 read GetAlpha write SetAlpha;
    property MaterialDiffuseColor: TColor index 6 read GetTColor write SetTColor;
    property MaterialEmissiveAlpha: Single index 7 read GetAlpha write SetAlpha;
    property MaterialEmissiveColor: TColor index 7 read GetTColor write SetTColor;
    property MaterialSpecularAlpha: Single index 8 read GetAlpha write SetAlpha;
    property MaterialSpecularColor: TColor index 8 read GetTColor write SetTColor;
    property Power: Single read GetPower write SetPower;
    property Visible: Boolean read FVisible write FVisible;
  end;

  { TxorDXPolygon }

  TxorDXPolygon = class(TxorDXCustomInsertObject)
  private
    FVertexCount: Cardinal;
    FVertexFlags: TxorDXVertexFlag;
    FVertexSize: TxorDXVertexSize;
    FVertexMem: TxorDXVertexSize;
  protected
    FDrawAllVertices: Boolean;
    FDrawMode: TxorDXPrimitiveType;
    FDrawPrimCount: Cardinal;
    FDrawStart: Cardinal;
    FVertices: TxorDXVertexArray;
    function GetMaxPrimCount: Cardinal; virtual;
    function GetVertexPointer(Idx: Cardinal): Pointer; overload;
    procedure InitNormalAndTextureVectors; virtual;
    procedure SetDrawAll(Value: Boolean);
    procedure SetDrawMode(Value: TxorDXPrimitiveType);
    procedure SetDrawPrimCount(Value: Cardinal);
    procedure SetDrawStart(Value: Cardinal);
  public
    constructor Create(Scene: TxorDXScene; VertexCount: Cardinal);
    constructor CreateParam(Scene: TxorDXScene; VertexCount: Cardinal; VertexSize: TxorDXVertexSize; VertexFlags: TxorDXVertexFlag);
    destructor Destroy; override;
    function Center: TxorDXVector;
    procedure Draw; override;
    procedure Insert; override;
    procedure MoveCenter(MoveCenterToNull: Boolean);
    procedure SetStandard; override;
    property DrawAll: Boolean read FDrawAllVertices write SetDrawAll;
    property DrawMode: TxorDXPrimitiveType read FDrawMode write SetDrawMode;
    property DrawStart: Cardinal read FDrawStart write SetDrawStart;
    property DrawPrimCount: Cardinal read FDrawPrimCount write SetDrawPrimCount;
    property MaxPrimCount: Cardinal read GetMaxPrimCount;
    property VertexCount: Cardinal read FVertexCount;
    property Vertex[Idx: Cardinal]: Pointer read GetVertexPointer;
    property VertexFlags: TxorDXVertexFlag read FVertexFlags;
    property VertexSize: TxorDXVertexSize read FVertexSize;
    property Vertices: TxorDXVertexArray read FVertices;
  end;

  { TxorDXPointAdapter }

  TxorDXPointAdapter = class(TxorDXPolygon) //DONE 5: herausfinden, warum die punkte beim wegzoomen grer werden.
  private
    FColor: TxorDXColor;
    FPointSize: Single;
    FPreviousObject: TxorDXPolygon;
    FUseColor: Boolean;
  public
    constructor Create(Scene: TxorDXScene; MaxVertexCount: Cardinal);
    procedure Insert; overload; override;
    procedure Insert(Polygon: TxorDXPolygon); overload; virtual;
    procedure SetColor(r, g, b, Alpha: LongWord);
    property Alpha: Single index 11 read GetAlpha write SetAlpha;
    property Color: TColor index 11 read GetTColor write SetTColor;
    property PointSize: Single read FPointSize write FPointSize;
    property UseColor: Boolean read FUseColor write FUseColor;
  end;

  { TxorDXShadow }

  TxorDXShadow = class(TxorDXPolygon)
  protected
    FColor: TxorDXColor;
    FDeactivateFog: Boolean;
    FLightPosition: TD3DXVector4;
    FPlane: TD3DXPlane;
    FPreviousObject: TxorDXPolygon;
    function GetLightPositionVector: TxorDXVector;
    function GetLightScattering: Single;
    procedure SetLightPositionVector(const Value: TxorDXVector);
  public
    constructor Create(Scene: TxorDXScene; MaxVertexCount: Cardinal); //MaxVertexCount is the max vertexcount of a shadowed (=inserted) polygon
    procedure Draw; override;
    procedure Insert; overload; override;
    procedure Insert(Polygon: TxorDXPolygon); overload; virtual;
    procedure SetColor(r, g, b, Alpha: LongWord);
    procedure SetLightPosition(x, y, z: Single);
    procedure SetLightScattering(Scattering: Single);
    procedure SetStandard; override;
    property Alpha: Single index 9 read GetAlpha write SetAlpha;
    property DeactivateFog: Boolean read FDeactivateFog write FDeactivateFog;
    property LightPosition: TxorDXVector read GetLightPositionVector write SetLightPositionVector;
    property LightScattering: Single read GetLightScattering write SetLightScattering;
    property Plane: TxorDXPlane read FPlane write FPlane;
    property Color: TColor index 9 read GetTColor write SetTColor;
  end;

  { TxorDXIndexedPolygon }

  TxorDXIndexedPolygon = class(TxorDXPolygon)
  private
    FIndexCount: Cardinal;
    FIndexMem: Cardinal;
  protected
    FDrawMinUsedVertexIndex: Cardinal;
    FDrawUsedVertexCount: Cardinal;
    FIndices: TxorDXIndexArray;
    function GetIndex(Idx: Cardinal): TxorDXIndex;
    function GetIndex32(Idx: Cardinal): TxorDXIndex32;
    function GetLinePointer(Idx: Cardinal): PxorDXLine;
    function GetLinePointer32(Idx: Cardinal): PxorDXLine32;
    function GetMaxPrimCount: Cardinal; override;
    function GetTrianglePointer(Idx: Cardinal): PxorDXTriangle;
    function GetTrianglePointer32(Idx: Cardinal): PxorDXTriangle32;
    procedure InitNormalAndTextureVectors; override;
    procedure InsertByShadow; override;
    procedure SetIndex(Idx: Cardinal; Value: TxorDXIndex);
    procedure SetIndex32(Idx: Cardinal; Value: TxorDXIndex32);
    procedure SetIndices; virtual;
  public
    constructor Create(Scene: TxorDXScene; VertexCount, IndexCount: Cardinal);
    constructor CreateParam(Scene: TxorDXScene; VertexCount, IndexCount: Cardinal; VertexSize: TxorDXVertexSize; VertexFlags: TxorDXVertexFlag);
    destructor Destroy; override;
    procedure Draw; override;
    procedure SetStandard; override;
    property DrawMinUsedVertexIndex: Cardinal read FDrawMinUsedVertexIndex write FDrawMinUsedVertexIndex;
    property DrawUsedVertexCount: Cardinal read FDrawUsedVertexCount write FDrawUsedVertexCount;
    property Idx[Idx: Cardinal]: TxorDXIndex read GetIndex write SetIndex;
    property Idx32[Idx: Cardinal]: TxorDXIndex32 read GetIndex32 write SetIndex32;
    property IndexCount: Cardinal read FIndexCount;
    property Indices: TxorDXIndexArray read FIndices;
    property Triangle[Idx: Cardinal]: PxorDXTriangle read GetTrianglePointer;
    property Triangle32[Idx: Cardinal]: PxorDXTriangle32 read GetTrianglePointer32;
    property Line[Idx: Cardinal]: PxorDXLine read GetLinePointer;
    property Line32[Idx: Cardinal]: PxorDXLine32 read GetLinePointer32;
    property Point[Idx: Cardinal]: TxorDXIndex read GetIndex write SetIndex;
    property Point32[Idx: Cardinal]: TxorDXIndex32 read GetIndex32 write SetIndex32;
  end;

  { TxorDXCustomMap }

  TxorDXCustomMap = class(TxorDXIndexedPolygon)
  private
    FMapHeight: Cardinal;
    FMapWidth: Cardinal;
  protected
    function GetVertexPointer(u, v: Cardinal): Pointer; overload;
    procedure InitNormalAndTextureVectors; override;
  public
    property MapHeight: Cardinal read FMapHeight;
    property MapWidth: Cardinal read FMapWidth;
    property Vertex[u, v: Cardinal]: Pointer read GetVertexPointer;
  end;

  { TxorDXGridMap }

  TxorDXGridMap = class(TxorDXCustomMap)
  protected
    procedure SetIndices; override;
  public
    constructor Create(Scene: TxorDXScene; MapWidth, MapHeight: Cardinal);
    constructor CreateParam(Scene: TxorDXScene; Width, Height: Cardinal; VertexSize: TxorDXVertexSize; VertexFlags: TxorDXVertexFlag);
    procedure SetStandard; override;
  end;

  { TxorDXMap }

  TxorDXMap = class(TxorDXCustomMap)
  protected
    procedure SetIndices; override;
  public
    constructor Create(Scene: TxorDXScene; MapWidth, MapHeight: Cardinal);
    constructor CreateParam(Scene: TxorDXScene; MapWidth, MapHeight: Cardinal; VertexSize: TxorDXVertexSize; VertexFlags: TxorDXVertexFlag);
    procedure SetStandard; override;
  end;

  { TxorDXRect }

  TxorDXRect = class(TxorDXMap)
  protected
    FRectWidth: Single;
    FRectHeight: Single;
  public
    constructor Create(Scene: TxorDXScene; MapWidth, MapHeight: Cardinal; RectWidth, RectHeight: Single);
    constructor CreateParam(Scene: TxorDXScene; MapWidth, MapHeight: Cardinal; RectWidth, RectHeight: Single; VertexSize: TxorDXVertexSize; VertexFlags: TxorDXVertexFlag);
    procedure SetStandard; override;
    property RectWidth: Single read FRectWidth;
    property RectHeigth: Single read FRectHeight;
  end;

implementation

uses Math;

//==============================================================================
{ Tools }

const TickStart: LongWord = 0;

function GetAlp(Col: TxorDXColor): LongWord;
begin
  Result := Col shr 24;
end;

function GetAlpVal(Col: TxorDXColor): Single;
begin
  Result := (Col shr 24) / 255;
end;

function GetAngle(V1, V2: TxorDXVector): Single;
begin
  D3DXVec3Normalize(V1, V1);
  D3DXVec3Normalize(V2, V2);
  Result := D3DXVec3Dot(V1, V2);
  if Abs(Result) > 1 then if Result > 0 then Result := 1 else Result := -1;
  Result := ArcCos(Result);
end;

function GetAngleProj(V1, V2: TxorDXVector): Single;
begin
  Result := GetAngle(V1, V2);
  if Result > Pi05 then Result := Pi - Result;
end;

function GetBlendLightOp(BOp: TxorDXBlendLightOperation): TxorDXTexOp;
begin
  Case BOp of
    bop1X: Result := xorDXTexOpMODULATE;
    bop2X: Result := xorDXTexOpMODULATE2X;
    bop4X: Result := xorDXTexOpMODULATE4X;
  else
    Result := xorDXTexOpDISABLE;
  end;
end;

function GetCol(Col: TColor): TxorDXColor;
var
  CCol: LongWord;
begin
  CCol := Abs(Col);
  Result := $FF000000 or ((CCol and $000000FF) shl 16) or (CCol and $0000FF00) or ((CCol and $00FF0000) shr 16); //col = 0 b g r; res = a, r, g, b; a=FF
end;

function GetColA(Col: TColor; Alpha: LongWord): TxorDXColor;
var
  CCol: LongWord;
begin
  CCol := Abs(Col);
  Result := ((Alpha and $000000FF) shl 24) or ((CCol and $000000FF) shl 16) or (CCol and $0000FF00) or ((CCol and $00FF0000) shr 16);
end;

function GetColS(Col: TColor; Alpha: Single): TxorDXColor;
var
  CCol: LongWord;
begin
  CCol := Abs(Col);
  Result := ((Round(Alpha * 255) and $000000FF) shl 24) or ((CCol and $000000FF) shl 16) or (CCol and $0000FF00) or ((CCol and $00FF0000) shr 16);
end;

function GetColRGB(r, g, b: LongWord): TxorDXColor;
begin
  Result := ($FF000000) or (r shl 16) or (g shl 8) or b; //alpha = 255
end;

function GetColRGBA(r, g, b, Alpha: LongWord): TxorDXColor;
begin
  Result := (Alpha shl 24) or (r shl 16) or (g shl 8) or b;
end;

function GetColRGBS(r, g, b: LongWord; Alpha: Single): TxorDXColor;
begin
  Result := (Round(Alpha * 255) shl 24) or (r shl 16) or (g shl 8) or b;
end;

function GetColValA(r, g, b, Alpha: LongWord): TxorDXColorValue;
begin
  Result.r := r / 255;
  Result.g := g / 255;
  Result.b := b / 255;
  Result.a := Alpha / 255;
end;

function GetColValS(r, g, b: LongWord; Alpha: Single): TxorDXColorValue;
begin
  Result.r := r / 255;
  Result.g := g / 255;
  Result.b := b / 255;
  Result.a := Alpha;
end;

function GetColValSS(r, g, b, Alpha: Single): TxorDXColorValue;
begin
  Result.r := r;
  Result.g := g;
  Result.b := b;
  Result.a := Alpha;
end;

function GetCoord(x, y: Single): TxorDXCoord;
begin
  Result := D3DXVector2(x, y);
end;

function GetEntryCount(Mode: TxorDXPrimitiveType; PrimCount: Cardinal): Cardinal; //returns the minimal number of points used for PrimCount primitives in this mode
begin
  Case Mode of
    D3DPT_POINTLIST: Result := PrimCount;                                       //einzelne punkte
    D3DPT_LINELIST: Result := PrimCount shl 1;                                  //linie zwischen 2a und 2a+1
    D3DPT_LINESTRIP: Result := PrimCount + 1;                                   //linie zwischen zwei aufeinanderfolgenden punkten
    D3DPT_TRIANGLELIST: Result := PrimCount * 3;                                //dreieck zwischen 3a, 3a+1 und 3a+2
    D3DPT_TRIANGLESTRIP: Result := PrimCount + 2;                               //dreieck zwischen je 3 aufeinanderfolgenden punkten
    D3DPT_TRIANGLEFAN: Result := PrimCount + 2;                                 //dreieck zwischen je zwei aufeinanderfolgenden punkten und mit dem ersten
  else
    Result := PrimCount * 3;
  end;
end;

function GetLine(Idx1, Idx2: TxorDXIndex): TxorDXLine;
begin
  Result.V1 := Idx1;
  Result.V2 := Idx2;
end;

function GetLine32(Idx1, Idx2: TxorDXIndex32): TxorDXLine32;
begin
  Result.V1 := Idx1;
  Result.V2 := Idx2;
end;

function GetPlaneFromPoints(V1, V2, V3: TxorDXVector): TxorDXPlane;
begin
  D3DXPlaneFromPoints(Result, V1, V2, V3);
end;

function GetPlaneFromPointVector(P, V: TxorDXVector): TxorDXPlane;
begin
  D3DXVec3Normalize(V, V);
  D3DXPlaneFromPointNormal(Result, P, V);
end;

function GetPlaneXY(Z: Single): TxorDXPlane;
var
  V, N: TxorDXVector;
begin
  V := GetVector(0, 0, Z);
  N := GetVector(0, 0, 1);
  D3DXPlaneFromPointNormal(Result, V, N);
end;

function GetPlaneXZ(Y: Single): TxorDXPlane;
var
  V, N: TxorDXVector;
begin
  V := GetVector(0, Y, 0);
  N := GetVector(0, 1, 0);
  D3DXPlaneFromPointNormal(Result, V, N);
end;

function GetPlaneYZ(X: Single): TxorDXPlane;
var
  V, N: TxorDXVector;
begin
  V := GetVector(X, 0, 0);
  N := GetVector(1, 0, 0);
  D3DXPlaneFromPointNormal(Result, V, N);
end;

function GetPrimCount(Mode: TxorDXPrimitiveType; EntryCount: Cardinal): Cardinal; //returns the number of primitives using PointCount points in this mode
begin
  Case Mode of
    D3DPT_POINTLIST: Result := EntryCount;                                      //einzelne punkte
    D3DPT_LINELIST: Result := EntryCount div 2;                                 //linie zwischen 2a und 2a+1
    D3DPT_LINESTRIP: Result := Max(1, EntryCount) - 1;                          //linie zwischen zwei aufeinanderfolgenden punkten
    D3DPT_TRIANGLELIST: Result := EntryCount div 3;                             //dreieck zwischen 3a, 3a+1 und 3a+2
    D3DPT_TRIANGLESTRIP: Result := Max(2, EntryCount) - 2;                      //dreieck zwischen je 3 aufeinanderfolgenden punkten
    D3DPT_TRIANGLEFAN: Result := Max(2, EntryCount) - 2;                        //dreieck zwischen je zwei aufeinanderfolgenden punkten und mit dem ersten
  else
    Result := 0;
  end;
end;

function GetRadiansMod2Pi(Rad: Single): Single;
begin
  Result := Rad - Int(Rad/Pi2) * Pi2;
  if Result < 0 then Result := Result + Pi2;
end;

procedure GetResolution(Res: TxorDXResolution; var Width, Height, Depth: Cardinal);
begin
  Case Res of
    rs800x600x16:  begin Width := 800; Height := 600; Depth := 16; end;
    rs800x600x32:  begin Width := 800; Height := 600; Depth := 32; end;
    rs1024x768x16: begin Width := 1024; Height := 768; Depth := 16; end;
    rs1024x768x32: begin Width := 1024; Height := 768; Depth := 32; end;
  else
    Width := 0; Height := 0; Depth := 0;
  end;
end;

function GetTCol(Col: TxorDXColor): TColor; 
begin
  Result := ((Col and $00FF0000) shr 16) or (Col and $0000FF00) or ((Col and $000000FF) shl 16); //col = a r g b; res = 0 b g r
end;

function GetTCol(Col: TxorDXColorValue): TColor;
var
  b, g, r: LongWord;
begin
  r := Round(Abs(255 * Col.r));  g := Round(Abs(255 * Col.g));  b := Round(Abs(255 * Col.b));
  Result := (b shl 16) or (g shl 8) or r;
end;

function GetTriangle(Idx1, Idx2, Idx3: TxorDXIndex): TxorDXTriangle;
begin
  Result.V1 := Idx1;
  Result.V2 := Idx2;
  Result.V3 := Idx3;
end;

function GetTriangle32(Idx1, Idx2, Idx3: TxorDXIndex32): TxorDXTriangle32;
begin
  Result.V1 := Idx1;
  Result.V2 := Idx2;
  Result.V3 := Idx3;
end;

function GetVector(x, y, z: Single): TxorDXVector;
begin
  Result := D3DXVector3(x, y, z);
end;

function GetVertexClassic(x, y, z, n1, n2, n3: Single; r, g, b, Alpha: LongWord): TxorDXVertexClassic; //red, green, blue, alpha
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Normal  := D3DXVector3(n1, n2, n3);
    Color := GetColRGBA(r, g, b, Alpha);
    end;
end;

function GetVertexClassic(x, y, z, n1, n2, n3: Single; r, g, b: LongWord; Alpha: Single): TxorDXVertexClassic;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Normal  := D3DXVector3(n1, n2, n3);
    Color := GetColRGBS(r, g, b, Alpha);
    end;
end;

function GetVertexClassic(x, y, z, n1, n2, n3: Single; Col: TColor; Alpha: LongWord): TxorDXVertexClassic;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Normal  := D3DXVector3(n1, n2, n3);
    Color := GetColA(Col, Alpha);
    end;
end;

function GetVertexClassic(x, y, z, n1, n2, n3: Single; Col: TColor; Alpha: Single): TxorDXVertexClassic;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Normal  := D3DXVector3(n1, n2, n3);
    Color := GetColS(Col, Alpha);
    end;
end;

function GetVertexColored(x, y, z: Single; r, g, b, Alpha: LongWord): TxorDXVertexColored; //red, green, blue, alpha
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Color := GetColRGBA(r, g, b, Alpha);
    end;
end;

function GetVertexColored(x, y, z: Single; r, g, b: LongWord; Alpha: Single): TxorDXVertexColored;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Color := GetColRGBS(r, g, b, Alpha);
    end;
end;

function GetVertexColored(x, y, z: Single; Col: TColor; Alpha: LongWord): TxorDXVertexColored;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Color := GetColA(Col, Alpha);
    end;
end;

function GetVertexColored(x, y, z: Single; Col: TColor; Alpha: Single): TxorDXVertexColored;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Color := GetColS(Col, Alpha);
    end;
end;

function GetVertexPlain(x, y, z: Single): TxorDXVertexPlain;
begin
  Result.Position := D3DXVector3(x, y, z);
end;

function GetVertexPoint(x, y, z, n1, n2, n3, PSize: Single; r, g, b, Alpha: LongWord): TxorDXVertexPoint; //red, green, blue, alpha
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Normal  := D3DXVector3(n1, n2, n3);
    Color := GetColRGBA(r, g, b, Alpha);
    Size := PSize;
    end;
end;

function GetVertexPoint(x, y, z, n1, n2, n3, PSize: Single; r, g, b: LongWord; Alpha: Single): TxorDXVertexPoint;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Normal  := D3DXVector3(n1, n2, n3);
    Color := GetColRGBS(r, g, b, Alpha);
    Size := PSize;
    end;
end;

function GetVertexPoint(x, y, z, n1, n2, n3, PSize: Single; Col: TColor; Alpha: LongWord): TxorDXVertexPoint;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Normal  := D3DXVector3(n1, n2, n3);
    Color := GetColA(Col, Alpha);
    Size := PSize;
    end;
end;

function GetVertexPoint(x, y, z, n1, n2, n3, PSize: Single; Col: TColor; Alpha: Single): TxorDXVertexPoint;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Normal  := D3DXVector3(n1, n2, n3);
    Color := GetColS(Col, Alpha);
    Size := PSize;
    end;
end;

function GetVertexTexture(x, y, z, n1, n2, n3, tx, ty: Single; r, g, b, Alpha: LongWord): TxorDXVertexTexture;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Normal  := D3DXVector3(n1, n2, n3);
    Color := GetColRGBA(r, g, b, Alpha);
    TexCoordinates := D3DXVector2(tx, ty);
    end;
end;

function GetVertexTexture(x, y, z, n1, n2, n3, tx, ty: Single; r, g, b: LongWord; Alpha: Single): TxorDXVertexTexture;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Normal  := D3DXVector3(n1, n2, n3);
    Color := GetColRGBS(r, g, b, Alpha);
    TexCoordinates := D3DXVector2(tx, ty);
    end;
end;

function GetVertexTexture(x, y, z, n1, n2, n3, tx, ty: Single; Col: TColor; Alpha: LongWord): TxorDXVertexTexture;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Normal  := D3DXVector3(n1, n2, n3);
    Color := GetColA(Col, Alpha);
    TexCoordinates := D3DXVector2(tx, ty);
    end;
end;

function GetVertexTexture(x, y, z, n1, n2, n3, tx, ty: Single; Col: TColor; Alpha: Single): TxorDXVertexTexture;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Normal  := D3DXVector3(n1, n2, n3);
    Color := GetColS(Col, Alpha);
    TexCoordinates := D3DXVector2(tx, ty);
    end;
end;

function GetVertexTexture2(x, y, z, n1, n2, n3, tx, ty, tx2, ty2: Single; r, g, b, Alpha: LongWord): TxorDXVertexTexture2;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Normal  := D3DXVector3(n1, n2, n3);
    Color := GetColRGBA(r, g, b, Alpha);
    TexCoordinates := D3DXVector2(tx, ty);
    TexCoordinates2 := D3DXVector2(tx2, ty2);
    end;
end;

function GetVertexTexture2(x, y, z, n1, n2, n3, tx, ty, tx2, ty2: Single; r, g, b: LongWord; Alpha: Single): TxorDXVertexTexture2;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Normal  := D3DXVector3(n1, n2, n3);
    Color := GetColRGBS(r, g, b, Alpha);
    TexCoordinates := D3DXVector2(tx, ty);
    TexCoordinates2 := D3DXVector2(tx2, ty2);
    end;
end;

function GetVertexTexture2(x, y, z, n1, n2, n3, tx, ty, tx2, ty2: Single; Col: TColor; Alpha: LongWord): TxorDXVertexTexture2;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Normal  := D3DXVector3(n1, n2, n3);
    Color := GetColA(Col, Alpha);
    TexCoordinates := D3DXVector2(tx, ty);
    TexCoordinates2 := D3DXVector2(tx2, ty2);
    end;
end;

function GetVertexTexture2(x, y, z, n1, n2, n3, tx, ty, tx2, ty2: Single; Col: TColor; Alpha: Single): TxorDXVertexTexture2;
begin
  with Result do
    begin
    Position := D3DXVector3(x, y, z);
    Normal  := D3DXVector3(n1, n2, n3);
    Color := GetColS(Col, Alpha);
    TexCoordinates := D3DXVector2(tx, ty);
    TexCoordinates2 := D3DXVector2(tx2, ty2);
    end;
end;

procedure InitRotationPerSecond;
begin
  TickStart := GetTickCount;
end;

function RotationsPerSecond(R: Single): Single;
begin
  Result := GetTickCount;
  if TickStart > Result then TickStart := 0;
  Result := Frac((Result - TickStart) * R / 1000) * Pi2;
end;

procedure SetAlp(var Col: TxorDXColor; Alpha: LongWord);
begin
  Col := (Col and $00FFFFFF) or (Alpha shl 24);
end;

procedure SetAlpS(var Col: TxorDXColor; Alpha: Single);
begin
  Col := (Col and $00FFFFFF) or (Round(255 * Alpha) shl 24);
end;

procedure SetCol(var Col: TxorDXColor; TCol: TColor);
var
  CCol: LongWord;
begin
  CCol := Abs(TCol);
  Col := (Col and $FF000000) or ((CCol and $000000FF) shl 16) or (CCol and $0000FF00) or ((CCol and $00FF0000) shr 16);
end;

procedure SetCol(var Col: TxorDXColorValue; TCol: TColor); overload;
begin
  with Col do
    begin
    r := (TCol and $000000FF) / 255;
    g := ((TCol and $0000FF00) shr 8) / 255;
    b := ((TCol and $00FF0000) shr 16) / 255;
    end;
end;

function VertexFormatHasClassicStructure(VertexFlag: TxorDXVertexFlag): Boolean;
begin
  Result := (VertexFlag and $007F) = xorDXVertexFlagClassic;
end;

function VertexFormatHasTextureStructure(VertexFlag: TxorDXVertexFlag): LongWord;
begin
  Result := ( (VertexFlag and $00000F00) shr 8 )                                   //tex-coordinaten anzahl
            * LongWord(Ord((VertexFlag and $000000FF) = xorDXVertexFlagClassic));  //vor tex-coord klassich
end;

//==============================================================================
{ TxorDXCustomObject }

constructor TxorDXCustomObject.Create(Scene: TxorDXScene);
begin
  inherited Create;
  if not Assigned(Scene) then ErrorTest(E_INVALIDARG, @errBadArgument, GetErrorMsgD3D, True);
  FScene := Scene;
end;

function TxorDXCustomObject.ErrorTest(HR: HResult; Msg: String; InternalMsg: TxorDXErrorMessage; ThrowException: Boolean): Boolean;
begin
  if Windows.Failed(HR) then
    begin
    Result := False;
    if Msg <> '' then Msg := Self.ClassName + ': ' + Msg;
    if Assigned(InternalMsg) then
      if Msg = '' then Msg := D3DXErrorString(HR) + ': ' + InternalMsg(HR)
      else
        Msg := Msg + Enter + Enter + '(' + D3DXErrorString(HR) + ': ' + InternalMsg(HR) + ')';
    if Assigned(FScene) then with FScene do
      begin
      ClearError;
      FLastError := HR;
      FLastErrorMsg := Msg;
      end;
    if ThrowException then raise Exception.Create(Msg);
    end
  else
    begin
    Result := True;
    if Assigned(FScene) then with FScene do
      begin
      ClearError;
      FLastError := HR;
      FLastErrorMsg := D3DXErrorString(HR) + ': ' + InternalMsg(HR);
      end;
    end;
end;

function TxorDXCustomObject.ErrorTest(HR: HResult; ResourceMsg: PResStringRec; InternalMsg: TxorDXErrorMessage; ThrowException: Boolean): Boolean;
begin
  Result := ErrorTest(HR, LoadResString(ResourceMsg), InternalMsg, ThrowException);
end;

function TxorDXCustomObject.GetAlpha(Index: Integer): Single;
var
  C: TxorDXColor;
begin
  Case Index of
    0: Result := GetAlpVal(TxorDXFog(Self).FColor);
    1: Result := GetAlpVal(TxorDXScene(Self).FBackgroundColor);
    2: Result := TxorDXCustomLight(Self).FProperties.Ambient.a;
    3: Result := TxorDXCustomLight(Self).FProperties.Diffuse.a;
    4: Result := TxorDXCustomLight(Self).FProperties.Specular.a;
    5: Result := TxorDXCustomInsertObject(Self).FMaterial.Ambient.a;
    6: Result := TxorDXCustomInsertObject(Self).FMaterial.Diffuse.a;
    7: Result := TxorDXCustomInsertObject(Self).FMaterial.Emissive.a;
    8: Result := TxorDXCustomInsertObject(Self).FMaterial.Specular.a;
    9: Result := GetAlpVal(TxorDXShadow(Self).FColor);
   10: begin //scene ambient light
       C := 0;
       ErrorTest(FScene.FDevice.GetRenderState(D3DRS_AMBIENT, LongWord(C)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
       Result := GetAlpVal(C);
       end;
   11: Result := GetAlpVal(TxorDXPointAdapter(Self).FColor);
  else
    Result := 0;
  end;
end;

function TxorDXCustomObject.GetTColor(Index: Integer): TColor;
var
  C: TxorDXColor;
begin
  Case Index of
    0: Result := GetTCol(TxorDXFog(Self).FColor);
    1: Result := GetTCol(TxorDXScene(Self).FBackgroundColor);
    2: Result := GetTCol(TxorDXCustomLight(Self).FProperties.Ambient);
    3: Result := GetTCol(TxorDXCustomLight(Self).FProperties.Diffuse);
    4: Result := GetTCol(TxorDXCustomLight(Self).FProperties.Specular);
    5: Result := GetTCol(TxorDXCustomInsertObject(Self).FMaterial.Ambient);
    6: Result := GetTCol(TxorDXCustomInsertObject(Self).FMaterial.Diffuse);
    7: Result := GetTCol(TxorDXCustomInsertObject(Self).FMaterial.Emissive);
    8: Result := GetTCol(TxorDXCustomInsertObject(Self).FMaterial.Specular);
    9: Result := GetTCol(TxorDXShadow(Self).FColor);
   10: begin
       C := 0;
       ErrorTest(FScene.FDevice.GetRenderState(D3DRS_AMBIENT, LongWord(C)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
       Result := GetTCol(C);
       end;
   11: Result := GetTCol(TxorDXPointAdapter(Self).FColor);
  else
    Result := clBlack;
  end;
end;

procedure TxorDXCustomObject.SetAlpha(const Index: Integer; const Value: Single);
var
  C: TxorDXColor;
begin
  Case Index of
    0: SetAlpS(TxorDXFog(Self).FColor, Value);
    1: SetAlpS(TxorDXScene(Self).FBackgroundColor, Value);
    2: TxorDXCustomLight(Self).FProperties.Ambient.a := Value;
    3: TxorDXCustomLight(Self).FProperties.Diffuse.a := Value;
    4: TxorDXCustomLight(Self).FProperties.Specular.a := Value;
    5: TxorDXCustomInsertObject(Self).FMaterial.Ambient.a := Value;
    6: TxorDXCustomInsertObject(Self).FMaterial.Diffuse.a := Value;
    7: TxorDXCustomInsertObject(Self).FMaterial.Emissive.a := Value;
    8: TxorDXCustomInsertObject(Self).FMaterial.Specular.a := Value;
    9: SetAlpS(TxorDXShadow(Self).FColor, Value);
   10: begin
       C := 0;
       ErrorTest(FScene.FDevice.GetRenderState(D3DRS_AMBIENT, LongWord(C)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
       SetAlpS(C, Value);
       ErrorTest(FScene.FDevice.SetRenderState(D3DRS_AMBIENT, C), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
       end;
   11: SetAlpS(TxorDXPointAdapter(Self).FColor, Value);
  end;
end;

procedure TxorDXCustomObject.SetTColor(const Index: Integer; const Value: TColor);
var
  C: TxorDXColor;
begin
  Case Index of
    0: SetCol(TxorDXFog(Self).FColor, Value);
    1: SetCol(TxorDXScene(Self).FBackgroundColor, Value);
    2: SetCol(TxorDXCustomLight(Self).FProperties.Ambient, Value);
    3: SetCol(TxorDXCustomLight(Self).FProperties.Diffuse, Value);
    4: SetCol(TxorDXCustomLight(Self).FProperties.Specular, Value);
    5: SetCol(TxorDXCustomInsertObject(Self).FMaterial.Ambient, Value);
    6: SetCol(TxorDXCustomInsertObject(Self).FMaterial.Diffuse, Value);
    7: SetCol(TxorDXCustomInsertObject(Self).FMaterial.Emissive, Value);
    8: SetCol(TxorDXCustomInsertObject(Self).FMaterial.Specular, Value);
    9: SetCol(TxorDXShadow(Self).FColor, Value);
   10: begin
       C := 0;
       ErrorTest(FScene.FDevice.GetRenderState(D3DRS_AMBIENT, LongWord(C)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
       SetCol(C, Value);
       ErrorTest(FScene.FDevice.SetRenderState(D3DRS_AMBIENT, C), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
       end;
   11: SetCol(TxorDXPointAdapter(Self).FColor, Value);
  end;
end;

{ TxorDXRendering }

procedure TxorDXRendering.GetRenderState(State: TxorDXRenderState; var Color: TColor; var Alpha: Single);
var
  DW: TxorDXColor;
begin
  ErrorTest(FScene.FDevice.GetRenderState(State, LongWord(DW)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  Color := GetTCol(DW);  Alpha := GetAlpVal(DW);
end;

procedure TxorDXRendering.GetRenderState(State: TxorDXRenderState; var Value: Boolean);
var
  DW: LongWord;
begin
  ErrorTest(FScene.FDevice.GetRenderState(State, DW), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  Value := DW <> 0;
end;

procedure TxorDXRendering.GetRenderState(State: TxorDXRenderState; var Value: LongWord);
begin
  ErrorTest(FScene.FDevice.GetRenderState(State, Value), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXRendering.GetRenderState(State: TxorDXRenderState; var Value: Single);
var
  DW: LongWord;
begin
  ErrorTest(FScene.FDevice.GetRenderState(State, DW), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  Value := PSingle(@DW)^;
end;

procedure TxorDXRendering.Render(State: TxorDXRenderState; Color: TColor; Alpha: Single);
var
  C: TxorDXColor;
begin
  C := GetColS(Color, Alpha);
  ErrorTest(FScene.FDevice.SetRenderState(State, LongWord(C)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXRendering.Render(State: TxorDXRenderState; Value: Boolean);
begin
  ErrorTest(FScene.FDevice.SetRenderState(State, Ord(Value)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXRendering.Render(State: TxorDXRenderState; Value: LongWord);
begin
  ErrorTest(FScene.FDevice.SetRenderState(State, Value), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXRendering.Render(State: TxorDXRenderState; Value: Single);
begin
  ErrorTest(FScene.FDevice.SetRenderState(State, PLongWord(@Value)^), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

{ TxorDXCustomInput }

constructor TxorDXCustomInput.Create(Scene: TxorDXScene);
begin
  inherited;
  FDevice := nil;
  if FScene.FInput = nil then
    ErrorTest(DirectInput8Create(HInstance, DIRECTINPUT_VERSION, IID_IDirectInput8, FScene.FInput, nil), @errInputCreate, GetErrorMsgInput, True);
  Inc(FScene.FActiveInputs);
end;

destructor TxorDXCustomInput.Destroy;
begin
  if FDevice <> nil then                                                        //free device
    begin
    ErrorTest(FDevice.Unacquire, @errInputUnacquire, GetErrorMsgInput, xorDXThrowAllOtherExceptions);
    FDevice := nil;
    end;
  Dec(FScene.FActiveInputs);
  if FScene.FActiveInputs = 0 then FScene.FInput := nil;                        //free input, if no more devices
  inherited;
end;

function TxorDXCustomInput.GetAttached: Boolean;
begin
  Result := GetCaps and ((FCaps.dwFlags and DIDC_ATTACHED) <> 0);
end;

function TxorDXCustomInput.GetCaps: Boolean;
begin
  FillChar(FCaps, SizeOf(FCaps), 0);
  FCaps.dwSize := SizeOf(FCaps);
  Result := ErrorTest(FDevice.GetCapabilities(FCaps), @errInputCaps, GetErrorMsgInput, xorDXThrowCapsException);
end;

{ TxorDXKeyboard }

constructor TxorDXKeyboard.Create(Scene: TxorDXScene);
begin
  inherited;
  ErrorTest(FScene.FInput.CreateDevice(GUID_SysKeyboard, FDevice, nil), @errInputKeyboard, GetErrorMsgInput, True);
  if Not GetAttached then ErrorTest(DI_NOTATTACHED, @errInput, GetErrorMsgInput, xorDXThrowAllOtherExceptions);
  ErrorTest(FDevice.SetDataFormat(@c_dfDIKeyboard), @errInputDataFormat, GetErrorMsgInput, True);
  ErrorTest(FDevice.SetCooperativeLevel(FScene.FHandleWindow, DISCL_BACKGROUND or DISCL_NONEXCLUSIVE), @errInputCooperation, GetErrorMsgInput, xorDXThrowAllOtherExceptions);
  ErrorTest(FDevice.Acquire, @errInputAcquire, GetErrorMsgInput, xorDXThrowAllOtherExceptions);
end;

function TxorDXKeyboard.Get: Boolean;
begin
  Result := ErrorTest(FDevice.GetDeviceState(SizeOf(FKeyboardState),@FKeyboardState), @errInputDeviceState, GetErrorMsgInput, xorDXThrowAllOtherExceptions);
  if not Result then
    if FScene.ErrorCode = DIERR_INPUTLOST then
      if ErrorTest(FDevice.Acquire, @errInputAcquire, GetErrorMsgInput, xorDXThrowAllOtherExceptions)
      then Result := ErrorTest(FDevice.GetDeviceState(SizeOf(FKeyboardState),@FKeyboardState), @errInputDeviceState, GetErrorMsgInput, xorDXThrowAllOtherExceptions);
end;

function TxorDXKeyboard.KeyPressed(Key: Byte): Boolean;
begin
  Result := (FKeyboardState[VK2DIK[Key]] and $80) <> 0;
end;

function TxorDXKeyboard.KeyPressedAny: Boolean;
var
  i: Byte;
begin
  Result := True;
  for i := 0 to 255 do
    if (FKeyboardState[i] and $80) <> 0 then Exit;
  Result := False;
end;

function TxorDXKeyboard.KeyPressedDIK(Key: Byte): Boolean;
begin
  Result := (FKeyboardState[Key] and $80) <> 0;
end;

{ TxorDXMouse }

constructor TxorDXMouse.Create(Scene: TxorDXScene);
begin
  inherited;
  FVisible := True;
  FX := 0; FY := 0; FZ := 0; FDX := 0; FDY := 0; FDZ := 0; FSpeed := 1;
  FClickedLeft := False; FClickedRight := False; FClickedButton3 := False; FClickedButton4 := False;
  FInvertX := False;
  FInvertY := False;
  FInvertZ := False;
  ErrorTest(FScene.FInput.CreateDevice(GUID_SysMouse, FDevice, nil), @errInputMouse, GetErrorMsgInput, True);
  if Not GetAttached then ErrorTest(DI_NOTATTACHED, @errInput, GetErrorMsgInput, xorDXThrowAllOtherExceptions);
  ErrorTest(FDevice.SetDataFormat(@c_dfDIMouse), @errInputDataFormat, GetErrorMsgInput, True);
  ErrorTest(FDevice.SetCooperativeLevel(FScene.FHandleWindow, DISCL_BACKGROUND or DISCL_NONEXCLUSIVE), @errInputCooperation, GetErrorMsgInput, xorDXThrowAllOtherExceptions);
  ErrorTest(FDevice.Acquire, @errInputAcquire, GetErrorMsgInput, xorDXThrowAllOtherExceptions);
end;

function TxorDXMouse.Get: Boolean;
label CopyState;
begin
  Result := False;
  if not ErrorTest(FDevice.GetDeviceState(SizeOf(TDIMouseState),@FMouseState), @errInputDeviceState, GetErrorMsgInput, xorDXThrowAllOtherExceptions) then
    with FMouseState do
      begin
      FDX := 0;  FDY := 0;  FDZ := 0;
      FClickedLeft := False;  FClickedRight := False;  FClickedButton3 := False;  FClickedButton4 := False;
      if FScene.ErrorCode = DIERR_INPUTLOST then
        if ErrorTest(FDevice.Acquire, @errInputAcquire, GetErrorMsgInput, xorDXThrowAllOtherExceptions) then
          if ErrorTest(FDevice.GetDeviceState(SizeOf(TDIMouseState),@FMouseState), @errInputDeviceState, GetErrorMsgInput, xorDXThrowAllOtherExceptions)
          then goto CopyState;
      Exit;
      end;
CopyState:
  Result := True;
  with FMouseState do
    begin
    FDX := FSpeed * lX * (1 - Ord(FInvertX) shl 1);
    FDY := FSpeed * lY * (Ord(FInvertY) shl 1 - 1);
    FDZ := FSpeed * lZ * (1 - Ord(FInvertZ) shl 1);
    FX := FX + FDX;  FY := FY + FDY;  FZ := FZ + FDZ;
    FClickedLeft := (rgbButtons[0] and  $80) <> 0;
    FClickedRight := (rgbButtons[1] and  $80) <> 0;
    FClickedButton3 := (rgbButtons[2] and  $80) <> 0;
    FClickedButton4 := (rgbButtons[3] and  $80) <> 0;
    end;
end;

procedure TxorDXMouse.SetVisible(Value: Boolean);
begin
  if FScene.FullScreen then
    begin
    ErrorTest(FDevice.Unacquire, @errInputUnacquire, GetErrorMsgInput, xorDXThrowAllOtherExceptions);
    if Value then ErrorTest(FDevice.SetCooperativeLevel(FScene.FHandleWindow, DISCL_FOREGROUND or DISCL_NONEXCLUSIVE), @errInputCooperation, GetErrorMsgInput, xorDXThrowAllOtherExceptions)
    else
      ErrorTest(FDevice.SetCooperativeLevel(FScene.FHandleWindow, DISCL_FOREGROUND or DISCL_EXCLUSIVE), @errInputCooperation, GetErrorMsgInput, xorDXThrowAllOtherExceptions);
    FVisible := Value;
    ErrorTest(FDevice.Acquire, @errInputAcquire, GetErrorMsgInput, xorDXThrowAllOtherExceptions);
    end
  else ErrorTest(E_Fail, @errInputAccess, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

{ TxorDXCustomState }

procedure TxorDXCustomState.Activate;
begin
  FIsActive := True;
end;

procedure TxorDXCustomState.Deactivate;
begin
  FIsActive := False;
end;

procedure TxorDXCustomState.SetActive(const Value: Boolean);
begin
  if Value then Activate else Deactivate
end;

{ TxorDXFog }

procedure TxorDXFog.Activate;
begin
  inherited;
  if FAutoColor then FColor := FScene.FBackgroundColor;
  ErrorTest(FScene.FDevice.SetRenderState(D3DRS_FOGENABLE, Ord(True)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  if FRadial then
    ErrorTest(FScene.FDevice.SetRenderState(D3DRS_RANGEFOGENABLE, Ord(True)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  ErrorTest(FScene.FDevice.SetRenderState(D3DRS_FOGCOLOR, FColor), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  ErrorTest(FScene.FDevice.SetRenderState(D3DRS_FOGVERTEXMODE, FVertexMode), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  ErrorTest(FScene.FDevice.SetRenderState(D3DRS_FOGTABLEMODE, FTableMode), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  ErrorTest(FScene.FDevice.SetRenderState(D3DRS_FOGDENSITY, PLongWord(@FDensity)^), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  ErrorTest(FScene.FDevice.SetRenderState(D3DRS_FOGSTART, PLongWord(@FStart)^), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  ErrorTest(FScene.FDevice.SetRenderState(D3DRS_FOGEND, PLongWord(@FEnd)^), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

constructor TxorDXFog.Create(Scene: TxorDXScene);
begin
  inherited;
  FAutoColor := True;
  FColor := FScene.FBackgroundColor;
  FDensity := 1.0;
  FEnd := 9.0;
  FTableMode := xorDXFogLINEAR;
  FVertexMode := xorDXFogLINEAR;
  FRadial := False;
  FStart := 6.0;
end;

procedure TxorDXFog.Deactivate;
begin
  inherited;
  ErrorTest(FScene.FDevice.SetRenderState(D3DRS_FOGENABLE, Ord(False)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  ErrorTest(FScene.FDevice.SetRenderState(D3DRS_RANGEFOGENABLE, Ord(False)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

function TxorDXFog.GetRadialEnable: Boolean;
begin
  if not FScene.FCapsRead then ErrorTest(D3DERR_INVALIDCALL, @errCapsNotRead, GetErrorMsgD3D, True);
  Result := (FScene.FCaps.RasterCaps and D3DPRasterCaps_FogRange) <> 0;
end;

procedure TxorDXFog.SetColor(r, g, b, Alpha: LongWord);
begin
  FColor := GetColRGBA(r, g, b, Alpha);
end;

procedure TxorDXFog.SetRadial(const Value: Boolean);
begin
  if FScene.FCapsRead and not ((FScene.FCaps.RasterCaps and D3DPRasterCaps_FogRange) <> 0) then ErrorTest(D3DERR_NOTAVAILABLE, @errCapsFogRange, GetErrorMsgD3D, xorDXThrowCapsException);
  FRadial := Value;
end;

{ TxorDXAlphaBlending }

procedure TxorDXAlphaBlending.Activate;
begin
  inherited;
  ErrorTest(FScene.FDevice.SetRenderState(D3DRS_ALPHABLENDENABLE, Ord(True)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXAlphaBlending.AlphaStageBlend(Arg1, Arg2: TxorDXTexArg; Op: TxorDXTexOp);
begin
  ErrorTest(FScene.FDevice.SetTextureStageState(FStage, D3DTSS_ALPHAARG1, Arg1), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  ErrorTest(FScene.FDevice.SetTextureStageState(FStage, D3DTSS_ALPHAARG2, Arg2), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  ErrorTest(FScene.FDevice.SetTextureStageState(FStage, D3DTSS_ALPHAOP, Op), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXAlphaBlending.AlphaStageBlendCurrentOnly;
begin
  AlphaStageBlend(xorDXTexArgCURRENT, xorDXTexArgTEXTURE, xorDXTexOpSELECTARG1);
end;

procedure TxorDXAlphaBlending.AlphaStageBlendDefault;
begin
  if FStage = 0 then AlphaStageBlendCurrentOnly else AlphaStageBlendDisabled;
end;

procedure TxorDXAlphaBlending.AlphaStageBlendDiffuseOnly;
begin
  AlphaStageBlend(xorDXTexArgDIFFUSE, xorDXTexArgTEXTURE, xorDXTexOpSELECTARG1);
end;

procedure TxorDXAlphaBlending.AlphaStageBlendDisabled;
begin
  ErrorTest(FScene.FDevice.SetTextureStageState(FStage, D3DTSS_ALPHAOP, xorDXTexOpDISABLE), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXAlphaBlending.AlphaStageBlendModCurrentAndTexture;
begin
  AlphaStageBlend(xorDXTexArgCURRENT, xorDXTexArgTEXTURE, xorDXTexOpMODULATE);
end;

procedure TxorDXAlphaBlending.AlphaStageBlendModDiffuseAndTexture;
begin
  AlphaStageBlend(xorDXTexArgDIFFUSE, xorDXTexArgTEXTURE, xorDXTexOpMODULATE);
end;

procedure TxorDXAlphaBlending.AlphaStageBlendTextureOnly;
begin
  AlphaStageBlend(xorDXTexArgCurrent, xorDXTexArgTEXTURE, xorDXTexOpSELECTARG2);
end;

procedure TxorDXAlphaBlending.Blend(SrcBlend, DestBlend: TxorDXBlend);
begin
  ErrorTest(FScene.FDevice.SetRenderState(D3DRS_SRCBLEND, SrcBlend), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  ErrorTest(FScene.FDevice.SetRenderState(D3DRS_DESTBLEND, DestBlend), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXAlphaBlending.BlendDefault;
begin
  BlendDisabled;
end;

procedure TxorDXAlphaBlending.BlendDestAlpha;
begin
  Blend(xorDXBlendDESTALPHA, xorDXBlendINVDESTALPHA);
end;

procedure TxorDXAlphaBlending.BlendDestAlphaInv;
begin
  Blend(xorDXBlendINVDESTALPHA, xorDXBlendDESTALPHA);
end;

procedure TxorDXAlphaBlending.BlendDestColor;
begin
  Blend(xorDXBlendDESTCOLOR, xorDXBlendINVDESTCOLOR);
end;

procedure TxorDXAlphaBlending.BlendDestColorInv;
begin
  Blend(xorDXBlendINVDESTCOLOR, xorDXBlendDESTCOLOR);
end;

procedure TxorDXAlphaBlending.BlendDestOnly;
begin
  Blend(xorDXBlendZERO, xorDXBlendONE);
end;

procedure TxorDXAlphaBlending.BlendDisabled;
begin
  ErrorTest(FScene.FDevice.SetRenderState(D3DRS_ALPHABLENDENABLE, Ord(False)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXAlphaBlending.BlendSrcAlpha;
begin
  Blend(xorDXBlendSRCALPHA, xorDXBlendINVSRCALPHA);
end;

procedure TxorDXAlphaBlending.BlendSrcAlphaInv;
begin
  Blend(xorDXBlendINVSRCALPHA, xorDXBlendSRCALPHA);
end;

procedure TxorDXAlphaBlending.BlendSrcColor;
begin
  Blend(xorDXBlendSRCCOLOR, xorDXBlendINVSRCCOLOR);
end;

procedure TxorDXAlphaBlending.BlendSrcColorInv;
begin
  Blend(xorDXBlendINVSRCCOLOR, xorDXBlendSRCCOLOR);
end;

procedure TxorDXAlphaBlending.BlendSrcOnly;
begin
  Blend(xorDXBlendONE, xorDXBlendZERO);
end;

constructor TxorDXAlphaBlending.Create(Scene: TxorDXScene);
begin
  inherited;
  FStage := 0;
end;

procedure TxorDXAlphaBlending.Deactivate;
begin
  inherited;
  ErrorTest(FScene.FDevice.SetRenderState(D3DRS_ALPHABLENDENABLE, Ord(False)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

function TxorDXAlphaBlending.GetMaxSimultaneousTextures: Word;
begin
  if not FScene.FCapsRead then ErrorTest(D3DERR_INVALIDCALL, @errCapsNotRead, GetErrorMsgD3D, True);
  Result := FScene.FCaps.MaxSimultaneousTextures;
end;

function TxorDXAlphaBlending.GetMaxStageNo: TxorDXStage;
begin
  if not FScene.FCapsRead then ErrorTest(D3DERR_INVALIDCALL, @errCapsNotRead, GetErrorMsgD3D, True);
  if FScene.FCaps.MaxTextureBlendStages > 0 then Result := FScene.FCaps.MaxTextureBlendStages - 1 else Result := 0;
end;

procedure TxorDXAlphaBlending.SetStage(const Value: TxorDXStage);
begin
  if FScene.FCapsRead and ((FScene.FCaps.MaxTextureBlendStages <= Value) or (FScene.FCaps.MaxSimultaneousTextures <= Value)) then ErrorTest(D3DERR_NOTAVAILABLE, @errCapsStage, GetErrorMsgD3D, xorDXThrowCapsException);
  FStage := Value;
end;

{ TxorDXTransparency }

procedure TxorDXTransparency.Activate;
begin
  AlphaStageBlendModDiffuseAndTexture;
  BlendSrcAlpha;
  inherited;
end;

procedure TxorDXTransparency.Deactivate;
begin
  inherited;
  AlphaStageBlendDefault;
end;

{ TxorDXCustomDevice }

constructor TxorDXCustomDevice.Create(Scene: TxorDXScene);
begin
  inherited;
  PrevDevice := FScene.FLastInsertedDevice;
  FScene.FLastInsertedDevice := Self;
end;

procedure TxorDXCustomDevice.OnLostDevice;
begin
  if Assigned(PrevDevice) then PrevDevice.OnLostDevice;
end;

procedure TxorDXCustomDevice.OnResetDevice;
begin
  if Assigned(PrevDevice) then PrevDevice.OnResetDevice;
end;

{ TxorDXFont }

constructor TxorDXFont.Create(Scene: TxorDXScene);
var
  F: TLogFont;
begin
  inherited;
  F := xorDXStandardFont;
  ErrorTest(D3DXCreateFontIndirect(Scene.FDevice, F, FDevice), @errFontCreate, GetErrorMsgD3D, True);
end;

constructor TxorDXFont.Create(Scene: TxorDXScene; FontParams: TLogFont);
begin
  inherited Create(Scene);
  ErrorTest(D3DXCreateFontIndirect(Scene.FDevice, FontParams, FDevice), @errFontCreate, GetErrorMsgD3D, True);
end;

constructor TxorDXFont.Create(Scene: TxorDXScene; Name: ShortString; Height: Word; Style: TFontStyles);
var
  F: TLogFont; i: Integer;
begin
  inherited Create(Scene);
  F := xorDXStandardFont;
  with F do
    begin
    for i := 0 to LF_FACESIZE - 1 do
      if i = Length(Name) then Break else lfFaceName[i] := Name[i + 1];
    lfHeight := Height;
    lfWeight := FW_BOLD * Ord(fsBold in Style) + FW_NORMAL * Ord(not (fsBold in Style));
    lfItalic := Ord(fsItalic in Style);
    lfUnderline := Ord(fsUnderline in Style);
    lfStrikeOut := Ord(fsStrikeOut in Style);
    end;
  ErrorTest(D3DXCreateFontIndirect(Scene.FDevice, F, FDevice), @errFontCreate, GetErrorMsgD3D, True);
end;

destructor TxorDXFont.Destroy;
begin
  if FDevice <> nil then FDevice := nil;
  inherited;
end;

procedure TxorDXFont.OnLostDevice;
begin
  ErrorTest(FDevice.OnLostDevice, @errDeviceLostReset, GetErrorMsgD3D, True);
  inherited;
end;

procedure TxorDXFont.OnResetDevice;
begin
  inherited;
  ErrorTest(FDevice.OnResetDevice, @errDeviceLostReset, GetErrorMsgD3D, True);
end;

procedure TxorDXFont.TextOut(Rect: TRect; Format: TxorDXFontFormat; Color: TColor; Alpha: Single; const Text: AnsiString);
begin
  ErrorTest(FDevice.DrawTextA(PAnsiChar(Text), -1, Rect, Format, GetColS(Color, Alpha)), @errFontShow, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXFont.TextOut(Rect: TRect; Format: TxorDXFontFormat; Color: TColor; Alpha: LongWord; const Text: AnsiString);
begin
  ErrorTest(FDevice.DrawTextA(PAnsiChar(Text), -1, Rect, Format, GetColA(Color, Alpha)), @errFontShow, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

{ TxorDXCustomTransform }

procedure TxorDXCustomTransform.ClearTransformation;
begin
  D3DXMatrixIdentity(FTransformation);
  FixTransformation;
  FTransformed := True;
end;

constructor TxorDXCustomTransform.Create(Scene: TxorDXScene);
begin
  inherited;
  ClearTransformation;
  FTransformed := False;
end;

procedure TxorDXCustomTransform.FixTransformation;
begin
  FTransformationSave := FTransformation;
end;

function TxorDXCustomTransform.GetIsTransformed: Boolean;
begin
  Result := not D3DXMatrixIsIdentity(FTransformation);
end;

function TxorDXCustomTransform.GetTransformationMatrix: TxorDXMatrix;
begin
  Result := FTransformation;
end;

procedure TxorDXCustomTransform.Move(x, y, z: Single);
var
  M: TxorDXMatrix;
begin
  FTransformed := True;
  D3DXMatrixTranslation(M, x, y, z);
  D3DXMatrixMultiply(FTransformation, FTransformation, M);
end;

procedure TxorDXCustomTransform.RotateAxis(x, y, z, Angle: Single);
var
  M: TxorDXMatrix;
  V: TxorDXVector;
begin
  V := GetVector(x, y, z);
  FTransformed := True;
  D3DXMatrixRotationAxis(M, V, Angle);
  D3DXMatrixMultiply(FTransformation, FTransformation, M);
end;

procedure TxorDXCustomTransform.RotateX(Angle: Single);
var
  M: TxorDXMatrix;
begin
  FTransformed := True;
  D3DXMatrixRotationX(M, Angle);
  D3DXMatrixMultiply(FTransformation, FTransformation, M);
end;

procedure TxorDXCustomTransform.RotateY(Angle: Single);
var
  M: TxorDXMatrix;
begin
  FTransformed := True;
  D3DXMatrixRotationY(M, Angle);
  D3DXMatrixMultiply(FTransformation, FTransformation, M);
end;

procedure TxorDXCustomTransform.RotateZ(Angle: Single);
var
  M: TxorDXMatrix;
begin
  FTransformed := True;
  D3DXMatrixRotationZ(M, Angle);
  D3DXMatrixMultiply(FTransformation, FTransformation, M);
end;

procedure TxorDXCustomTransform.Scale(x, y, z: Single);
var
  M: TxorDXMatrix;
begin
  FTransformed := True;
  D3DXMatrixScaling(M, x, y, z);
  D3DXMatrixMultiply(FTransformation, FTransformation, M);
end;

procedure TxorDXCustomTransform.SetTransformationMatrix(const Value: TxorDXMatrix);
begin
  FTransformed := True;
  FTransformation := Value;
end;

procedure TxorDXCustomTransform.UndoTransformation;
begin
  FTransformation := FTransformationSave;
  FTransformed := True;
end;

{ TxorDXScene }

procedure TxorDXScene.Clear;
begin
  ErrorTest(FDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER, FBackgroundColor, 1.0, 0), @errClearDevice, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXScene.ClearError;
begin
  FLastError := S_OK;
  FLastErrorMsg := '';
end;

constructor TxorDXScene.Create(FormHandle: HWND);
begin
  CreateParams(FormHandle, False, xorDXSwapDiscard, xorDXPresentIntervalDefault, xorDXMultiSample_NONE, 0, 0, True, vpSoft);
end;

procedure TxorDXScene.CreateBuffers;
var
  IdxFormat: TD3DFormat;
begin
  if FBufferCreated then ErrorTest(D3DERR_INVALIDCALL,@errAssignedBuffer, GetErrorMsgD3D, True);
  FBufferCreated := True;
  FFormatFixed := True;
  if FUse32BitIndex then IdxFormat := D3DFMT_INDEX32 else IdxFormat := D3DFMT_INDEX16;
  if FVertexBufferSize > 0 then
    ErrorTest(FDevice.CreateVertexBuffer(FVertexBufferSize, 0, FVertexBufferFlags, D3DPOOL_DEFAULT, FVertexBuffer), @errBufferCreation, GetErrorMsgD3D, True); // Erstellen des Vertexbuffers im Device
  if FIndexBufferSize > 0 then
    ErrorTest(FDevice.CreateIndexBuffer(FIndexBufferSize, 0, IdxFormat, D3DPOOL_DEFAULT, FIndexBuffer), @errBufferCreation, GetErrorMsgD3D, True);
end;

constructor TxorDXScene.CreateFullScreen(FormHandle: HWND);
begin
  CreateParams(FormHandle, True, xorDXSwapDiscard, xorDXPresentIntervalDefault, xorDXMultiSample_NONE, 800, 600, True, vpSoft);
end;

constructor TxorDXScene.CreateFullScreen(FormHandle: HWND; Resolution: TxorDXResolution);
var
  W, H, D: Cardinal;
begin
  GetResolution(Resolution, W, H, D);
  CreateParams(FormHandle, True, xorDXSwapDiscard, xorDXPresentIntervalDefault, xorDXMultiSample_NONE, W, H, D = 32, vpSoft);
end;

constructor TxorDXScene.CreateParams(FormHandle: HWND; FullScreen: Boolean; BufferSwapEffect: TxorDXSwapEffect; PresentationInterval: TxorDXPresentationInterval; AntiAliasValue: TxorDXMultiSampleType; ScreenWidth, ScreenHeight: Cardinal; ColorDepth32Bit: Boolean; Behaviour: TxorDXRenderOption);
var
  CurrentDisplaySettings: TD3DDisplayMode;
  DevCreated: HResult;
  i: Integer;

        function DeviceTest(DisplayFormat, BackbufferFormat: TD3DFormat): Boolean;
        begin
          Result := Windows.Succeeded(FInterface.CheckDeviceType(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, DisplayFormat, BackbufferFormat, False))
        end;
begin
  FCapsRead := False;
  FVertexBuffer := nil;
  FIndexBuffer := nil;
  FLastError := S_OK;
  FLastErrorMsg := '';
  inherited Create(Self);
  FOnDraw := nil;
  FOnDrawException := nil;
  FAutoLightOnOff := True;
  FInput := nil;
  FActiveInputs := 0;
  FIsDrawing := False;
  FBufferCreated := False;
  FFormatFixed := False;
  FUse32BitIndex := False;
  FUsedLights := 0;
  FActiveLights := 0;
  FDrawCalls := 0;           
  FBackgroundColor := GetColS(clWhite, 1.0);
  FLastActivatedCamera := nil;
  FLastInsertedObject := nil;
  FLastInsertedDevice := nil;
  for i := 0 to High(TxorDXStage) do FLastActivatedTextures[i] := nil;
  FHandleWindow := FormHandle;
  FHandleOutput := FormHandle;
  try
    //interface initialisieren
    FInterface := Direct3DCreate8(D3D_SDK_VERSION);
    if FInterface = nil then ErrorTest(E_Fail, @errInterfaceCreate, GetErrorMsgD3D, True);
    //Presentation parameter
    FillChar(FPresentationParameters, SizeOf(FPresentationParameters), 0); //PresentationParameters initialisieren
    with FPresentationParameters do
      begin
      Windowed := not FullScreen;
      hDeviceWindow := FormHandle;
      //16-bit Z-Buffer
      EnableAutoDepthStencil := TRUE;
      AutoDepthStencilFormat := D3DFMT_D16;
      //Antialiasing
      AntiAliasValue := AntiAliasValue mod 17;
      MultiSampleType := AntiAliasValue * LongWord(Ord(AntiAliasValue <> 1) * Ord(BufferSwapEffect = xorDXSwapDiscard));
      //BackBuffer
      BackBufferCount := 1;
      SwapEffect := BufferSwapEffect;
      BackBufferWidth := ScreenWidth;
      BackBufferHeight := ScreenHeight;
      if not FullScreen then
        begin
        ErrorTest(FInterface.GetAdapterDisplayMode(D3DADAPTER_DEFAULT, CurrentDisplaySettings), @errDisplaySettings, GetErrorMsgD3D, True);
        BackBufferFormat := CurrentDisplaySettings.Format;
        if (BackBufferWidth = 0) and (BackBufferHeight = 0) then begin BackBufferWidth := Screen.Width; BackBufferHeight := Screen.Height; end;
        end
      else
        begin
        FullScreen_PresentationInterval := PresentationInterval;
        BackBufferFormat := D3DFMT_R5G6B5;
        if ColorDepth32Bit then
          if DeviceTest(D3DFMT_X8R8G8B8, D3DFMT_X8R8G8B8) then BackBufferFormat := D3DFMT_X8R8G8B8
          else if DeviceTest(D3DFMT_X8R8G8B8, D3DFMT_A8R8G8B8) then BackBufferFormat := D3DFMT_A8R8G8B8;
        if BackBufferFormat = D3DFMT_R5G6B5 then //not ColorDepth32Bit or 32 Bit not possible
          if DeviceTest(D3DFMT_R5G6B5, D3DFMT_R5G6B5) then BackBufferFormat := D3DFMT_R5G6B5
          else if DeviceTest(D3DFMT_X1R5G5B5, D3DFMT_X1R5G5B5) then BackBufferFormat := D3DFMT_X1R5G5B5
          else if DeviceTest(D3DFMT_X1R5G5B5, D3DFMT_A1R5G5B5) then BackBufferFormat := D3DFMT_A1R5G5B5
          else ErrorTest(E_Fail, @errDisplayFormat, GetErrorMsgD3D, True);
        end;
      end;
    //Device erstellen
    DevCreated := E_Fail;
    if Behaviour <= vpPure then                                                   //1.Standardgrafikkarte, 2.Hardwarebeschleunigung, 3.Fensterhandle, 4.Vertexprocessing, 5. obiger parameterrecord, 6.result device
      DevCreated := FInterface.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, FormHandle,
                    D3DCREATE_HARDWARE_VERTEXPROCESSING or D3DCREATE_PUREDEVICE, FPresentationParameters, FDevice);
    if (Behaviour <= vpHard) and Windows.Failed(DevCreated) then
      DevCreated := FInterface.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, FormHandle,
                    D3DCREATE_HARDWARE_VERTEXPROCESSING, FPresentationParameters, FDevice);
    if (Behaviour <= vpMix) and Windows.Failed(DevCreated) then
      DevCreated := FInterface.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, FormHandle,
                    D3DCREATE_MIXED_VERTEXPROCESSING, FPresentationParameters, FDevice);
    if (Behaviour <= vpSoft) and Windows.Failed(DevCreated) then
      DevCreated := FInterface.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, FormHandle,
                    D3DCREATE_SOFTWARE_VERTEXPROCESSING, FPresentationParameters, FDevice);
    ErrorTest(DevCreated, @errDeviceCreate, GetErrorMsgD3D, True);
  except
    on E: Exception do
      begin
      if FDevice <> nil then FDevice := nil;
      if FInterface <> nil then FInterface := nil;
      raise;
      end;
  end;
  //Caps auslesen
  FCapsRead := ErrorTest(FDevice.GetDeviceCaps(FCaps), @errCaps, GetErrorMsgD3D, xorDXThrowCapsException);
  //Renderstate und TextureStageState initialisieren
  with FDevice do
    begin
    ErrorTest(SetRenderState(D3DRS_DITHERENABLE, Ord(True)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions); //Graphische Qualitt
    ErrorTest(SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions); //Culling nicht aktiv (cull = nicht rendern)
    ErrorTest(SetRenderState(D3DRS_ZENABLE, Ord(True)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions); //z-Buffer aktiv
    ErrorTest(SetRenderState(D3DRS_LIGHTING, Ord(False)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions); //Licht aus: kein Benutzen der Vektornormalen
    ErrorTest(SetRenderState(D3DRS_SPECULARENABLE, Ord(True)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions); //Graphische Qualitt
    ErrorTest(SetRenderState(D3DRS_AMBIENT, GetColRGBS(110, 110, 110, 1.0)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(SetRenderState(D3DRS_DIFFUSEMATERIALSOURCE, D3DMCS_COLOR1), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(SetRenderState(D3DRS_AMBIENTMATERIALSOURCE, D3DMCS_COLOR1), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(SetRenderState(D3DRS_EMISSIVEMATERIALSOURCE, D3DMCS_MATERIAL), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(SetRenderState(D3DRS_SPECULARMATERIALSOURCE, D3DMCS_MATERIAL), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(0, D3DTSS_COLORARG1, xorDXTexArgDIFFUSE), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(1, D3DTSS_COLORARG1, xorDXTexArgCURRENT), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(0, D3DTSS_COLORARG2, xorDXTexArgTEXTURE), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(1, D3DTSS_COLORARG2, xorDXTexArgTEXTURE), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(0, D3DTSS_COLOROP, xorDXTexOpDISABLE), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(1, D3DTSS_COLOROP, xorDXTexOpDISABLE), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(0, D3DTSS_ADDRESSU, xorDXTexAddrWRAP), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(1, D3DTSS_ADDRESSU, xorDXTexAddrWRAP), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(0, D3DTSS_ADDRESSV, xorDXTexAddrWRAP), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(1, D3DTSS_ADDRESSV, xorDXTexAddrWRAP), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, xorDXTexCoordIndex0), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(1, D3DTSS_TEXCOORDINDEX, xorDXTexCoordIndex0), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(1, D3DTSS_MAGFILTER, D3DTEXF_LINEAR), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(1, D3DTSS_MINFILTER, D3DTEXF_LINEAR), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_LINEAR), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    ErrorTest(FScene.FDevice.SetTextureStageState(1, D3DTSS_MIPFILTER, D3DTEXF_LINEAR), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    end;
  Clear;
end;

constructor TxorDXScene.CreateWindowed(FormHandle: HWND);
begin
  CreateParams(FormHandle, False, xorDXSwapDiscard, xorDXPresentIntervalDefault, xorDXMultiSample_NONE, Screen.Width, Screen.Height, True, vpSoft);
end;

destructor TxorDXScene.Destroy;
begin
  if FIndexBuffer <> nil then FIndexBuffer := nil;
  if FVertexBuffer <> nil then FVertexBuffer := nil;
  if FInput <> nil then FInput := nil;
  if FDevice <> nil then FDevice := nil;
  if FInterface <> nil then FInterface := nil;
  inherited;
end;

procedure TxorDXScene.Draw;
var
  Dummy: Boolean;
begin
  DrawScene(nil, Dummy);
end;

procedure TxorDXScene.DrawScene(Sender: TObject; var Done: Boolean);
begin
  Done := False;
  if FIsDrawing then Exit;
  Inc(FDrawCalls);
  with FDevice do
    try
      if not Assigned(FOnDraw) then ErrorTest(D3DERR_INVALIDCALL, @errNoOnDraw, GetErrorMsgD3D, True);
      FIsDrawing := True;
      ErrorTest(BeginScene, @errScene, GetErrorMsgD3D, True);
      FOnDraw(Self);
      ErrorTest(EndScene, @errScene, GetErrorMsgD3D, True);
      ErrorTest(Present(nil, nil, FHandleOutput, nil), @errScene, GetErrorMsgD3D, True);
      FIsDrawing := False;
    except
      on E: Exception do
        begin
        FIsDrawing := False;
        StopLooping;
        if Assigned(FOnDrawException) then FOnDrawException(E.Message) else raise;
        end;
    end;
end;

function TxorDXScene.Failed: Boolean;
begin
  Result := Windows.Failed(FScene.FLastError);
end;

function TxorDXScene.GetBoolValue(Index: Integer): Boolean;
var
  C: LongWord;
begin
  C := 0;
  Case index of
    0: ErrorTest(FDevice.GetRenderState(D3DRS_LIGHTING, C), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    1: ErrorTest(FDevice.GetRenderState(D3DRS_DITHERENABLE, C), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    2: ErrorTest(FDevice.GetRenderState(D3DRS_SPECULARENABLE, C), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    3: ErrorTest(FDevice.GetRenderState(D3DRS_ZENABLE, C), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    4: C := Ord(not FPresentationParameters.Windowed); //fullscreen
  end;
  Result := C <> 0;
end;

function TxorDXScene.GetCardinalValue(Index: Integer): Cardinal;
begin
  Case index of
    0: Result := FPresentationParameters.BackBufferHeight;
    1: Result := FPresentationParameters.BackBufferWidth;
  else Result := 0;
  end;
end;

function TxorDXScene.GetCullMode: TxorDXCull;
begin
  ErrorTest(FDevice.GetRenderState(D3DRS_CULLMODE, Result), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

function TxorDXScene.GetMaterialSource(Index: Integer): TxorDXMaterialSource;
begin
  Case index of
    0: ErrorTest(FDevice.GetRenderState(D3DRS_AMBIENTMATERIALSOURCE, Result), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    1: ErrorTest(FDevice.GetRenderState(D3DRS_DIFFUSEMATERIALSOURCE, Result), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    2: ErrorTest(FDevice.GetRenderState(D3DRS_EMISSIVEMATERIALSOURCE, Result), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    3: ErrorTest(FDevice.GetRenderState(D3DRS_SPECULARMATERIALSOURCE, Result), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  end;
end;

procedure TxorDXScene.LostDevice;
begin
  if Assigned(FLastInsertedDevice) then FLastInsertedDevice.OnLostDevice;
end;

procedure TxorDXScene.ResetDevice;
begin
  if Assigned(FLastInsertedDevice) then FLastInsertedDevice.OnResetDevice;
end;

procedure TxorDXScene.Set32BitIndex(Value: Boolean);
begin
  if FUse32BitIndex <> Value then
    if FFormatFixed then ErrorTest(D3DERR_INVALIDCALL, @errFormatBuffer, GetErrorMsgD3D, True)
    else
      begin
      if FScene.FCapsRead and (65536 > FScene.FCaps.MaxVertexIndex) then ErrorTest(D3DERR_NOTAVAILABLE, @errCaps32Bit, GetErrorMsgD3D, xorDXThrowCapsException);
      FUse32BitIndex := Value;
      end;
end;

procedure TxorDXScene.SetAmbientLight(r, g, b, Alpha: LongWord);
begin
  ErrorTest(FDevice.SetRenderState(D3DRS_AMBIENT, GetColRGBA(r, g, b, Alpha)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXScene.SetAmbientLight(r, g, b: LongWord; Alpha: Single);
begin
  ErrorTest(FDevice.SetRenderState(D3DRS_AMBIENT, GetColRGBS(r, g, b, Alpha)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXScene.SetBackgroundColor(r, g, b, Alpha: LongWord);
begin
  FBackgroundColor := GetColRGBA(r, g, b, Alpha);
end;

procedure TxorDXScene.SetBackgroundColor(r, g, b: LongWord; Alpha: Single);
begin
  FBackgroundColor := GetColRGBS(r, g, b, Alpha);
end;

procedure TxorDXScene.SetBoolValue(const Index: Integer; const Value: Boolean);
begin
  Case index of
    0: ErrorTest(FDevice.SetRenderState(D3DRS_LIGHTING, Ord(Value)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    1: ErrorTest(FDevice.SetRenderState(D3DRS_DITHERENABLE, Ord(Value)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    2: ErrorTest(FDevice.SetRenderState(D3DRS_SPECULARENABLE, Ord(Value)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    3: ErrorTest(FDevice.SetRenderState(D3DRS_ZENABLE, Ord(Value)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  end;
end;

procedure TxorDXScene.SetCullMode(Value: TxorDXCull);
begin
  ErrorTest(FDevice.SetRenderState(D3DRS_CULLMODE, Value), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXScene.SetIndexBufferSize(Value: Cardinal);
begin
  if FIndexBufferSize < Value then
    if FBufferCreated then ErrorTest(D3DERR_INVALIDCALL, @errAssignedBuffer, GetErrorMsgD3D, True)
    else FIndexBufferSize := Value;
end;

procedure TxorDXScene.SetMaterialSource(const Index: Integer; const Value: TxorDXMaterialSource);
begin
  Case index of
    0: ErrorTest(FDevice.SetRenderState(D3DRS_AMBIENTMATERIALSOURCE, Value), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    1: ErrorTest(FDevice.SetRenderState(D3DRS_DIFFUSEMATERIALSOURCE, Value), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    2: ErrorTest(FDevice.SetRenderState(D3DRS_EMISSIVEMATERIALSOURCE, Value), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    3: ErrorTest(FDevice.SetRenderState(D3DRS_SPECULARMATERIALSOURCE, Value), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  end;
end;

procedure TxorDXScene.SetTransformation;
var
  M: TxorDXMatrix;
begin
  D3DXMatrixMultiply(M, FLastInsertedObject.FTransformation, FTransformation);  //erst locale Transformation, dann globale ausfhren
  ErrorTest(FDevice.SetTransform(D3DTS_WORLD, M), @errTransform, GetErrorMsgD3D, xorDXThrowAllOtherExceptions); //Transformation an Device bertragen
  FTransformed := False;
end;

procedure TxorDXScene.SetVertexBufferFlags(Value: TxorDXVertexFlag);
begin
  if FBufferCreated then ErrorTest(D3DERR_INVALIDCALL, @errAssignedBuffer, GetErrorMsgD3D, True)
  else FVertexBufferFlags := FVertexBufferFlags or Value;
end;

procedure TxorDXScene.SetVertexBufferSize(Value: TxorDXVertexSize);
begin
  if FVertexBufferSize < Value then
    if FBufferCreated then ErrorTest(D3DERR_INVALIDCALL, @errAssignedBuffer, GetErrorMsgD3D, True)
    else FVertexBufferSize := Value;
end;

procedure TxorDXScene.StartLooping;
begin
  Application.OnIdle := DrawScene; //LoopScene wird regelmig aufgerufen
end;

procedure TxorDXScene.StopLooping;
begin
  Application.OnIdle := nil;
end;

function TxorDXScene.Succeeded: Boolean;
begin
  Result := Windows.Succeeded(FScene.FLastError);
end;

{ TxorDXCustomActiveTransform }

procedure TxorDXCustomActiveTransform.SetActive(const Value: Boolean);
begin
  if Value then Activate else Deactivate;
end;

{ TxorDXTexture }

procedure TxorDXTexture.Activate;
begin
  if FTexture = nil then ErrorTest(D3DERR_INVALIDCALL, @errTextureActivate, GetErrorMsgD3D, True);
  if Assigned(FScene.FLastActivatedTextures[FStage]) then ErrorTest(D3DERR_INVALIDCALL, @errTextureConflict, GetErrorMsgD3D, True);
  ErrorTest(FScene.FDevice.SetTexture(FStage, FTexture), @errTextureSet, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  FScene._Device.SetTransform(D3DTS_TEXTURE0 + FStage, FTransformation);
  FScene._Device.SetTextureStageState(FStage, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_COUNT2 * Ord(not D3DXMatrixIsIdentity(FTransformation)));
  FScene.FLastActivatedTextures[FStage] := Self;
end;

procedure TxorDXTexture.ColorStageBlend(Arg1, Arg2: TxorDXTexArg; Op: TxorDXTexOp);
begin
  ErrorTest(FScene.FDevice.SetTextureStageState(FStage, D3DTSS_COLORARG1, Arg1), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  ErrorTest(FScene.FDevice.SetTextureStageState(FStage, D3DTSS_COLORARG2, Arg2), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  ErrorTest(FScene.FDevice.SetTextureStageState(FStage, D3DTSS_COLOROP, Op), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXTexture.ColorStageBlendAdd;
begin
  ColorStageBlend(xorDXTexArgCURRENT, xorDXTexArgTEXTURE, xorDXTexOpADD);
end;

procedure TxorDXTexture.ColorStageBlendAsBumpMap;
begin
  ColorStageBlend(xorDXTexArgCURRENT, xorDXTexArgTEXTURE, xorDXTexOpBUMPENVMAP);
end;

procedure TxorDXTexture.ColorStageBlendAsBumpMapLUMINANCE;
begin
  ColorStageBlend(xorDXTexArgCURRENT, xorDXTexArgTEXTURE, xorDXTexOpBUMPENVMAPLUMINANCE);
end;

procedure TxorDXTexture.ColorStageBlendAsDarkMap;
begin
  ColorStageBlendModulate;
end;

procedure TxorDXTexture.ColorStageBlendAsDetailMap;
begin
  ColorStageBlend(xorDXTexArgCURRENT, xorDXTexArgTEXTURE, xorDXTexOpADDSIGNED);
end;

procedure TxorDXTexture.ColorStageBlendAsGlowMap;
begin
  ColorStageBlendAdd;
end;

procedure TxorDXTexture.ColorStageBlendAsLightMap(BlendLightOperation: TxorDXBlendLightOperation);
begin
  ColorStageBlend(xorDXTexArgCURRENT, xorDXTexArgTEXTURE, GetBlendLightOp(BlendLightOperation));
end;

procedure TxorDXTexture.ColorStageBlendCurrentAlpha;
begin
  ColorStageBlend(xorDXTexArgCURRENT, xorDXTexArgTEXTURE, xorDXTexOpBLENDCURRENTALPHA);
end;

procedure TxorDXTexture.ColorStageBlendDiffuseAlpha;
begin
  ColorStageBlend(xorDXTexArgCURRENT, xorDXTexArgTEXTURE, xorDXTexOpBLENDDIFFUSEALPHA);
end;

procedure TxorDXTexture.ColorStageBlendDiffuseColorOnly;
begin
  ColorStageBlend(xorDXTexArgDIFFUSE, xorDXTexArgTEXTURE, xorDXTexOpSELECTARG1);
end;

procedure TxorDXTexture.ColorStageBlendDisable;
begin
  ColorStageBlend(xorDXTexArgCURRENT, xorDXTexArgTEXTURE, xorDXTexOpDISABLE);
end;

procedure TxorDXTexture.ColorStageBlendFactor(PerCent: Single);
begin
  ErrorTest(FScene.FDevice.SetRenderState(D3DRS_TEXTUREFACTOR, GetColRGBS(0, 0, 0, PerCent)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  ColorStageBlend(xorDXTexArgCURRENT, xorDXTexArgTEXTURE, xorDXTexOpBLENDFACTORALPHA);
end;

procedure TxorDXTexture.ColorStageBlendModulate;
begin
  ColorStageBlend(xorDXTexArgCURRENT, xorDXTexArgTEXTURE, xorDXTexOpMODULATE);
end;

procedure TxorDXTexture.ColorStageBlendTextureAlpha;
begin
  ColorStageBlend(xorDXTexArgCURRENT, xorDXTexArgTEXTURE, xorDXTexOpBLENDTEXTUREALPHA);
end;

procedure TxorDXTexture.ColorStageBlendTextureAlphaPM;
begin
  ColorStageBlend(xorDXTexArgCURRENT, xorDXTexArgTEXTURE, xorDXTexOpBLENDTEXTUREALPHAPM);
end;

procedure TxorDXTexture.ColorStageBlendTextureAndCurrent(BlendLightOperation: TxorDXBlendLightOperation);
begin
  ColorStageBlend(xorDXTexArgCURRENT, xorDXTexArgTEXTURE, GetBlendLightOp(BlendLightOperation));
end;

procedure TxorDXTexture.ColorStageBlendTextureAndDiffuseColor(BlendLightOperation: TxorDXBlendLightOperation);
begin
  ColorStageBlend(xorDXTexArgDIFFUSE, xorDXTexArgTEXTURE, GetBlendLightOp(BlendLightOperation));
end;

procedure TxorDXTexture.ColorStageBlendTextureOnly;
begin
  ColorStageBlend(xorDXTexArgCURRENT, xorDXTexArgTEXTURE, xorDXTexOpSELECTARG2);
end;

constructor TxorDXTexture.Create(Scene: TxorDXScene);
begin
  inherited;
  FStage := 0;
  FTexture := nil;
end;

procedure TxorDXTexture.Deactivate;
begin
  if GetActive then
    begin
    ErrorTest(FScene.FDevice.SetTexture(FStage, nil), @errTextureSet, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    FScene.FLastActivatedTextures[FStage] := nil;
    FScene._Device.SetTextureStageState(FStage, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_DISABLE);
    FScene._Device.SetTransform(D3DTS_TEXTURE0 + FStage, IdentityMatrix);
    end
  else ErrorTest(D3DERR_INVALIDCALL, @errTextureDeactivate, GetErrorMsgD3D, True);
end;

function TxorDXTexture.GetActive: Boolean;
begin
  Result := FScene.FLastActivatedTextures[FStage] = Self;
end;

function TxorDXTexture.GetAnyStageActivity: Boolean;
var
  i: Integer;
begin
  Result := True;
  for i := 0 to 7 do
    if FScene.FLastActivatedTextures[i] = Self then Exit;
  Result := False;
end;

function TxorDXTexture.GetMaxSimultaneousTextures: Word;
begin
  if not FScene.FCapsRead then ErrorTest(D3DERR_INVALIDCALL, @errCapsNotRead, GetErrorMsgD3D, True);
  Result := FScene.FCaps.MaxSimultaneousTextures;
end;

function TxorDXTexture.GetMaxStageNo: TxorDXStage;
begin
  if not FScene.FCapsRead then ErrorTest(D3DERR_INVALIDCALL, @errCapsNotRead, GetErrorMsgD3D, True);
  if FScene.FCaps.MaxTextureBlendStages > 0 then Result := FScene.FCaps.MaxTextureBlendStages - 1 else Result := 0;
end;

function TxorDXTexture.GetTransparencyAlpha: Single;
begin
  Result := GetAlpVal(FTransparencyColor);
end;

function TxorDXTexture.GetTransparencyColor: TColor;
begin
  Result := GetTCol(FTransparencyColor);
end;

function TxorDXTexture.GetTransparent: Boolean;
begin
  Result := FTransparencyColor And $FF000000 <> $FF000000;
end;

procedure TxorDXTexture.LoadEmptyEditTexture;
begin
  LoadEmptyTexture(256, 256, 0, 0, D3DFMT_A4R4G4B4, D3DPOOL_MANAGED);
end;

procedure TxorDXTexture.LoadEmptyEditTexture(Width, Height: Cardinal);
begin
  LoadEmptyTexture(Width, Height, 0, 0, D3DFMT_A4R4G4B4, D3DPOOL_MANAGED);
end;

procedure TxorDXTexture.LoadEmptyEditTexture(Width, Height, MipLevels: Cardinal);
begin
  LoadEmptyTexture(Width, Height, MipLevels, 0, D3DFMT_A4R4G4B4, D3DPOOL_MANAGED);
end;

procedure TxorDXTexture.LoadEmptyRenderTexture;
begin
  LoadEmptyTexture(256, 256, 0, D3DUSAGE_RENDERTARGET, FScene.FPresentationParameters.BackBufferFormat, D3DPOOL_DEFAULT);
end;

procedure TxorDXTexture.LoadEmptyRenderTexture(Width, Height: Cardinal);
begin
  LoadEmptyTexture(Width, Height, 0, D3DUSAGE_RENDERTARGET, FScene.FPresentationParameters.BackBufferFormat, D3DPOOL_DEFAULT);
end;

procedure TxorDXTexture.LoadEmptyRenderTexture(Width, Height, MipLevels: Cardinal);
begin
  LoadEmptyTexture(Width, Height, MipLevels, D3DUSAGE_RENDERTARGET, FScene.FPresentationParameters.BackBufferFormat, D3DPOOL_DEFAULT);
end;

procedure TxorDXTexture.LoadEmptyTexture(Width, Height, MipLevels: Cardinal; Usage: TxorDXUsage; Format: TxorDXFormat; Pool: TxorDXPool);
begin
  if FTexture <> nil then if not GetAnyStageActivity then Unload else ErrorTest(D3DERR_INVALIDCALL, @errTextureOverload, GetErrorMsgD3D, True);
  FMipLevels := MipLevels;
  FTransparencyColor := $FF000000;
  ErrorTest(D3DXCreateTexture(FScene.FDevice, Width, Height, MipLevels, Usage, Format, Pool, FTexture), @errTextureLoad, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXTexture.LoadFromFile(FileName: AnsiString; Width, Height, MipLevels: Cardinal; Usage: TxorDXUsage; Format: TxorDXFormat; Pool: TxorDXPool; Filter, MipFilter: TxorDXFilter; Transparent: Boolean; TransparencyColor: TColor; TransparencyAlpha: Single; SrcInfo: PxorDXImageInfo; Palette: PxorDXPaletteEntry);
begin
  if FTexture <> nil then if not GetAnyStageActivity then Unload else ErrorTest(D3DERR_INVALIDCALL, @errTextureOverload, GetErrorMsgD3D, True);
  FMipLevels := MipLevels;
  if Transparent = False then FTransparencyColor := $FF000000 else FTransparencyColor := GetColS(TransparencyColor, TransparencyAlpha);
  ErrorTest(D3DXCreateTextureFromFileEx(FScene.FDevice, PAnsiChar(FileName), Width, Height, MipLevels, Usage, Format, Pool, Filter, MipFilter, FTransparencyColor, SrcInfo, Palette, FTexture), @errTextureLoad, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXTexture.LoadFromFile(FileName: AnsiString);
begin
  if FTexture <> nil then if not GetAnyStageActivity then Unload else ErrorTest(D3DERR_INVALIDCALL, @errTextureOverload, GetErrorMsgD3D, True);
  ErrorTest(D3DXCreateTextureFromFile(FScene.FDevice, PAnsiChar(FileName), FTexture), @errTextureLoad, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  FTransparencyColor := $FF000000;
  FMipLevels := 0;
end;

procedure TxorDXTexture.LoadFromFile(FileName: AnsiString; Transparent: Boolean; TransparencyColor: TColor; TransparencyAlpha: Single);
begin
  LoadFromFile(FileName, 0, 0, 0, D3DUSAGE_RENDERTARGET, D3DFMT_UNKNOWN, D3DPOOL_DEFAULT, D3DX_FILTER_LINEAR, D3DX_FILTER_NONE, Transparent, TransparencyColor, TransparencyAlpha, nil, nil);
end;

procedure TxorDXTexture.LoadFromFile(FileName: AnsiString; MipLevels: Cardinal; Transparent: Boolean; TransparencyColor: TColor; TransparencyAlpha: Single);
begin
  LoadFromFile(FileName, 0, 0, MipLevels, D3DUSAGE_RENDERTARGET, D3DFMT_UNKNOWN, D3DPOOL_DEFAULT, D3DX_FILTER_LINEAR, D3DX_FILTER_NONE, Transparent, TransparencyColor, TransparencyAlpha, nil, nil);
end;

procedure TxorDXTexture.LoadFromFile(FileName: AnsiString; Width, Height: Cardinal; Transparent: Boolean; TransparencyColor: TColor; TransparencyAlpha: Single);
begin
  LoadFromFile(FileName, Width, Height, 0, D3DUSAGE_RENDERTARGET, D3DFMT_UNKNOWN, D3DPOOL_DEFAULT, D3DX_FILTER_LINEAR, D3DX_FILTER_NONE, Transparent, TransparencyColor, TransparencyAlpha, nil, nil);
end;

procedure TxorDXTexture.LoadFromFile(FileName: AnsiString; Width, Height, MipLevels: Cardinal; Filter, MipFilter: TxorDXFilter; Transparent: Boolean; TransparencyColor: TColor; TransparencyAlpha: Single);
begin
  LoadFromFile(FileName, Width, Height, MipLevels, D3DUSAGE_RENDERTARGET, D3DFMT_UNKNOWN, D3DPOOL_DEFAULT, Filter, MipFilter, Transparent, TransparencyColor, TransparencyAlpha, nil, nil);
end;

procedure TxorDXTexture.Move(x, y, z: Single);
begin
  if z <> 0 then ErrorTest(D3DERR_INVALIDCALL, @errTextureTransform, GetErrorMsgD3D, True);
  inherited;
end;

procedure TxorDXTexture.RotateAxis(x, y, z, Angle: Single);
begin
  ErrorTest(D3DERR_INVALIDCALL, @errTextureTransform, GetErrorMsgD3D, True);
end;

procedure TxorDXTexture.RotateX(Angle: Single);
begin
  ErrorTest(D3DERR_INVALIDCALL, @errTextureTransform, GetErrorMsgD3D, True);
end;

procedure TxorDXTexture.RotateY(Angle: Single);
begin
  ErrorTest(D3DERR_INVALIDCALL, @errTextureTransform, GetErrorMsgD3D, True);
end;

procedure TxorDXTexture.Scale(x, y, z: Single);
begin
  if z <> 1 then ErrorTest(D3DERR_INVALIDCALL, @errTextureTransform, GetErrorMsgD3D, True);
  inherited;
end;

procedure TxorDXTexture.SetStage(const Value: TxorDXStage);
begin
  if FScene.FCapsRead and ((FScene.FCaps.MaxTextureBlendStages <= Value) or (FScene.FCaps.MaxSimultaneousTextures <= Value)) then ErrorTest(D3DERR_NOTAVAILABLE, @errCapsStage, GetErrorMsgD3D, xorDXThrowCapsException);
  FStage := Value;
end;

procedure TxorDXTexture.SetTextureAdressU(Adr: TxorDXTexAddress);
begin
  ErrorTest(FScene.FDevice.SetTextureStageState(FStage, D3DTSS_ADDRESSU, Adr), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXTexture.SetTextureAdressV(Adr: TxorDXTexAddress);
begin
  ErrorTest(FScene.FDevice.SetTextureStageState(FStage, D3DTSS_ADDRESSV, Adr), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXTexture.SetTextureBorderColor(C: TxorDXColor);
begin
  ErrorTest(FScene.FDevice.SetTextureStageState(FStage, D3DTSS_BORDERCOLOR, C), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXTexture.SetTextureCoordinates(Coord: TxorDXTexCoord);
begin
  ErrorTest(FScene.FDevice.SetTextureStageState(FStage, D3DTSS_TEXCOORDINDEX, Coord), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXTexture.SetTextureState(State: TxorDXTexState; Value: LongWord);
begin
  ErrorTest(FScene.FDevice.SetTextureStageState(FStage, State, Value), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXTexture.SetTextureState(State: TxorDXTexState; Value: Single);
begin
  ErrorTest(FScene.FDevice.SetTextureStageState(FStage, State, PLongWord(@Value)^), @errTextureState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXTexture.Unload;
begin
  if GetAnyStageActivity then ErrorTest(D3DERR_INVALIDCALL, @errTextureActive, GetErrorMsgD3D, True);
  if FTexture <> nil then FTexture := nil;
end;

{ TxorDXBumpMap }

procedure TxorDXBumpMap.Activate;
begin
  inherited;
  FScene._Device.SetTextureStageState(FStage, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_DISABLE);
  FScene._Device.SetTextureStageState(FStage, D3DTSS_BUMPENVMAT00, PWord(@FTransformation._11)^);
  FScene._Device.SetTextureStageState(FStage, D3DTSS_BUMPENVMAT01, PWord(@FTransformation._12)^);
  FScene._Device.SetTextureStageState(FStage, D3DTSS_BUMPENVMAT10, PWord(@FTransformation._21)^);
  FScene._Device.SetTextureStageState(FStage, D3DTSS_BUMPENVMAT11, PWord(@FTransformation._22)^);
  FScene._Device.SetTextureStageState(FStage, D3DTSS_BUMPENVLSCALE, PWord(@FScale)^);
  FScene._Device.SetTextureStageState(FStage, D3DTSS_BUMPENVLOFFSET, PWord(@FOffset)^);
  if FWithLuminance then ColorStageBlendAsBumpMapLuminance else ColorStageBlendAsBumpMap;
end;

constructor TxorDXBumpMap.Create(Scene: TxorDXScene);
begin
  inherited;
  FScale := 1.0;
  FOffset := 0.5;
  FStage := 1;
  FWithLuminance := True;
end;

procedure TxorDXBumpMap.Deactivate;
begin
  ColorStageBlendDisable;
  FScene._Device.SetTextureStageState(FStage, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_DISABLE);
  FScene._Device.SetTextureStageState(FStage, D3DTSS_BUMPENVMAT00, 0);
  FScene._Device.SetTextureStageState(FStage, D3DTSS_BUMPENVMAT01, 0);
  FScene._Device.SetTextureStageState(FStage, D3DTSS_BUMPENVMAT10, 0);
  FScene._Device.SetTextureStageState(FStage, D3DTSS_BUMPENVMAT11, 0);
  FScene._Device.SetTextureStageState(FStage, D3DTSS_BUMPENVLSCALE, 0);
  FScene._Device.SetTextureStageState(FStage, D3DTSS_BUMPENVLOFFSET, 0);
  inherited;
end;

{ TxorDXCustomCamera }

procedure TxorDXCustomCamera.Activate;
var
  C, Mov, Rot: TxorDXMatrix;
begin
  if Assigned(FScene.FLastActivatedCamera) then FScene.FLastActivatedCamera.Deactivate;
  FScene.FLastActivatedCamera := Self;
  Rot := FTransformation; Rot._14 := 0; Rot._24 := 0; Rot._34 := 0;             //provisorischen movevector lschen
  with FTransformation do D3DXMatrixTranslation(Mov, _14, _24, _34);            //provisorischen movevector lesen
  D3DXMatrixLookAtLH(C, FPosition, FLookAt, FUp);                               //D3DXMatrixLookAtLH(M, A, B); M besteht dann klassisch aus einer rotationsmatrix R, die B - A in z-Richung dreht, und aus einer translation von A * R. diese transf. soll auf alle punkte angewendet werden, dh. die sicht liegt dann im nullpunkt.
  D3DXMatrixMultiply(Mov, Mov, Rot);                                            //erst move, dann rot (bezieht sich stets auf den nullpunkt)
  D3DXMatrixMultiply(C, Mov, C);                                                //dann erst camera einstellungen
  ErrorTest(FScene.FDevice.SetTransform(D3DTS_VIEW, C), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  D3DXMatrixPerspectiveFovLH(C, FAngleOfView, FRatio, FLowerBoundDistance, FUpperBoundDistance);
  ErrorTest(FScene.FDevice.SetTransform(D3DTS_PROJECTION, C), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXCustomCamera.ClearTransformation;
begin
  FPosition := GetVector(0, 0, -1);
  FLookAt := GetVector(0, 0, 0);
  FUp := GetVector(0, 1, 0);
  inherited;
end;

constructor TxorDXCustomCamera.Create(Scene: TxorDXScene);
begin
  inherited; //calls cleartransformation = initialization of pos, up and lookat
  FMinAngle := 0.087;
  FIgnoreDirection := True;
  FAngleOfView := Pi/2;
  FRatio := 1;
  FLowerBoundDistance := 0.1;
  FUpperBoundDistance := 100;
  Activate;
end;

procedure TxorDXCustomCamera.Deactivate;
begin
  if FScene.FLastActivatedCamera <> Self then ErrorTest(D3DERR_INVALIDCALL, @errCamDeactivate, GetErrorMsgD3D, True);
  ErrorTest(FScene.FDevice.SetTransform(D3DTS_VIEW, IdentityMatrix), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  ErrorTest(FScene.FDevice.SetTransform(D3DTS_PROJECTION, IdentityMatrix), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  FScene.FLastActivatedCamera := nil;
end;

procedure TxorDXCustomCamera.FixTransformation;
begin
  inherited;
  FLookAtSave := FLookAt;
  FPositionSave := FPosition;
  FUpSave := Fup;
end;

function TxorDXCustomCamera.GetActive: Boolean;
begin
  Result := Self = FScene.FLastActivatedCamera;
end;

function TxorDXCustomCamera.GetDirection: TxorDXVector;
begin
  D3DXVec3Subtract(Result, FLookAt, FPosition);
end;

procedure TxorDXCustomCamera.Move(x, y, z: Single);
begin
  FTransformed := True;
  x := -x; y := -y; z := -z;
  with FTransformation do
    begin
    _14 := _14 + x;
    _24 := _24 + y;
    _34 := _34 + z;
    end;
end;

procedure TxorDXCustomCamera.MoveForward(D: Single; Transform: TxorDXCameraTransformation);
var
  Mov, Proj: TxorDXVector;
begin
  FTransformed := True;
  Mov := GetDirection;
  Case Transform of
    xorDXCamHuman: Mov.y := 0;                                                  //Mov = view direction but parallel to xz-plane
    xorDXCamID:;                                                                //Mov = view direction
    xorDXCamScene: Mov := GetVector(0, 0, 1 - 2 * Ord((Mov.z < 0) and not FIgnoreDirection)); //Mov = 001 direction; corresponding to the view side of xy-plane
    xorDXCamUp:                                                                 //Mov is orthogonal to Up but in view direction
      begin
      D3DXVec3Scale(Proj, FUp, D3DXVec3Dot(FUp, Mov) / D3DXVec3LengthSq(FUp));  //project Dir on Up
      D3DXVec3Subtract(Mov, Mov, Proj);
      end;
    else ErrorTest(E_INVALIDARG, @errBadArgument, GetErrorMsgD3D, True);
  end;
  D3DXVec3Scale(Mov, Mov, D / D3DXVec3Length(Mov));
  D3DXVec3Add(FPosition, FPosition, Mov);
  D3DXVec3Add(FLookAt, FLookAt, Mov);
end;

procedure TxorDXCustomCamera.MoveHoriz(D: Single; Transform: TxorDXCameraTransformation);
var
  Mov, UpVec, Dir: TxorDXVector;
begin
  FTransformed := True;
  Dir := GetDirection;
  Case Transform of
    xorDXCamHuman: UpVec := GetVector(0, 1 - 2 * Ord(FUp.y < 0), 0);   //mov = orthogonal to view and 010
    xorDXCamID: UpVec := FUp;                                                   //mov = orthogonal to view and up
    xorDXCamScene: UpVec := GetVector(0, 1 - 2 * Ord((FUp.y < 0) and not FIgnoreDirection), 0); //mov = orthogonal to view and up
    xorDXCamUp: UpVec := FUp;                                                   //mov = orthogonal to view and up
    else ErrorTest(E_INVALIDARG, @errBadArgument, GetErrorMsgD3D, True);
  end;
  D3DXVec3Cross(Mov, UpVec, Dir);
  D3DXVec3Scale(Mov, Mov, D / D3DXVec3Length(Mov));
  D3DXVec3Add(FPosition, FPosition, Mov);
  D3DXVec3Add(FLookAt, FLookAt, Mov);
end;

procedure TxorDXCustomCamera.MoveTo(x, y, z: Single; FixDirection: Boolean);
var
  NewPos, Dir: TxorDXVector;
begin
  FTransformed := True;
  NewPos := GetVector(x, y, z);
  if FixDirection then
    begin
    Dir := GetDirection;
    FPosition := NewPos;
    D3DXVec3Add(FLookAt, NewPos, Dir);
    end
  else
    SetPosition(NewPos);
end;

procedure TxorDXCustomCamera.MoveVert(D: Single; Transform: TxorDXCameraTransformation);                               //move directly upstairs
var
  Mov: TxorDXVector;
begin
  FTransformed := True;
  Case Transform of
    xorDXCamHuman: Mov := GetVector(0, D * (1 - 2 * Ord(FUp.y < 0)), 0); //mov = 010
    xorDXCamID:                                                                 //mov = orthogonal to view but in FUp direction
      begin
      Mov := GetDirection;
      D3DXVec3Scale(Mov, Mov, D3DXVec3Dot(FUp, Mov) / D3DXVec3LengthSq(Mov));   //projection FUp on Dir
      D3DXVec3Subtract(Mov, FUp, Mov);
      D3DXVec3Scale(Mov, Mov, D / D3DXVec3Length(Mov));
      end;
    xorDXCamScene: Mov := GetVector(0, D * (1 - 2 * Ord((FUp.y < 0) and not FIgnoreDirection)), 0); //mov = 010
    xorDXCamUp: D3DXVec3Scale(Mov, FUp, D / D3DXVec3Length(FUp));               //mov = orthogonal to view and up
    else ErrorTest(E_INVALIDARG, @errBadArgument, GetErrorMsgD3D, True);
  end;
  D3DXVec3Add(FPosition, FPosition, Mov);
  D3DXVec3Add(FLookAt, FLookAt, Mov);
end;

procedure TxorDXCustomCamera.RotateAround(x, y, z, axis_x, axis_y, axis_z, Angle: Single);
var
  U, Mov, Rot: TxorDXVector; C: TD3DXMatrix;
begin
  FTransformed := True;
  Mov := GetVector(x, y, z);
  Rot := GetVector(axis_x, axis_y, axis_z);
  D3DXMatrixRotationAxis(C, Rot, Angle);
  D3DXVec3Subtract(FPosition, FPosition, Mov);
  D3DXVec3Subtract(FLookAt, FLookAt, Mov);
  D3DXVec3Add(U, FPosition, FUp);                                               //u = absolut up-vector, not relative to position
  D3DXVec3TransformCoord(FPosition, FPosition, C);
  D3DXVec3TransformCoord(FLookAt, FLookAt, C);
  D3DXVec3TransformCoord(U, U, C);
  D3DXVec3Subtract(FUp, U, FPosition);
  D3DXVec3Add(FPosition, FPosition, Mov);
  D3DXVec3Add(FLookAt, FLookAt, Mov);
end;

procedure TxorDXCustomCamera.RotateAxis(x, y, z, Angle: Single);
begin
  inherited RotateAxis(x, y, z, -Angle);
end;

procedure TxorDXCustomCamera.RotateHoriz(Angle: Single; Transform: TxorDXCameraTransformation);
var
  Dir, Rot: TxorDXVector; C: TD3DXMatrix;
begin
  FTransformed := True;
  Dir := GetDirection;
  Case Transform of
    xorDXCamHuman: Rot := GetVector(0, 1 - 2 * Ord(FUp.y < 0), 0);     //rot = 010
    xorDXCamID:                                                                 //rot = orthogonal to view but in up direction
      begin
      D3DXVec3Scale(Rot, Dir, D3DXVec3Dot(FUp, Dir) / D3DXVec3LengthSq(Dir));   //project Up on Dir
      D3DXVec3Subtract(Rot, FUp, Rot);
      end;
    xorDXCamScene: Rot := GetVector(0, 1 - 2 * Ord((FUp.y < 0) and not FIgnoreDirection), 0); //rot = 010
    xorDXCamUp: Rot := FUp;                                                     //rot = around up
    else ErrorTest(E_INVALIDARG, @errBadArgument, GetErrorMsgD3D, True);
  end;
  D3DXMatrixRotationAxis(C, Rot, Angle);
  D3DXVec3TransformCoord(Dir, Dir, C);
  D3DXVec3Add(FLookAt, FPosition, Dir);
  if Transform <> xorDXCamUp then
    begin
    D3DXVec3TransformCoord(FUp, FUp, C);
    end;
end;

procedure TxorDXCustomCamera.RotateSlope(Angle: Single; Transform: TxorDXCameraTransformation);
var
  Dir, Rot: TxorDXVector; C: TD3DXMatrix;
begin
  FTransformed := True;
  Dir := GetDirection;
  Case Transform of
    xorDXCamHuman: Rot := Dir;                                                  //rot = view direction
    xorDXCamID: Rot := Dir;                                                     //rot = view direction
    xorDXCamScene: Rot := GetVector(0, 0, 1 - 2 * Ord((FUp.z < 0) and not FIgnoreDirection)); //rov = 001
    xorDXCamUp:                                                                 //rot = orthogonal to up but in view direction
      begin
      D3DXVec3Scale(Rot, FUp, D3DXVec3Dot(FUp, Dir) / D3DXVec3LengthSq(FUp));   //project Dir on Up
      D3DXVec3Subtract(Rot, Dir, Rot);
      end;
    else ErrorTest(E_INVALIDARG, @errBadArgument, GetErrorMsgD3D, True);
  end;
  D3DXMatrixRotationAxis(C, Rot, Angle);
  D3DXVec3TransformCoord(FUp, FUp, C);
  if Transform in [xorDXCamScene, xorDXCamUp] then
    begin
    D3DXVec3TransformCoord(Dir, Dir, C);
    D3DXVec3Add(FLookAt, FPosition, Dir);
    end;
end;

procedure TxorDXCustomCamera.RotateX(Angle: Single);
begin
  inherited RotateX(-Angle);
end;

procedure TxorDXCustomCamera.RotateY(Angle: Single);
begin
  inherited RotateY(-Angle);
end;

procedure TxorDXCustomCamera.RotateZ(Angle: Single);
begin
  inherited RotateZ(-Angle);
end;

procedure TxorDXCustomCamera.Scale(x, y, z: Single);
begin
  ErrorTest(D3DERR_INVALIDCALL, @errCamLightScale, GetErrorMsgD3D, True);
end;

procedure TxorDXCustomCamera.SetDirection(Value: TxorDXVector);
begin
  FTransformed := True;
  if D3DXVector3Equal(Value, D3DXVector3Zero) then
    begin
    ErrorTest(E_INVALIDARG, @errBadArgument, GetErrorMsgD3D, xorDXThrowCameraTransformationExceptions);
    Exit;
    end;
  if GetAngleProj(Value, FUp) < FMinAngle then ErrorTest(E_INVALIDARG, @errBadCameraAngle, GetErrorMsgD3D, xorDXThrowCameraTransformationExceptions)
  else D3DXVec3Add(FLookAt, FPosition, Value);
end;

procedure TxorDXCustomCamera.SetLookAt(Value: TxorDXVector);
var
  Dir: TxorDXVector;
begin
  FTransformed := True;
  if D3DXVector3Equal(Value, FPosition) then
    begin
    ErrorTest(E_INVALIDARG, @errBadCameraAngle, GetErrorMsgD3D, xorDXThrowCameraTransformationExceptions);
    Exit;
    end;
  D3DXVec3Subtract(Dir, Value, FPosition);
  if GetAngleProj(Dir, FUp) < FMinAngle then ErrorTest(E_INVALIDARG, @errBadCameraAngle, GetErrorMsgD3D, xorDXThrowCameraTransformationExceptions)
  else FLookAt := Value;
end;

procedure TxorDXCustomCamera.SetMinAngle(Value: Real);
begin
  if Value = 0 then ErrorTest(E_INVALIDARG, @errBadCameraAngle, GetErrorMsgD3D, xorDXThrowCameraTransformationExceptions)
  else
    FMinAngle := Value;
end;

procedure TxorDXCustomCamera.SetPosition(Value: TxorDXVector);
var
  Dir: TxorDXVector;
begin
  FTransformed := True;
  if D3DXVector3Equal(Value, FLookAt) then
    begin
    ErrorTest(E_INVALIDARG, @errBadCameraAngle, GetErrorMsgD3D, xorDXThrowCameraTransformationExceptions);
    Exit;
    end;
  D3DXVec3Subtract(Dir, FLookAt, Value);
  if GetAngleProj(Dir, FUp) < FMinAngle then ErrorTest(E_INVALIDARG, @errBadCameraAngle, GetErrorMsgD3D, xorDXThrowCameraTransformationExceptions)
  else FPosition := Value;
end;

procedure TxorDXCustomCamera.SetUp(Value: TxorDXVector);
var
  Dir: TxorDXVector;
begin
  FTransformed := True;
  if D3DXVector3Equal(Value, D3DXVector3Zero) then
    begin
    ErrorTest(E_INVALIDARG, @errBadArgument, GetErrorMsgD3D, xorDXThrowCameraTransformationExceptions);
    Exit;
    end;
  D3DXVec3Subtract(Dir, FLookAt, FPosition);
  if GetAngleProj(Dir, Value) < FMinAngle then ErrorTest(E_INVALIDARG, @errBadCameraAngle, GetErrorMsgD3D, xorDXThrowCameraTransformationExceptions)
  else FUp := Value;
end;

procedure TxorDXCustomCamera.UndoTransformation;
begin
  inherited;
  FLookAt := FLookAtSave;
  FPosition := FPositionSave;
  FUp := FupSave;
end;

{ TxorDXCamera }

procedure TxorDXCamera.RotateVert(Angle: Single; Transform: TxorDXCameraTransformation);
var
  Alpha: Single;  Dir, Dir2, Up2, Rot: TxorDXVector;  C: TD3DXMatrix;
begin
  FTransformed := True;
  Dir := GetDirection;
  Case Transform of
    xorDXCamHuman, xorDXCamID, xorDXCamUp: begin  Dir2 := Dir;  Up2 := FUp;  D3DXVec3Cross(Rot, FUp, Dir);  end;
    xorDXCamScene:
      begin
      Rot := GetVector(1 - 2 * Ord((FUp.y < 0) and not FIgnoreDirection), 0, 0); //rot = 100
      Dir2 := Dir; Dir2.x := 0;
      Up2 := FUp; Up2.x := 0;
      if D3DXVector3Equal(Dir2, D3DXVector3Zero) or D3DXVector3Equal(Up2, D3DXVector3Zero) then
        if D3DXVector3Equal(Dir2, D3DXVector3Zero) and D3DXVector3Equal(Up2, D3DXVector3Zero) then begin Dir2 := GetVector(1, 0, 0); Up2 := GetVector(-1, 0, 0); end
        else if D3DXVector3Equal(Dir2, D3DXVector3Zero) then Dir2 := GetVector(-FUp.x, 0, 0)
        else Up2 := GetVector(-Dir.x, 0, 0);
      end;
    else ErrorTest(E_INVALIDARG, @errBadArgument, GetErrorMsgD3D, True);
  end;
  //alpha test: dir and up differ a minimum angle
  Alpha := GetAngle(Dir2, Up2);
  if Angle > 0 then
    begin
    if Angle + Alpha + FMinAngle > Pi then
      begin
      Angle := Pi - Alpha - FMinAngle;
      ErrorTest(E_INVALIDARG, @errBadCameraAngle, GetErrorMsgD3D, xorDXThrowCameraTransformationExceptions);
      end;
    end
  else if Angle + Alpha < FMinAngle then
    begin
    Angle := FMinAngle - Alpha;
    ErrorTest(E_INVALIDARG, @errBadCameraAngle, GetErrorMsgD3D, xorDXThrowCameraTransformationExceptions);
    end;
  //rotation
  D3DXMatrixRotationAxis(C, Rot, Angle);
  D3DXVec3TransformCoord(Dir, Dir, C);
  D3DXVec3Add(FLookAt, FPosition, Dir);
end;

procedure TxorDXCamera.SetStandard;
begin
  FPosition := D3DXVector3(2.5, 2.5, -5);
  FLookAt := D3DXVector3(0, 0, 0);
  FUp := D3DXVector3(0, 1, 0);
  FMinAngle := 0.087;
  FAngleOfView := Pi/2;
  FRatio := 1;
  FLowerBoundDistance := 0.1;
  FUpperBoundDistance := 100;
  FixTransformation;
end;

{ TxorDXFreeCam }

procedure TxorDXFreeCam.RotateVert(Angle: Single; Transform: TxorDXCameraTransformation);
var
  Dir, Rot: TxorDXVector;  C: TD3DXMatrix;
begin
  FTransformed := True;
  Dir := GetDirection;
  Case Transform of
    xorDXCamHuman, xorDXCamID, xorDXCamUp: D3DXVec3Cross(Rot, FUp, Dir);
    xorDXCamScene: Rot := GetVector(1 - 2 * Ord((FUp.y < 0) and not FIgnoreDirection), 0, 0); //rot = 100
    else ErrorTest(E_INVALIDARG, @errBadArgument, GetErrorMsgD3D, True);
  end;
  D3DXMatrixRotationAxis(C, Rot, Angle);
  D3DXVec3TransformCoord(FUp, FUp, C);
  D3DXVec3TransformCoord(Dir, Dir, C);
  D3DXVec3Add(FLookAt, FPosition, Dir);
end;

procedure TxorDXFreeCam.SetStandard;
begin
  FPosition := D3DXVector3(0, 0, -5);
  FLookAt := D3DXVector3(0, 0, 0);
  FUp := D3DXVector3(0, 1, 0);
  FMinAngle := 0.087;
  FAngleOfView := Pi/2;
  FRatio := 1;
  FLowerBoundDistance := 0.1;
  FUpperBoundDistance := 100;
  FixTransformation;
end;

{ TxorDXCustomLight }

procedure TxorDXCustomLight.Activate;
begin
  if (not ErrorTest(FScene.FDevice.SetLight(FNumber, FProperties), @errLightMatSet, GetErrorMsgD3D, xorDXThrowAllOtherExceptions))
  or FActive then Exit;
  if ErrorTest(FScene.FDevice.LightEnable(FNumber, LongBool(True)), @errLightMatSet, GetErrorMsgD3D, xorDXThrowAllOtherExceptions) then
    begin
    FActive := True;
    Inc(FScene.FActiveLights);
    if FScene.FAutoLightOnOff then FScene.LightsOn := True;
    end;
end;

procedure TxorDXCustomLight.ClearTransformation;
begin
  FProperties.Position := GetVector(0, 1, 0);
  FProperties.Direction := GetVector(0, -1, 0);
  inherited;
end;

constructor TxorDXCustomLight.Create(Scene: TxorDXScene);
begin
  inherited; //setups position and direction by cleartransformation
  FNumber := FScene.FUsedLights;
  Inc(FScene.FUsedLights);
  if FScene.FCapsRead and (FNumber >= FScene.FCaps.MaxActiveLights) then
    ErrorTest(D3DERR_NOTAVAILABLE, @errCapsLights, GetErrorMsgD3D, xorDXThrowCapsException);
  FActive := False;
  FillChar(FProperties, Sizeof(FProperties), 0);
  FProperties.Ambient := GetColValA(0, 0, 0, 0);
  FProperties.Diffuse := GetColValA(255, 255, 255, 0);
  FProperties.Specular := GetColValA(255, 255, 255, 0);
  FProperties.Attenuation0 := 1;
  FProperties.Attenuation1 := 0;
  FProperties.Attenuation2 := 0;
  FProperties.Range := 1000.0;
  FProperties.Theta := 0.4;
  FProperties.Phi := 0.6;
  FProperties.Falloff := 2;
end;

procedure TxorDXCustomLight.Deactivate;
begin
  if not FActive then Exit;
  if ErrorTest(FScene.FDevice.LightEnable(FNumber, LongBool(False)), @errLightMatSet, GetErrorMsgD3D, xorDXThrowAllOtherExceptions) then
    begin
    FActive := False;
    Dec(FScene.FActiveLights);
    if FScene.FAutoLightOnOff and (FScene.FActiveLights = 0) then FScene.LightsOn := False;
    end;
end;

procedure TxorDXCustomLight.FixTransformation;
begin
  inherited;
  FPositionSave := FProperties.Position;
  FDirectionSave := FProperties.Direction;
end;

function TxorDXCustomLight.GetActive: Boolean;
begin
  Result := FActive;
end;

function TxorDXCustomLight.GetDirection: TxorDXVector;
begin
  Result := FProperties.Direction;
end;

function TxorDXCustomLight.GetPosition: TxorDXVector;
begin
  Result := FProperties.Position;
end;

function TxorDXCustomLight.GetProperties: PxorDXLightProperties;
begin
  Result := @FProperties;
end;

function TxorDXCustomLight.GetValue(Index: Integer): Single;
begin
  Case Index of
    0: Result := FProperties.Attenuation0;
    1: Result := FProperties.Attenuation1;
    2: Result := FProperties.Attenuation2;
    3: Result := FProperties.Range;
    4: Result := FProperties.Theta;
    5: Result := FProperties.Phi;
    6: Result := FProperties.Falloff;
  else
    Result := 0;
  end;
end;

procedure TxorDXCustomLight.Move(x, y, z: Single);
var
  V: TxorDXVector;
begin
  FTransformed := True;
  V := GetVector(x, y, z);
  D3DXVec3Add(FProperties.Position, FProperties.Position, V);
end;

procedure TxorDXCustomLight.RotateAxis(x, y, z, Angle: Single);
var
  Rot: TxorDXMatrix;
  V: TxorDXVector;
begin
  FTransformed := True;
  V := GetVector(x, y, z);
  D3DXMatrixRotationAxis(Rot, V, Angle);
  D3DXVec3TransformCoord(FProperties.Direction, FProperties.Direction, Rot);                                              //relatives lookat drehen
end;

procedure TxorDXCustomLight.RotateX(Angle: Single);
var
  Rot: TxorDXMatrix;
begin
  FTransformed := True;
  D3DXMatrixRotationX(Rot, Angle);
  D3DXVec3TransformCoord(FProperties.Direction, FProperties.Direction, Rot);                                              //relatives lookat drehen
end;

procedure TxorDXCustomLight.RotateY(Angle: Single);
var
  Rot: TxorDXMatrix;
begin
  FTransformed := True;
  D3DXMatrixRotationY(Rot, Angle);
  D3DXVec3TransformCoord(FProperties.Direction, FProperties.Direction, Rot);                                              //relatives lookat drehen
end;

procedure TxorDXCustomLight.RotateZ(Angle: Single);
var
  Rot: TxorDXMatrix;
begin
  FTransformed := True;
  D3DXMatrixRotationZ(Rot, Angle);
  D3DXVec3TransformCoord(FProperties.Direction, FProperties.Direction, Rot);                                              //relatives lookat drehen
end;

procedure TxorDXCustomLight.Scale(x, y, z: Single);
begin
  ErrorTest(D3DERR_INVALIDCALL, @errCamLightScale, GetErrorMsgD3D, True);
end;

procedure TxorDXCustomLight.SetAmbientColor(r, g, b, Alpha: Single);
begin
  FProperties.Ambient := GetColValSS(r, g, b, Alpha);
end;

procedure TxorDXCustomLight.SetAmbientColor(r, g, b, Alpha: LongWord);
begin
  FProperties.Ambient := GetColValA(r, g, b, Alpha);
end;

procedure TxorDXCustomLight.SetAmbientColor(r, g, b: LongWord; Alpha: Single);
begin
  FProperties.Ambient := GetColValS(r, g, b, Alpha);
end;

procedure TxorDXCustomLight.SetDiffuseColor(r, g, b, Alpha: Single);
begin
  FProperties.Diffuse := GetColValSS(r, g, b, Alpha);
end;

procedure TxorDXCustomLight.SetDiffuseColor(r, g, b, Alpha: LongWord);
begin
  FProperties.Diffuse := GetColValA(r, g, b, Alpha);
end;

procedure TxorDXCustomLight.SetDiffuseColor(r, g, b: LongWord; Alpha: Single);
begin
  FProperties.Diffuse := GetColValS(r, g, b, Alpha);
end;

procedure TxorDXCustomLight.SetDirection(Value: TxorDXVector);
begin
  FProperties.Direction := Value;
end;

procedure TxorDXCustomLight.SetPosition(Value: TxorDXVector);
begin
  FTransformed := True;
  FProperties.Position := Value;
end;

procedure TxorDXCustomLight.SetSpecularColor(r, g, b, Alpha: Single);
begin
  FProperties.Specular := GetColValSS(r, g, b, Alpha);
end;

procedure TxorDXCustomLight.SetSpecularColor(r, g, b, Alpha: LongWord);
begin
  FProperties.Specular := GetColValA(r, g, b, Alpha);
end;

procedure TxorDXCustomLight.SetSpecularColor(r, g, b: LongWord; Alpha: Single);
begin
  FProperties.Specular := GetColValS(r, g, b, Alpha);
end;

procedure TxorDXCustomLight.SetValue(Index: Integer; Value: Single);
begin
  Case Index of
    0: FProperties.Attenuation0 := Value;
    1: FProperties.Attenuation1 := Value;
    2: FProperties.Attenuation2 := Value;
    3: FProperties.Range := Value;
    4: FProperties.Theta := Value;
    5: FProperties.Phi := Value;
    6: FProperties.Falloff := Value;
  end;
end;

procedure TxorDXCustomLight.UndoTransformation;
begin
  inherited;
  FProperties.Position := FPositionSave;
  FProperties.Direction := FDirectionSave;
end;

{ TxorDXPointLight }

constructor TxorDXPointLight.Create(Scene: TxorDXScene);
begin
  inherited;
  FProperties._Type := D3DLIGHT_POINT;
end;

{ TxorDXSpotLight }

constructor TxorDXSpotLight.Create(Scene: TxorDXScene);
begin
  inherited;
  FProperties._Type := D3DLIGHT_SPOT;
//  FProperties.Position := GetVector(3, 8, 3);
end;

{ TxorDXDirectionalLight }

constructor TxorDXDirectionalLight.Create(Scene: TxorDXScene);
begin
  inherited;
  FProperties._Type := D3DLIGHT_DIRECTIONAL;
end;

{ TxorDXCustomInsertObject }

constructor TxorDXCustomInsertObject.Create(Scene: TxorDXScene);
begin
  inherited;
  FVisible := True;
  FillChar(FMaterial, Sizeof(FMaterial), 0);
  FMaterial.Ambient := GetColValS(200, 200, 200, 0.7);
  FMaterial.Diffuse := GetColValS(200, 200, 200, 0.7);
  FMaterial.Specular := GetColValS(0, 0, 0, 0.7);
  FMaterial.Emissive := GetColValS(0, 0, 0, 0.7);
  FMaterial.Power := 15;
end;

procedure TxorDXCustomInsertObject.Draw;
begin
  if FScene.FLastInsertedObject <> Self then ErrorTest(D3DERR_INVALIDCALL, @errNotLastInserted, GetErrorMsgD3D, True);
  if FTransformed or FScene.FTransformed then FScene.SetTransformation;
  FTransformed := False;
end;

function TxorDXCustomInsertObject.GetMaterialPointer: PxorDXMaterial;
begin
  Result := @FMaterial;
end;

function TxorDXCustomInsertObject.GetPower: Single;
begin
  Result := FMaterial.Power;
end;

procedure TxorDXCustomInsertObject.Insert;
begin
  FScene.FLastInsertedObject := Self;
  ErrorTest(FScene.FDevice.SetMaterial(FMaterial), @errLightMatSet, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  FTransformed := True;
  InsertByShadow;
end;

procedure TxorDXCustomInsertObject.SetMaterialAmbientColor(r, g, b, Alpha: Single);
begin
  FMaterial.Ambient := GetColValSS(r, g, b, Alpha);
end;

procedure TxorDXCustomInsertObject.SetMaterialAmbientColor(r, g, b, Alpha: LongWord);
begin
  FMaterial.Ambient := GetColValA(r, g, b, Alpha);
end;

procedure TxorDXCustomInsertObject.InsertByShadow;
begin
  //nothing to do
end;

procedure TxorDXCustomInsertObject.SetMaterialAmbientColor(r, g, b: LongWord; Alpha: Single);
begin
  FMaterial.Ambient := GetColValS(r, g, b, Alpha);
end;

procedure TxorDXCustomInsertObject.SetMaterialDiffuseColor(r, g, b, Alpha: Single);
begin
  FMaterial.Diffuse := GetColValSS(r, g, b, Alpha);
end;

procedure TxorDXCustomInsertObject.SetMaterialDiffuseColor(r, g, b, Alpha: LongWord);
begin
  FMaterial.Diffuse := GetColValA(r, g, b, Alpha);
end;

procedure TxorDXCustomInsertObject.SetMaterialDiffuseColor(r, g, b: LongWord; Alpha: Single);
begin
  FMaterial.Diffuse := GetColValS(r, g, b, Alpha);
end;

procedure TxorDXCustomInsertObject.SetMaterialEmissiveColor(r, g, b, Alpha: Single);
begin
  FMaterial.Emissive := GetColValSS(r, g, b, Alpha);
end;

procedure TxorDXCustomInsertObject.SetMaterialEmissiveColor(r, g, b, Alpha: LongWord);
begin
  FMaterial.Emissive := GetColValA(r, g, b, Alpha);
end;

procedure TxorDXCustomInsertObject.SetMaterialEmissiveColor(r, g, b: LongWord; Alpha: Single);
begin
  FMaterial.Emissive := GetColValS(r, g, b, Alpha);
end;

procedure TxorDXCustomInsertObject.SetMaterialSpecularColor(r, g, b, Alpha: Single);
begin
  FMaterial.Specular := GetColValSS(r, g, b, Alpha);
end;

procedure TxorDXCustomInsertObject.SetMaterialSpecularColor(r, g, b, Alpha: LongWord);
begin
  FMaterial.Specular := GetColValA(r, g, b, Alpha);
end;

procedure TxorDXCustomInsertObject.SetMaterialSpecularColor(r, g, b: LongWord; Alpha: Single);
begin
  FMaterial.Specular := GetColValS(r, g, b, Alpha);
end;

procedure TxorDXCustomInsertObject.SetPower(const Value: Single);
begin
  FMaterial.Power := Value;
end;

{ TxorDXPolygon }

function TxorDXPolygon.Center: TxorDXVector;
var
  i: Cardinal; xs, ys, zs: Single;
begin
  FillChar(Result, SizeOf(Result), 0);
  if FVertexCount = 0 then Exit;
  xs := 0; ys := 0; zs := 0;
  for i := 0 to FVertexCount - 1 do with PxorDXVertexPlain(Vertex[i])^ do
    begin
    xs := xs + Position.x; ys := ys + Position.y; zs := zs + Position.z;
    end;
  xs := xs / FVertexCount; ys := ys / FVertexCount; zs := zs / FVertexCount;
  Result.x := xs; Result.y := ys; Result.z := zs;
end;

constructor TxorDXPolygon.Create(Scene: TxorDXScene; VertexCount: Cardinal);
begin
  CreateParam(Scene, VertexCount, xorDXVertexSizeClassic, xorDXVertexFlagClassic)
end;

constructor TxorDXPolygon.CreateParam(Scene: TxorDXScene; VertexCount: Cardinal; VertexSize: TxorDXVertexSize; VertexFlags: TxorDXVertexFlag);
begin
  inherited Create(Scene);
  FDrawMode := xorDXPrimTriangleStrip;
  FVertexCount := VertexCount;
  if FScene.FCapsRead and (VertexCount > FScene.FCaps.MaxVertexIndex) then
    ErrorTest(D3DERR_NOTAVAILABLE, @errCapsVertexCount, GetErrorMsgD3D, xorDXThrowCapsException);
  FVertexSize := VertexSize;
  if FScene.FCapsRead and (FVertexSize > FScene.FCaps.MaxStreamStride) then
    ErrorTest(D3DERR_NOTAVAILABLE, @errCapsVertexSize, GetErrorMsgD3D, xorDXThrowCapsException);
  FVertexFlags := VertexFlags;
  FVertexMem := FVertexSize * FVertexCount;
  SetDrawAll(True);
  FScene.SetVertexBufferSize(FVertexMem);
  FScene.SetVertexBufferFlags(FVertexFlags);
  SetLength(FVertices, FVertexMem);
end;

destructor TxorDXPolygon.Destroy;
begin
  if Assigned(FVertices) then FVertices := nil;
  inherited;
end;

procedure TxorDXPolygon.Draw;
begin
  inherited;
  if FVisible then
    ErrorTest(FScene.FDevice.DrawPrimitive(FDrawMode, FDrawStart, FDrawPrimCount), @errDrawing, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

function TxorDXPolygon.GetMaxPrimCount: Cardinal;
begin
  Result := GetPrimCount(FDrawMode, FVertexCount)
end;

function TxorDXPolygon.GetVertexPointer(Idx: Cardinal): Pointer;
begin
  Result := @(FVertices[Idx * FVertexSize]);
end;

procedure TxorDXPolygon.InitNormalAndTextureVectors;
var
  i, idx1, idx2, idx3: Cardinal; P1, P2, P3: PxorDXVertexClassic; D1, D2, D3: TxorDXVector; NCount: Array of Integer; VF: Word;
begin
  if (FVertexCount < 3) then Exit;
  if not VertexFormatHasClassicStructure(FVertexFlags) then ErrorTest(D3DERR_INVALIDCALL, @errBadVertexFormat, GetErrorMsgD3D, True);
  if (FDrawMode <> xorDXPrimTriangleStrip) and (FDrawMode <> xorDXPrimTriangleList)
    and (FDrawMode <> xorDXPrimTriangleFan) then ErrorTest(D3DERR_INVALIDCALL, @errBadDrawMode, GetErrorMsgD3D, True);
  SetLength(NCount, FVertexCount);  idx1 := 0;  idx2 := 0;  idx3 := 0;
  for i := 0 to FVertexCount - 1 do PxorDXVertexClassic(Vertex[i])^.Normal := D3DXVector3Zero;
  for i := 0 to GetMaxPrimCount - 1 do
    begin
    Case FDrawMode of
      xorDXPrimTriangleStrip: begin idx1 := i + (i and $01) shl 1; idx2 := i + 1;  idx3 := i + ((i + 1) and $01) shl 1; end;
      xorDXPrimTriangleList: begin idx1 := 3 * i;  idx2 := 3 * i + 1;  idx3 := 3 * i + 2;  end;
      xorDXPrimTriangleFan: begin idx1 := 0;  idx2 := i + 1;  idx3 := i + 2;  end;
    end;
    P1 := Vertex[idx1];  P2 := Vertex[idx2];  P3 := Vertex[idx3];
    D3DXVec3Subtract(D1, P2.Position, P1.Position);
    D3DXVec3Subtract(D2, P3.Position, P1.Position);
    D3DXVec3Cross(D3, D1, D2);
    D3DXVec3Normalize(D3, D3);
    D3DXVec3Add(P1.Normal, P1.Normal, D3);
    D3DXVec3Add(P2.Normal, P2.Normal, D3);
    D3DXVec3Add(P3.Normal, P3.Normal, D3);
    Inc(NCount[idx1]);  Inc(NCount[idx2]);  Inc(NCount[idx3]);
    end;
  for i := 0 to FVertexCount - 1 do
    if NCount[i] > 0 then with PxorDXVertexClassic(Vertex[i])^ do D3DXVec3Scale(Normal, Normal, 1/NCount[i]);
  NCount := nil;
  VF := VertexFormatHasTextureStructure(FVertexFlags);
  if VF > 0 then
    for i := 0 to FVertexCount - 1 do
      begin
      PxorDXVertexTexture(Vertex[i]).TexCoordinates := GetCoord(0, 0);
      if 2 <= VF then PxorDXVertexTexture2(Vertex[i]).TexCoordinates2 := GetCoord(0, 0);
      end;
end;

procedure TxorDXPolygon.Insert;
var
  P: PByte;
begin
  inherited;
  //Vertexbuffer locken und Daten reinkopieren
  if ErrorTest(FScene.FVertexBuffer.Lock(0, FVertexMem, P, 0), @errBufferLock, GetErrorMsgD3D, xorDXThrowAllOtherExceptions) then
    begin
    System.Move(FVertices[0], P^, FVertexMem);
    ErrorTest(FScene.FVertexBuffer.Unlock, @errBufferLock, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    end;
  //Buffer an Device bergeben
  ErrorTest(FScene.FDevice.SetStreamSource(0, FScene.FVertexBuffer, FVertexSize), @errStreamSource, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
  ErrorTest(FScene.FDevice.SetVertexShader(FVertexFlags), @errVertexShader, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXPolygon.MoveCenter(MoveCenterToNull: Boolean);
begin
  with Center do
    if MoveCenterToNull then Move(-x - FTransformation._41, -y - FTransformation._42, -z - FTransformation._43)
    else Move(x - FTransformation._41, y - FTransformation._42, z - FTransformation._43);
end;

procedure TxorDXPolygon.SetDrawAll(Value: Boolean);
begin
  FDrawAllVertices := Value;
  if Value then
    begin
    FDrawStart := 0;
    FDrawPrimCount := GetMaxPrimCount;
    if FScene.FCapsRead and (FDrawPrimCount > FScene.FCaps.MaxPrimitiveCount) then
      ErrorTest(D3DERR_NOTAVAILABLE, @errCapsPrimCount, GetErrorMsgD3D, xorDXThrowCapsException);
    end;                                        
end;

procedure TxorDXPolygon.SetDrawMode(Value: TxorDXPrimitiveType);
begin
  FDrawMode := Value;
  if FDrawAllVertices then FDrawPrimCount := GetMaxPrimCount;
end;

procedure TxorDXPolygon.SetDrawPrimCount(Value: Cardinal);
begin
  FDrawPrimCount := Value;
  SetDrawAll(False);
end;

procedure TxorDXPolygon.SetDrawStart(Value: Cardinal);
begin
  FDrawStart := Value;
  SetDrawAll(False);
end;

procedure TxorDXPolygon.SetStandard;
var
  i, i2, VH: Cardinal; theta: Single; P0, PVH: PxorDXVertexClassic;
begin
  if (FVertexCount <= 1) then Exit;
  if not VertexFormatHasClassicStructure(FVertexFlags) then ErrorTest(D3DERR_INVALIDCALL, @errBadVertexFormat, GetErrorMsgD3D, True);
  SetDrawMode(xorDXPrimTriangleStrip);
  SetDrawAll(True);
  VH := FVertexCount div 2 - 1;
  for i := 0 to VH do
    begin
    i2 := 2 * i;
    theta := PI * i2 / VH;
    PxorDXVertexClassic(Vertex[i2])^ := GetVertexClassic(cos(theta), -1.0, sin(theta), 0.0, 0.0, 0.0, 40, 0, 255, 0.7);
    PxorDXVertexClassic(Vertex[i2 + 1])^ := GetVertexClassic(cos(theta), 1.0, sin(theta), 0,0,0, 40, 255, 0, 0.7);
    end;
  InitNormalAndTextureVectors;
  //letzte und erste beiden punkte fallen hier zusammen, mischen die normalen
  P0 := PxorDXVertexClassic(Vertex[0]);
  PVH := PxorDXVertexClassic(Vertex[2 * VH]);
  D3DXVec3Add(P0.Normal, P0.Normal, PVH.Normal);
  D3DXVec3Normalize(P0.Normal, P0.Normal);
  PVH.Normal := P0.Normal;
  P0 := PxorDXVertexClassic(Vertex[1]);
  PVH := PxorDXVertexClassic(Vertex[2 * VH + 1]);
  D3DXVec3Add(P0.Normal, P0.Normal, PVH.Normal);
  D3DXVec3Normalize(P0.Normal, P0.Normal);
  PVH.Normal := P0.Normal;
end;

{ TxorDXPointAdapter }

constructor TxorDXPointAdapter.Create(Scene: TxorDXScene; MaxVertexCount: Cardinal);
begin
  inherited CreateParam(Scene, MaxVertexCount, xorDXVertexSizePoint, xorDXVertexFlagPoint);
  SetDrawMode(xorDXPrimPointList);
  SetDrawAll(True);
  FUseColor := False;
  FColor := GetColS(clRed, 0.7);
  FPointSize := 10.0;
end;

procedure TxorDXPointAdapter.Insert;
begin
  if (FScene.FLastInsertedObject = nil) or not (FScene.FLastInsertedObject is TxorDXPolygon)
  then ErrorTest(D3DERR_NOTAVAILABLE, @errNoPolygon, GetErrorMsgD3D, True);
  Insert(TxorDXPolygon(FScene.FLastInsertedObject));
end;

procedure TxorDXPointAdapter.Insert(Polygon: TxorDXPolygon);
var
  i: Cardinal;
begin
  if not VertexFormatHasClassicStructure(Polygon.FVertexFlags) then ErrorTest(D3DERR_NOTAVAILABLE, @errNoClassicStructure, GetErrorMsgD3D, True);
  FPreviousObject := Polygon;
  for i := 0 to Min(FVertexCount, FPreviousObject.FVertexCount) - 1 do
    begin
    System.Move(Polygon.Vertex[i]^, Vertex[i]^, xorDXVertexSizeClassic); //koordinaten rberschaufeln
    if FUseColor then PxorDXVertexPoint(Vertex[i])^.Color := FColor
    else
      System.Move(PxorDXVertexClassic(Polygon.Vertex[i])^.Color, PxorDXVertexPoint(Vertex[i])^.Color, SizeOf(TxorDXColor));  //farbe setzen
    PxorDXVertexPoint(Vertex[i])^.Size := FPointSize;
    end;
  inherited Insert;
end;

procedure TxorDXPointAdapter.SetColor(r, g, b, Alpha: LongWord);
begin
  FColor := GetColRGBA(r, g, b, Alpha);
end;

{ TxorDXShadow }

constructor TxorDXShadow.Create(Scene: TxorDXScene; MaxVertexCount: Cardinal);
begin
  inherited CreateParam(Scene, MaxVertexCount, xorDXVertexSizeColored, xorDXVertexFlagColored);
  FDeactivateFog := True;
  SetStandard;
end;

procedure TxorDXShadow.Draw;
var
  World, NewWorld: TxorDXMatrix;
  RSLight, RSFog, RSFogRadial: LongWord;
  SceneIsTransformed, ObjectIsTransformed, ObjectIsVisible: Boolean;
begin
  FTransformed := False;
  SceneIsTransformed := FScene.FTransformed;
  FScene.FTransformed := False;
  if not FVisible then inherited
  else
    begin
    FVisible := False;
    inherited;
    FVisible := True;
    D3DXMatrixShadow(NewWorld, FLightPosition, FPlane);                         //Projektionsmatrix
    D3DXMatrixMultiply(World, FPreviousObject.FTransformation, FScene.FTransformation);
    D3DXMatrixMultiply(NewWorld, World, NewWorld);
    D3DXMatrixMultiply(NewWorld, NewWorld, FTransformation);
    with FScene.FDevice do
      begin
      ErrorTest(GetTransform(D3DTS_WORLD, World), @errTransform, GetErrorMsgD3D, xorDXThrowAllOtherExceptions); //alte Weltmatrix speichern
      ErrorTest(SetTransform(D3DTS_WORLD, NewWorld), @errTransform, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);                                      //neue Weltmatrix setzen
      ErrorTest(GetRenderState(D3DRS_LIGHTING, RSLight), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
      ErrorTest(GetRenderState(D3DRS_FOGENABLE, RSFog), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
      ErrorTest(GetRenderState(D3DRS_RANGEFOGENABLE, RSFogRadial), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
      ErrorTest(SetRenderState(D3DRS_LIGHTING, Ord(False)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions); //kurzzeitig Licht und Nebel ausschalten, um Schatten in Originalfarbe zu zeigen
      if FDeactivateFog then
        begin
        ErrorTest(SetRenderState(D3DRS_FOGENABLE, Ord(False)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
        ErrorTest(SetRenderState(D3DRS_RANGEFOGENABLE, Ord(False)), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
        end;
      FScene.FLastInsertedObject := FPreviousObject;                            //kurzzeitig altes Objekt wieder einfgen
      with FPreviousObject do
        begin
        ObjectIsTransformed := FTransformed;
        FTransformed := False;
        ObjectIsVisible := FVisible;
        FVisible := True;
        Draw;
        FTransformed := ObjectIsTransformed;
        FVisible := ObjectIsVisible;
        end;
      FScene.FLastInsertedObject := Self;
      ErrorTest(SetRenderState(D3DRS_LIGHTING, RSLight), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions); //alten Lichtzustand und Nebel wiederherstellen
      ErrorTest(SetRenderState(D3DRS_FOGENABLE, RSFog), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
      ErrorTest(SetRenderState(D3DRS_RANGEFOGENABLE, RSFogRadial), @errRenderState, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
      ErrorTest(SetTransform(D3DTS_WORLD, World), @errTransform, GetErrorMsgD3D, xorDXThrowAllOtherExceptions); //alte Weltmatrix wiederherstellen
      end;
    end;
  FScene.FTransformed := SceneIsTransformed;
end;

function TxorDXShadow.GetLightPositionVector: TxorDXVector;
begin
  System.Move(FLightPosition, Result, SizeOf(Result));
end;

function TxorDXShadow.GetLightScattering: Single;
begin
  Result := FLightPosition.w;
end;

procedure TxorDXShadow.Insert;
begin
  if (FScene.FLastInsertedObject = nil) or not (FScene.FLastInsertedObject is TxorDXPolygon) then
    ErrorTest(D3DERR_NOTAVAILABLE, @errNoPolygon, GetErrorMsgD3D, True);
  Insert(TxorDXPolygon(FScene.FLastInsertedObject));
end;

procedure TxorDXShadow.Insert(Polygon: TxorDXPolygon);
var
  i: Cardinal;
begin
  FPreviousObject := Polygon;
  for i := 0 to Min(FVertexCount, FPreviousObject.FVertexCount) - 1 do
    begin
    System.Move(FPreviousObject.Vertex[i]^, Vertex[i]^, xorDXVertexSizePlain); //koordinaten rberschaufeln
    System.Move(FColor, PxorDXVertexColored(Vertex[i])^.Color, SizeOf(TxorDXColor));  //farbe setzen
    end;
  if FScene.FLastInsertedObject <> FPreviousObject then FPreviousObject.InsertByShadow; //ruft alle fr draw wichtigen informationen auf (falls inzwischen anderes objekt eingefgt wurde)
  inherited Insert;
end;

procedure TxorDXShadow.SetColor(r, g, b, Alpha: LongWord);
begin
  FColor := GetColRGBA(r, g, b, Alpha);
end;

procedure TxorDXShadow.SetLightPosition(x, y, z: Single);
begin
  FLightPosition.x := x;
  FLightPosition.y := y;
  FLightPosition.z := z;
end;

procedure TxorDXShadow.SetLightPositionVector(const Value: TxorDXVector);
begin
  System.Move(Value, FLightPosition, SizeOf(Value));
end;

procedure TxorDXShadow.SetLightScattering(Scattering: Single);
begin
  FLightPosition.w := Scattering;
end;

procedure TxorDXShadow.SetStandard;
var
  V, N: TxorDXVector;
begin
  V := GetVector(0, -5, 0);
  N := GetVector(0, 1, 0);
  FLightPosition := D3DXVector4(0, 10, 0, 0.2);
  D3DXPlaneFromPointNormal(FPlane, V, N);
  FColor := GetColRGBS(240, 240, 240, 0.7);
end;

{ TxorDXIndexedPolygon }

constructor TxorDXIndexedPolygon.Create(Scene: TxorDXScene; VertexCount, IndexCount: Cardinal);
begin
  CreateParam(Scene, VertexCount, IndexCount, xorDXVertexSizeClassic, xorDXVertexFlagClassic);
end;

constructor TxorDXIndexedPolygon.CreateParam(Scene: TxorDXScene; VertexCount, IndexCount: Cardinal; VertexSize: TxorDXVertexSize; VertexFlags: TxorDXVertexFlag);
var
  IndexWordSize: Cardinal;
begin
  inherited CreateParam(Scene, VertexCount, VertexSize, VertexFlags);            //setzt DrawAll true, so dass start=0
  if (VertexCount > 65536) and (not Scene.FUse32BitIndex) then
    ErrorTest(D3DERR_NOTAVAILABLE, @errIndexSize, GetErrorMsgD3D, xorDXThrowIndexSizeException);
  FScene.FFormatFixed := True;                                                  //ab jetzt kann das indexformat nicht mehr gendert werden
  FDrawMinUsedVertexIndex := 0;
  FDrawUsedVertexCount := VertexCount;
  FIndexCount := IndexCount;
  SetDrawMode(xorDXPrimTriangleList);                                           //setzt primcount=max, da drawall true ist
  IndexWordSize := FIndexCount shl Ord(FScene.FUse32BitIndex);                  //Gre in Words
  FIndexMem := IndexWordSize * SizeOf(TxorDXIndex);                             //Gre in Bytes
  FScene.SetIndexBufferSize(FIndexMem);
  SetLength(FIndices, IndexWordSize);
end;

destructor TxorDXIndexedPolygon.Destroy;
begin
  if Assigned(FIndices) then FIndices := nil;
  inherited;
end;

procedure TxorDXIndexedPolygon.Draw;
begin
  if not FVisible then inherited
  else
    begin
    FVisible := False;
    inherited;
    FVisible := True;
    ErrorTest(FScene.FDevice.DrawIndexedPrimitive(FDrawMode, FDrawMinUsedVertexIndex, FDrawUsedVertexCount, FDrawStart, FDrawPrimCount), @errDrawing, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    end;
end;

function TxorDXIndexedPolygon.GetIndex(Idx: Cardinal): TxorDXIndex;
begin
  Result := FIndices[Idx];
end;

function TxorDXIndexedPolygon.GetIndex32(Idx: Cardinal): TxorDXIndex32;
begin
  System.Move(FIndices[Idx shl 1], Result, SizeOf(TxorDXIndex32));
end;

function TxorDXIndexedPolygon.GetLinePointer(Idx: Cardinal): PxorDXLine;
begin
  Case FDrawMode of
    D3DPT_LINELIST: Result := @(FIndices[Idx shl 1]);                           //linie zwischen 2a und 2a+1
    D3DPT_LINESTRIP: Result := @(FIndices[Idx]);                                //linie zwischen zwei aufeinanderfolgenden punkten
  else
    Result := nil;  ErrorTest(D3DERR_INVALIDCALL, @errBadDrawMode, GetErrorMsgD3D, True);
  end
end;

function TxorDXIndexedPolygon.GetLinePointer32(Idx: Cardinal): PxorDXLine32;
begin
  Case FDrawMode of
    D3DPT_LINELIST: Result := @(FIndices[Idx shl 2]);                           //linie zwischen 2a und 2a+1
    D3DPT_LINESTRIP: Result := @(FIndices[Idx shl 1]);                          //linie zwischen zwei aufeinanderfolgenden punkten
  else
    Result := nil;  ErrorTest(D3DERR_INVALIDCALL, @errBadDrawMode, GetErrorMsgD3D, True);
  end
end;

function TxorDXIndexedPolygon.GetMaxPrimCount: Cardinal;
begin
  Result := GetPrimCount(FDrawMode, FIndexCount)
end;

function TxorDXIndexedPolygon.GetTrianglePointer(Idx: Cardinal): PxorDXTriangle;
begin
  Case FDrawMode of
    D3DPT_TRIANGLELIST: Result := @(FIndices[Idx * 3]);                         //dreieck zwischen 3a, 3a+1 und 3a+2
    D3DPT_TRIANGLESTRIP: Result := @(FIndices[Idx]);                            //dreieck zwischen je 3 aufeinanderfolgenden punkten
  else
    Result := nil;  ErrorTest(D3DERR_INVALIDCALL, @errBadDrawMode, GetErrorMsgD3D, True);
  end
end;

function TxorDXIndexedPolygon.GetTrianglePointer32(Idx: Cardinal): PxorDXTriangle32;
begin
  Case FDrawMode of
    D3DPT_TRIANGLELIST: Result := @(FIndices[Idx * 6]);                         //dreieck zwischen 3a, 3a+1 und 3a+2
    D3DPT_TRIANGLESTRIP: Result := @(FIndices[Idx shl 1]);                      //dreieck zwischen je 3 aufeinanderfolgenden punkten
  else
    Result := nil;  ErrorTest(D3DERR_INVALIDCALL, @errBadDrawMode, GetErrorMsgD3D, True);
  end
end;

procedure TxorDXIndexedPolygon.InitNormalAndTextureVectors;
var
  i, idx1, idx2, idx3: Cardinal; P1, P2, P3: PxorDXVertexClassic; D1, D2, D3: TxorDXVector; NCount: Array of Integer; VF: Word;
begin
  if (FVertexCount < 3) or (FIndexCount < 3)  then Exit;
  if not VertexFormatHasClassicStructure(FVertexFlags) then ErrorTest(D3DERR_INVALIDCALL, @errBadVertexFormat, GetErrorMsgD3D, True);
  if (FDrawMode <> xorDXPrimTriangleStrip) and (FDrawMode <> xorDXPrimTriangleList)
    and (FDrawMode <> xorDXPrimTriangleFan) then ErrorTest(D3DERR_INVALIDCALL, @errBadDrawMode, GetErrorMsgD3D, True);
  SetLength(NCount, FVertexCount); idx1  := 0;  idx2 := 0;  idx3 := 0;
  for i := 0 to FVertexCount - 1 do PxorDXVertexClassic(Vertex[i])^.Normal := D3DXVector3Zero;
  for i := 0 to GetMaxPrimCount - 1 do
    begin
    if FScene.FUse32BitIndex then
      Case FDrawMode of
        xorDXPrimTriangleStrip: begin idx1 := Idx32[i + 2 * (i mod 2)]; idx2 := Idx32[i + 1]; idx3 := Idx32[i + 2 - 2 * (i mod 2)]; end;
        xorDXPrimTriangleList: begin idx1 := Idx32[3 * i]; idx2 := Idx32[3 * i + 1]; idx3 := Idx32[3 * i + 2]; end;
        xorDXPrimTriangleFan: begin idx1 := Idx32[0]; idx2 := Idx32[i + 1]; idx3 := Idx32[i + 2]; end;
      end
    else
      Case FDrawMode of
        xorDXPrimTriangleStrip: begin idx1 := Idx[i]; idx2 := Idx[i + 1]; idx3 := Idx[i + 2]; end;
        xorDXPrimTriangleList: begin idx1 := Idx[3 * i]; idx2 := Idx[3 * i + 1]; idx3 := Idx[3 * i + 2]; end;
        xorDXPrimTriangleFan: begin idx1 := Idx[0]; idx2 := Idx[i + 1]; idx3 := Idx[i + 2]; end;
      end;
    P1 := Vertex[idx1];  P2 := Vertex[idx2];  P3 := Vertex[idx3];
    D3DXVec3Subtract(D1, P2.Position, P1.Position);
    D3DXVec3Subtract(D2, P3.Position, P1.Position);
    D3DXVec3Cross(D3, D1, D2);
    D3DXVec3Normalize(D3, D3);
    D3DXVec3Add(P1.Normal, P1.Normal, D3);
    D3DXVec3Add(P2.Normal, P2.Normal, D3);
    D3DXVec3Add(P3.Normal, P3.Normal, D3);
    Inc(NCount[idx1]);  Inc(NCount[idx2]);  Inc(NCount[idx3]);
    end;
  for i := 0 to FVertexCount - 1 do
    if NCount[i] > 0 then with PxorDXVertexClassic(Vertex[i])^ do D3DXVec3Scale(Normal, Normal, 1/NCount[i]);
  NCount := nil;
  VF := VertexFormatHasTextureStructure(FVertexFlags);
  if VF > 0 then
    for i := 0 to FVertexCount - 1 do
      begin
      PxorDXVertexTexture(Vertex[i]).TexCoordinates := GetCoord(0, 0);
      if 2 <= VF then PxorDXVertexTexture2(Vertex[i]).TexCoordinates2 := GetCoord(0, 0);
      end;
end;

procedure TxorDXIndexedPolygon.InsertByShadow;
var
  P: PByte;
begin
  //Indexbuffer locken und Daten reinkopieren
  if ErrorTest(FScene.FIndexBuffer.Lock(0, FIndexMem, P, 0), @errBufferLock, GetErrorMsgD3D, xorDXThrowAllOtherExceptions) then
    begin
    System.Move(FIndices[0], P^, FIndexMem);
    ErrorTest(FScene.FIndexBuffer.Unlock, @errBufferLock, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
    end;
  //Buffer an Device bergeben
  ErrorTest(FScene.FDevice.SetIndices(FScene.FIndexBuffer, 0), @errIndices, GetErrorMsgD3D, xorDXThrowAllOtherExceptions);
end;

procedure TxorDXIndexedPolygon.SetIndex(Idx: Cardinal; Value: TxorDXIndex);
begin
  FIndices[Idx] := Value;
end;

procedure TxorDXIndexedPolygon.SetIndex32(Idx: Cardinal; Value: TxorDXIndex32);
begin
  System.Move(Value, FIndices[Idx shl 1], SizeOf(TxorDXIndex32));
end;

procedure TxorDXIndexedPolygon.SetIndices;
var
  u, v, w, z, base: Cardinal;
begin
  FillChar(FIndices[0], (FIndexCount * SizeOf(TxorDXIndex)) shl Ord(FScene.FUse32BitIndex), 0);
  w := VertexCount; w := Trunc(sqrt(w));
  if w < 2 then exit;
  z := 0;
  if FScene.FUse32BitIndex then
    for v := 0 to w - 2 do
      for u := 0 to w - 2 do
        begin
        if z >= FDrawPrimCount then break;
        base := v * w + u;
        Triangle32[z]^ := GetTriangle32(base + 1, base, base + w);
        inc(z);
        end
  else
    for v := 0 to w - 2 do
      for u := 0 to w - 2 do
        begin
        if z >= FDrawPrimCount then break;
        base := v * w + u;
        Triangle[z]^ := GetTriangle(base + 1, base, base + w);
        inc(z);
        end;
end;

procedure TxorDXIndexedPolygon.SetStandard;
var
  u, v, w: Cardinal;
  x, y, h: Single;
  P, Po, Pu, Pl, Pr: PxorDXVertexClassic;
  N1, N2: TxorDXVector;
begin
  if (FVertexCount < 4) or (FIndexCount < 6) then Exit;
  if not VertexFormatHasClassicStructure(FVertexFlags) then ErrorTest(D3DERR_INVALIDCALL, @errBadVertexFormat, GetErrorMsgD3D, True);
  SetDrawMode(xorDXPrimTriangleList);
  SetDrawAll(True);
  w := Trunc(Sqrt(FVertexCount));
  for v := 0 to w - 1 do
    for u := 0 to w - 1 do
      begin
      x := 3 * (u / w - 0.5);  y := 3 * (v / w - 0.5);  h := 0.4 * x * x - 0.3 * y * y;
      PxorDXVertexClassic(Vertex[v * w + u])^ := GetVertexClassic(x, h, y,  0, 0, 0,  Trunc(Abs(250 * sin(x))), $B0, Trunc(Abs(150 * sin(h))), 0.7)
      end;
  SetIndices;
  //normalvectors
  for v := 0 to w - 1 do
    for u := 0 to w - 1 do
      begin
      P := Vertex[v * w + u];
      if u = 0 then Pl := P else Pl := Vertex[v * w + u - 1];
      if u = w - 1 then Pr := P else Pr := Vertex[v * w + u + 1];
      if v = 0 then Pu := P else Pu := Vertex[v * w + u - w];
      if v = w - 1 then Po := P else Po := Vertex[v * w + u + w];
      D3DXVec3Subtract(N1, Po.Position, Pu.Position);
      D3DXVec3Subtract(N2, Pr.Position, Pl.Position);
      D3DXVec3Cross(P.Normal, N1, N2);
      D3DXVec3Normalize(P.Normal, P.Normal);
      end;
end;

{ TxorDXCustomMap }

function TxorDXCustomMap.GetVertexPointer(u, v: Cardinal): Pointer;
begin
  Result := @(FVertices[(v * FMapWidth + u) * FVertexSize]);
end;

procedure TxorDXCustomMap.InitNormalAndTextureVectors;
var
  u, v, NCount: Cardinal; P: PxorDXVertexClassic; VF: Word;

          procedure Cross(P1, P2: PxorDXVertexClassic);
          var
            V1, V2, N: TxorDXVector;
          begin
            D3DXVec3Subtract(V1, P1.Position, P.Position);
            D3DXVec3Subtract(V2, P2.Position, P.Position);
            D3DXVec3Cross(N, V1, V2);
            D3DXVec3Normalize(N, N);
            D3DXVec3Add(P.Normal, P.Normal, N);
            Inc(NCount);
          end;
begin
  if (FMapHeight < 1) or (FMapWidth < 1) then Exit;
  if not VertexFormatHasClassicStructure(FVertexFlags) then ErrorTest(D3DERR_INVALIDCALL, @errBadVertexFormat, GetErrorMsgD3D, True);
  VF := VertexFormatHasTextureStructure(FVertexFlags);
  for v := 0 to FMapHeight - 1 do
    for u := 0 to FMapWidth - 1 do
      begin
      NCount := 0;
      P := Vertex[u, v];
      P.Normal := D3DXVector3Zero;
      //gouraudnormale = kreuzprodukte aus allen angrenzenden flchen (8 stk), je normalisieren und anschlieend mitteln
      if u > 0 then
        begin
        if v > 0 then begin Cross(Vertex[u,v-1],Vertex[u-1,v-1]); Cross(Vertex[u-1,v-1],Vertex[u-1,v]); end;
        if v < FMapHeight - 1 then begin Cross(Vertex[u-1,v],Vertex[u-1,v+1]); Cross(Vertex[u-1,v+1],Vertex[u,v+1]); end;
        end;
      if u < FMapWidth - 1 then
        begin
        if v > 0 then begin Cross(Vertex[u+1,v],Vertex[u+1,v-1]); Cross(Vertex[u+1,v-1],Vertex[u,v-1]); end;
        if v < FMapHeight - 1 then begin Cross(Vertex[u,v+1],Vertex[u+1,v+1]); Cross(Vertex[u+1,v+1],Vertex[u+1,v]); end;
        end;
      if NCount > 0 then D3DXVec3Scale(P.Normal, P.Normal, 1/NCount);
      if VF > 0 then
          begin
          PxorDXVertexTexture(P).TexCoordinates := GetCoord(u / (FMapWidth - 1), v / (FMapHeight - 1));
          if 2 <= VF then PxorDXVertexTexture2(P).TexCoordinates2 := GetCoord(u / (FMapWidth - 1), v / (FMapHeight - 1));
          end;
      end;
end;

{ TxorDXGridMap }

constructor TxorDXGridMap.Create(Scene: TxorDXScene; MapWidth, MapHeight: Cardinal);
begin
  CreateParam(Scene, MapWidth, MapHeight, xorDXVertexSizeClassic, xorDXVertexFlagClassic);
end;

constructor TxorDXGridMap.CreateParam(Scene: TxorDXScene; Width, Height: Cardinal; VertexSize: TxorDXVertexSize; VertexFlags: TxorDXVertexFlag);
begin
  if (Width = 0) or (Height = 0) then ErrorTest(E_INVALIDARG, @errBadArgument, GetErrorMsgD3D, True);
  inherited CreateParam(Scene, Width * Height, GetEntryCount(xorDXPrimLineList, 3 * Width * Height - 2 * (Width + Height)), VertexSize, VertexFlags);
  FMapWidth := Width;
  FMapHeight := Height;
  SetDrawMode(xorDXPrimLineList);
  SetIndices;
end;

procedure TxorDXGridMap.SetIndices;
var
  u, v, w, base: Cardinal;
begin
  w := 0;
  if (FMapHeight < 2) or (FMapWidth < 2) then Exit;
  for v := 0 to FMapHeight - 1 do
    begin
    for u := 0 to FMapWidth - 2 do
      begin
      base := v * FMapWidth + u;
      if FScene.FUse32BitIndex then Line32[w]^ := GetLine32(base, base + 1) else Line[w]^ := GetLine(base, base + 1);
      Inc(w);
      end;
    if v < FMapHeight - 1 then
      for u := 0 to FMapWidth - 2 do
        begin
        base := v * FMapWidth + u;
        if FScene.FUse32BitIndex then Line32[w]^ := GetLine32(base, base + FMapWidth) else Line[w]^ := GetLine(base, base + FMapWidth);
        Inc(w);
        end;
    end;
  for v := 0 to FMapHeight - 2 do
    for u := 0 to FMapWidth - 2 do
      begin
      base := v * FMapWidth + u;
      if FScene.FUse32BitIndex then Line32[w]^ := GetLine32(base + 1, base + FMapWidth) else Line[w]^ := GetLine(base + 1, base + FMapWidth);
      Inc(w);
      end;
end;

procedure TxorDXGridMap.SetStandard;
var
  u, v: Cardinal;
  x, y, z, cosv, ru, rv, mind : Single;
begin
  if (FMapHeight < 1) or (FMapWidth < 1) then Exit;
  if not VertexFormatHasClassicStructure(FVertexFlags) then ErrorTest(D3DERR_INVALIDCALL, @errBadVertexFormat, GetErrorMsgD3D, True);
  SetDrawMode(xorDXPrimLineList);
  SetDrawAll(True);
  for v := 0 to FMapHeight - 1 do
    for u := 0 to FMapWidth - 1 do
      begin
      ru := Pi2 * u / (FMapWidth - 1);
      rv := Pi2 * v / (FMapHeight - 1);
      cosv := cos(rv);
      mind := Min(FMapHeight, FMapWidth);
      x := 1.0 * FMapWidth * cos(ru) * cosv / mind;
      y := 1.0 * FMapHeight * sin(ru) * cosv / mind;
      z := 1.0 * sin(rv);
      PxorDXVertexClassic(Vertex[u, v])^ := GetVertexClassic(x, z, y,  0, 0, 0, 30, 70 + 30 * Trunc(ru), 70 + 30 * Trunc(rv), 0.7);
      end;
  SetIndices;
  InitNormalAndTextureVectors;
end;

{ TxorDXMap }

constructor TxorDXMap.Create(Scene: TxorDXScene; MapWidth, MapHeight: Cardinal);
begin
  CreateParam(Scene, MapWidth, MapHeight, xorDXVertexSizeClassic, xorDXVertexFlagClassic);
end;

constructor TxorDXMap.CreateParam(Scene: TxorDXScene; MapWidth, MapHeight: Cardinal; VertexSize: TxorDXVertexSize; VertexFlags: TxorDXVertexFlag);
begin
  if (MapWidth = 0) or (MapHeight = 0) then ErrorTest(E_INVALIDARG, @errBadArgument, GetErrorMsgD3D, True);
  inherited CreateParam(Scene, MapWidth * MapHeight, GetEntryCount(xorDXPrimTriangleList, 2 * (MapWidth - 1) * (MapHeight - 1)), VertexSize, VertexFlags);
  FMapWidth := MapWidth;
  FMapHeight := MapHeight;
  SetDrawMode(xorDXPrimTriangleList);
  SetIndices;
end;

procedure TxorDXMap.SetIndices;
var
  u, v, z, base: Cardinal;
begin
  if (FMapHeight < 2) or (FMapWidth < 2) then Exit;
  z := 0;
  if FScene.FUse32BitIndex then
    for v := 0 to FMapHeight - 2 do
      for u := 0 to FMapWidth - 2 do
        begin
        base := v * FMapWidth + u;
        Triangle32[z]^ := GetTriangle32(base + 1, base, base + FMapWidth);
        inc(z);
        Triangle32[z]^ := GetTriangle32(base + 1, base + FMapWidth, base + FMapWidth + 1);
        inc(z);
        end
  else
    for v := 0 to FMapHeight - 2 do
      for u := 0 to FMapWidth - 2 do
        begin
        base := v * FMapWidth + u;
        Triangle[z]^ := GetTriangle(base + 1, base, base + FMapWidth);
        inc(z);
        Triangle[z]^ := GetTriangle(base + 1, base + FMapWidth, base + FMapWidth + 1);
        inc(z);
        end;
end;

procedure TxorDXMap.SetStandard;
var
  u, v, mind: Cardinal;
  x, y, r, sinr2: Single;
begin
  if (FMapHeight < 1) or (FMapWidth < 1) then Exit;
  if not VertexFormatHasClassicStructure(FVertexFlags) then ErrorTest(D3DERR_INVALIDCALL, @errBadVertexFormat, GetErrorMsgD3D, True);
  SetDrawMode(xorDXPrimTriangleList);
  SetDrawAll(True);
  mind := Min(FMapHeight, FMapWidth);
  for v := 0 to FMapHeight - 1 do
    for u := 0 to FMapWidth - 1 do
      begin
      x := 3 * (u / (mind - 1) - 0.5);
      y := 3 * (v / (mind - 1) - 0.5);
      r := Pi2 * sqrt((x * x + y * y)) + RotationsPerSecond(0.5);
      sinr2 := sin(r) / 2;
      PxorDXVertexClassic(Vertex[u, v])^ := GetVertexClassic(x, sinr2, y,  0, 0, 0, $B0, Trunc(Abs(x * $90) ), 0, 0.7)
      end;
  SetIndices;
  InitNormalAndTextureVectors;
end;

{ TxorDXRect }

constructor TxorDXRect.Create(Scene: TxorDXScene; MapWidth, MapHeight: Cardinal; RectWidth, RectHeight: Single);
begin
  CreateParam(Scene, MapWidth, MapHeight, RectWidth, RectHeight, xorDXVertexSizeClassic, xorDXVertexFlagClassic);
end;

constructor TxorDXRect.CreateParam(Scene: TxorDXScene; MapWidth, MapHeight: Cardinal; RectWidth, RectHeight: Single; VertexSize: TxorDXVertexSize; VertexFlags: TxorDXVertexFlag);
begin
  inherited CreateParam(Scene, MapWidth, MapHeight, VertexSize, VertexFlags);
  FRectWidth := RectWidth;
  FRectHeight := RectHeight;
  SetStandard;
end;

procedure TxorDXRect.SetStandard;
var
  u, v: Cardinal;
  x, y, px, py: Single;
begin
  if (FMapHeight < 1) or (FMapWidth < 1) then Exit;
  if not VertexFormatHasClassicStructure(FVertexFlags) then ErrorTest(D3DERR_INVALIDCALL, @errBadVertexFormat, GetErrorMsgD3D, True);
  SetDrawMode(xorDXPrimTriangleList);
  SetDrawAll(True);
  for v := 0 to FMapHeight - 1 do
    for u := 0 to FMapWidth - 1 do
      begin
      px := u / (FMapWidth - 1);
      py := v / (FMapHeight - 1);
      x := FRectWidth * px;
      y := FRectHeight * py;
      PxorDXVertexClassic(Vertex[u, v])^ := GetVertexClassic(x, y, 0, 0, 0, 0, $FF - Round($D0 * px), $FF - Round($D0 * py), Round($FF * Abs(px - py)), 0.7);
      end;
  SetIndices;
  InitNormalAndTextureVectors;
end;

initialization

Pi2 := 2 * Pi;
Pi05 := Pi / 2;
TickStart := GetTickCount;

end.

//DONE 5: lostdev bei maindev, und auch reset und chmod
//DONE 5: weitere shapes

